=== modified file '.bzrignore'
--- .bzrignore	2008-09-06 16:33:54 +0000
+++ .bzrignore	2008-10-03 09:32:30 +0000
@@ -1,8 +1,14 @@
 *.5
 *.8
 *.8mandos
+confdir
+debian/po/messages.mo
+debian/po/templates.pot
+keydir
+man
 plugin-runner
+plugins.d/askpass-fifo
+plugins.d/mandos-client
 plugins.d/password-prompt
-plugins.d/mandos-client
-confdir
-keydir
+plugins.d/splashy
+plugins.d/usplash

=== modified file 'INSTALL'
--- INSTALL	2008-09-08 18:54:47 +0000
+++ INSTALL	2009-01-27 00:07:26 +0000
@@ -43,13 +43,14 @@
     + Python-GnuTLS 1.1.5 http://pypi.python.org/pypi/python-gnutls/
     + dbus-python 0.82.4  http://dbus.freedesktop.org/doc/dbus-python/
     + python-ctypes 1.0.0 http://pypi.python.org/pypi/ctypes
+    + PyGObject 2.14.2    http://library.gnome.org/devel/pygobject/
     
     Strongly recommended:
     + fping 2.4b2-to-ipv6 http://www.fping.com/
     
     Package names:
     python-gnutls avahi-daemon python python-avahi python-dbus
-    python-ctypes
+    python-ctypes python-gobject
    
 *** Mandos Client
     + initramfs-tools 0.85i
@@ -97,10 +98,28 @@
      and append this to the file "/etc/mandos/clients.conf" *on the
      server computer*.
   
-  4. On the server computer, start the server by running the command
+  4. Configure the client to use the correct network interface.  The
+     default is "eth0", and if this needs to be adjusted, it will be
+     necessary to edit /etc/mandos/plugin-runner.conf to uncomment and
+     change the line there.  If that file is changed, the initrd.img
+     file must be updated, possibly using the following command:
+     
+	# update-initramfs -k all -u
+  
+  5. On the server computer, start the server by running the command
      For Debian: su -c 'invoke-rc.d mandos start'
      For Ubuntu: sudo invoke-rc.d mandos start
      
+     At this point, it is possible to verify that the correct password
+     will be received by the client by running the command:
+     
+	# /usr/lib/mandos/plugins.d/mandos-client \
+		--pubkey=/etc/keys/mandos/pubkey.txt \
+		--seckey=/etc/keys/mandos/seckey.txt; echo
+     
+     This command should retrieve the password from the server,
+     decrypt it, and output it to standard output.
+     
      After this, the client computer should be able to reboot without
      needing a password entered on the console, as long as it does not
      take more than an hour to reboot.
@@ -109,6 +128,6 @@
   
   You may want to tighten or loosen the timeouts in the server
   configuration files; see mandos.conf(5) and mandos-clients.conf(5).
-  Is IPsec is not used, it is suggested that a more cryptographically
+  If IPsec is not used, it is suggested that a more cryptographically
   secure checker program is used and configured, since without IPsec
   ping packets can be faked.

=== modified file 'Makefile'
--- Makefile	2008-09-17 00:34:09 +0000
+++ Makefile	2009-01-17 02:12:05 +0000
@@ -9,10 +9,15 @@
 #DEBUG=-ggdb3
 # For info about _FORTIFY_SOURCE, see
 # <http://gcc.gnu.org/ml/gcc-patches/2004-09/msg02055.html>
-FORTIFY=-D_FORTIFY_SOURCE=2 # -fstack-protector-all
+FORTIFY=-D_FORTIFY_SOURCE=2 -fstack-protector-all -fPIC -fPIE
+LINK_FORTIFY_LD=-z relro -fPIE
+LINK_FORTIFY=-pie
 #COVERAGE=--coverage
 OPTIMIZE=-Os
 LANGUAGE=-std=gnu99
+htmldir=man
+version=1.0.5
+SED=sed
 
 ## Use these settings for a traditional /usr/local install
 # PREFIX=$(DESTDIR)/usr/local
@@ -39,8 +44,9 @@
 
 # Do not change these two
 CFLAGS=$(WARN) $(DEBUG) $(FORTIFY) $(COVERAGE) $(OPTIMIZE) \
-	$(LANGUAGE) $(GNUTLS_CFLAGS) $(AVAHI_CFLAGS) $(GPGME_CFLAGS)
-LDFLAGS=$(COVERAGE)
+	$(LANGUAGE) $(GNUTLS_CFLAGS) $(AVAHI_CFLAGS) $(GPGME_CFLAGS) \
+	-DVERSION='"$(version)"'
+LDFLAGS=$(COVERAGE) $(LINK_FORTIFY) $(foreach flag,$(LINK_FORTIFY_LD),-Xlinker $(flag))
 
 # Commands to format a DocBook <refentry> document into a manual page
 DOCBOOKTOMAN=cd $(dir $<); xsltproc --nonet --xinclude \
@@ -53,71 +59,152 @@
 	$(notdir $<); \
 	$(MANPOST) $(notdir $@)
 # DocBook-to-man post-processing to fix a '\n' escape bug
-MANPOST=sed --in-place --expression='s,\\\\en,\\en,g;s,\\n,\\en,g'
-
-PLUGINS=plugins.d/password-prompt plugins.d/mandos-client
-PROGS=plugin-runner $(PLUGINS)
+MANPOST=$(SED) --in-place --expression='s,\\\\en,\\en,g;s,\\n,\\en,g'
+
+DOCBOOKTOHTML=xsltproc --nonet --xinclude \
+	--param make.year.ranges		1 \
+	--param make.single.year.ranges		1 \
+	--param man.output.quietly		1 \
+	--param man.authors.section.enabled	0 \
+	--param citerefentry.link		1 \
+	--output $@ \
+	/usr/share/xml/docbook/stylesheet/nwalsh/xhtml/docbook.xsl \
+	$<; $(HTMLPOST) $@
+# Fix citerefentry links
+HTMLPOST=$(SED) --in-place \
+	--expression='s/\(<a class="citerefentry" href="\)\("><span class="citerefentry"><span class="refentrytitle">\)\([^<]*\)\(<\/span>(\)\([^)]*\)\()<\/span><\/a>\)/\1\3.\5\2\3\4\5\6/g'
+
+PLUGINS=plugins.d/password-prompt plugins.d/mandos-client \
+	plugins.d/usplash plugins.d/splashy plugins.d/askpass-fifo
+CPROGS=plugin-runner $(PLUGINS)
+PROGS=mandos mandos-keygen mandos-ctl $(CPROGS)
 DOCS=mandos.8 plugin-runner.8mandos mandos-keygen.8 \
 	plugins.d/mandos-client.8mandos \
 	plugins.d/password-prompt.8mandos mandos.conf.5 \
-	mandos-clients.conf.5
-
-objects=$(addsuffix .o,$(PROGS))
-
-all: $(PROGS)
+	plugins.d/usplash.8mandos plugins.d/splashy.8mandos \
+	plugins.d/askpass-fifo.8mandos mandos-clients.conf.5
+
+htmldocs=$(addsuffix .xhtml,$(DOCS))
+
+objects=$(addsuffix .o,$(CPROGS))
+
+all: $(PROGS) mandos.lsm
 
 doc: $(DOCS)
 
-%.5: %.xml legalnotice.xml
-	$(DOCBOOKTOMAN)
-
-%.8: %.xml legalnotice.xml
-	$(DOCBOOKTOMAN)
-
-%.8mandos: %.xml legalnotice.xml
-	$(DOCBOOKTOMAN)
-
-mandos.8: mandos.xml mandos-options.xml overview.xml legalnotice.xml
-	$(DOCBOOKTOMAN)
-
-mandos-keygen.8: mandos-keygen.xml overview.xml legalnotice.xml
-	$(DOCBOOKTOMAN)
-
-mandos.conf.5: mandos.conf.xml mandos-options.xml legalnotice.xml
-	$(DOCBOOKTOMAN)
-
-plugin-runner.8mandos: plugin-runner.xml overview.xml legalnotice.xml
-	$(DOCBOOKTOMAN)
+html: $(htmldocs)
+
+%.5: %.xml common.ent legalnotice.xml
+	$(DOCBOOKTOMAN)
+%.5.xhtml: %.xml common.ent legalnotice.xml
+	$(DOCBOOKTOHTML)
+
+%.8: %.xml common.ent legalnotice.xml
+	$(DOCBOOKTOMAN)
+%.8.xhtml: %.xml common.ent legalnotice.xml
+	$(DOCBOOKTOHTML)
+
+%.8mandos: %.xml common.ent legalnotice.xml
+	$(DOCBOOKTOMAN)
+%.8mandos.xhtml: %.xml common.ent legalnotice.xml
+	$(DOCBOOKTOHTML)
+
+mandos.8: mandos.xml common.ent mandos-options.xml overview.xml \
+		legalnotice.xml
+	$(DOCBOOKTOMAN)
+mandos.8.xhtml: mandos.xml common.ent mandos-options.xml \
+		overview.xml legalnotice.xml
+	$(DOCBOOKTOHTML)
+
+mandos-keygen.8: mandos-keygen.xml common.ent overview.xml \
+		legalnotice.xml
+	$(DOCBOOKTOMAN)
+mandos-keygen.8.xhtml: mandos-keygen.xml common.ent overview.xml \
+		 legalnotice.xml
+	$(DOCBOOKTOHTML)
+
+mandos.conf.5: mandos.conf.xml common.ent mandos-options.xml \
+		legalnotice.xml
+	$(DOCBOOKTOMAN)
+mandos.conf.5.xhtml: mandos.conf.xml common.ent mandos-options.xml \
+		legalnotice.xml
+	$(DOCBOOKTOHTML)
+
+plugin-runner.8mandos: plugin-runner.xml common.ent overview.xml \
+		legalnotice.xml
+	$(DOCBOOKTOMAN)
+plugin-runner.8mandos.xhtml: plugin-runner.xml common.ent \
+		overview.xml legalnotice.xml
+	$(DOCBOOKTOHTML)
 
 plugins.d/mandos-client.8mandos: plugins.d/mandos-client.xml \
+					common.ent \
 					mandos-options.xml \
 					overview.xml legalnotice.xml
 	$(DOCBOOKTOMAN)
+plugins.d/mandos-client.8mandos.xhtml: plugins.d/mandos-client.xml \
+					common.ent \
+					mandos-options.xml \
+					overview.xml legalnotice.xml
+	$(DOCBOOKTOHTML)
+
+# Update all these files with version number $(version)
+common.ent: Makefile
+	$(SED) --in-place \
+		--expression='s/^\(<ENTITY VERSION "\)[^"]*">$$/\1$(version)"/' \
+		$@
+
+mandos: Makefile
+	$(SED) --in-place \
+		--expression='s/^\(version = "\)[^"]*"$$/\1$(version)"/' \
+		$@
+
+mandos-keygen: Makefile
+	$(SED) --in-place \
+		--expression='s/^\(VERSION="\)[^"]*"$$/\1$(version)"/' \
+		$@
+
+mandos-ctl: Makefile
+	$(SED) --in-place \
+		--expression='s/^\(version = "\)[^"]*"$$/\1$(version)"/' \
+		$@
+
+mandos.lsm: Makefile
+	$(SED) --in-place \
+		--expression='s/^\(Version:\).*/\1\t$(version)/' \
+		$@
+	$(SED) --in-place \
+		--expression='s/^\(Entered-date:\).*/\1\t$(shell date --rfc-3339=date --reference=Makefile)/' \
+		$@
+	$(SED) --in-place \
+		--expression='s/\(mandos_\)[0-9.]\+\(\.orig\.tar\.gz\)/\1$(version)\2/' \
+		$@
 
 plugins.d/mandos-client: plugins.d/mandos-client.o
 	$(LINK.o) $(GNUTLS_LIBS) $(AVAHI_LIBS) $(GPGME_LIBS) \
 		$(COMMON) $^ $(LOADLIBES) $(LDLIBS) -o $@
 
-.PHONY : all doc clean distclean run-client run-server install \
+.PHONY : all doc html clean distclean run-client run-server install \
 	install-server install-client uninstall uninstall-server \
 	uninstall-client purge purge-server purge-client
 
 clean:
-	-rm --force $(PROGS) $(objects) $(DOCS) core
+	-rm --force $(CPROGS) $(objects) $(htmldocs) $(DOCS) core
 
 distclean: clean
 mostlyclean: clean
 maintainer-clean: clean
 	-rm --force --recursive keydir confdir
 
-check:
+check:	all
 	./mandos --check
 
 # Run the client with a local config and key
 run-client: all keydir/seckey.txt keydir/pubkey.txt
 	./plugin-runner --plugin-dir=plugins.d \
 		--config-file=plugin-runner.conf \
-		--options-for=mandos-client:--seckey=keydir/seckey.txt,--pubkey=keydir/pubkey.txt
+		--options-for=mandos-client:--seckey=keydir/seckey.txt,--pubkey=keydir/pubkey.txt \
+		$(CLIENTARGS)
 
 # Used by run-client
 keydir/seckey.txt keydir/pubkey.txt: mandos-keygen
@@ -126,7 +213,7 @@
 
 # Run the server with a local config
 run-server: confdir/mandos.conf confdir/clients.conf
-	./mandos --debug --configdir=confdir
+	./mandos --debug --configdir=confdir $(SERVERARGS)
 
 # Used by run-server
 confdir/mandos.conf: mandos.conf
@@ -140,6 +227,11 @@
 
 install: install-server install-client-nokey
 
+install-html: html
+	install --directory $(htmldir)
+	install --mode=u=rw,go=r --target-directory=$(htmldir) \
+		$(htmldocs)
+
 install-server: doc
 	install --directory $(CONFDIR)
 	install --mode=u=rwx,go=rx mandos $(PREFIX)/sbin/mandos
@@ -151,7 +243,9 @@
 		$(DESTDIR)/etc/init.d/mandos
 	install --mode=u=rw,go=r default-mandos \
 		$(DESTDIR)/etc/default/mandos
-	update-rc.d mandos defaults
+	if [ -z $(DESTDIR) ]; then \
+		update-rc.d mandos defaults 25 15;\
+	fi
 	gzip --best --to-stdout mandos.8 \
 		> $(MANDIR)/man8/mandos.8.gz
 	gzip --best --to-stdout mandos.conf.5 \
@@ -165,9 +259,7 @@
 		$(PREFIX)/lib/mandos/plugins.d
 	if [ "$(CONFDIR)" != "$(PREFIX)/lib/mandos" ]; then \
 		install --mode=u=rwx \
-			--directory "$(CONFDIR)/plugins.d" && \
-		install --mode=u=rw,go=r etc-plugins.d-README \
-			$(CONFDIR)/plugins.d/README ; \
+			--directory "$(CONFDIR)/plugins.d"; \
 	fi
 	install --mode=u=rwx,go=rx \
 		--target-directory=$(PREFIX)/lib/mandos plugin-runner
@@ -179,9 +271,15 @@
 	install --mode=u=rwxs,go=rx \
 		--target-directory=$(PREFIX)/lib/mandos/plugins.d \
 		plugins.d/mandos-client
-	install --mode=u=rwx,go=rx \
+	install --mode=u=rwxs,go=rx \
 		--target-directory=$(PREFIX)/lib/mandos/plugins.d \
 		plugins.d/usplash
+	install --mode=u=rwxs,go=rx \
+		--target-directory=$(PREFIX)/lib/mandos/plugins.d \
+		plugins.d/splashy
+	install --mode=u=rwxs,go=rx \
+		--target-directory=$(PREFIX)/lib/mandos/plugins.d \
+		plugins.d/askpass-fifo
 	install initramfs-tools-hook \
 		$(INITRAMFSTOOLS)/hooks/mandos
 	install --mode=u=rw,go=r initramfs-tools-hook-conf \
@@ -197,6 +295,12 @@
 		> $(MANDIR)/man8/password-prompt.8mandos.gz
 	gzip --best --to-stdout plugins.d/mandos-client.8mandos \
 		> $(MANDIR)/man8/mandos-client.8mandos.gz
+	gzip --best --to-stdout plugins.d/usplash.8mandos \
+		> $(MANDIR)/man8/usplash.8mandos.gz
+	gzip --best --to-stdout plugins.d/splashy.8mandos \
+		> $(MANDIR)/man8/splashy.8mandos.gz
+	gzip --best --to-stdout plugins.d/askpass-fifo.8mandos \
+		> $(MANDIR)/man8/askpass-fifo.8mandos.gz
 
 install-client: install-client-nokey
 # Post-installation stuff
@@ -224,16 +328,18 @@
 		$(PREFIX)/lib/mandos/plugins.d/password-prompt \
 		$(PREFIX)/lib/mandos/plugins.d/mandos-client \
 		$(PREFIX)/lib/mandos/plugins.d/usplash \
+		$(PREFIX)/lib/mandos/plugins.d/splashy \
+		$(PREFIX)/lib/mandos/plugins.d/askpass-fifo \
 		$(INITRAMFSTOOLS)/hooks/mandos \
 		$(INITRAMFSTOOLS)/conf-hooks.d/mandos \
 		$(INITRAMFSTOOLS)/scripts/local-top/mandos \
 		$(MANDIR)/man8/plugin-runner.8mandos.gz \
 		$(MANDIR)/man8/mandos-keygen.8.gz \
 		$(MANDIR)/man8/password-prompt.8mandos.gz \
+		$(MANDIR)/man8/usplash.8mandos.gz \
+		$(MANDIR)/man8/splashy.8mandos.gz \
+		$(MANDIR)/man8/askpass-fifo.8mandos.gz \
 		$(MANDIR)/man8/mandos-client.8mandos.gz
-	if [ "$(CONFDIR)" != "$(PREFIX)/lib/mandos" ]; then \
-		rm --force $(CONFDIR)/plugins.d/README; \
-	fi
 	-rmdir $(PREFIX)/lib/mandos/plugins.d $(CONFDIR)/plugins.d \
 		 $(PREFIX)/lib/mandos $(CONFDIR) $(KEYDIR)
 	update-initramfs -k all -u

=== added file 'NEWS'
--- NEWS	1970-01-01 00:00:00 +0000
+++ NEWS	2009-01-17 02:12:05 +0000
@@ -0,0 +1,51 @@
+This NEWS file records noteworthy changes, very tersely.
+See the manual for detailed information.
+
+Version 1.0.5 (2009-01-17)
+* Client
+** Fix small memory leak in plugin-runner.
+
+Version 1.0.4 (2009-01-15)
+* Server
+** Only find matched user/group pairs when searching for suitable
+   nonprivileged user/group to switch to.
+
+* Client
+** New kernel parameter "mandos=off" makes client not run at boot.
+** Fix linking errors and compilation warnings on AMD64.
+** Parse numbers in command line options better.
+** The splashy and usplash plugins are more robust while traversing
+   /proc, and will not abort if a process suddenly disappears.
+
+Version 1.0.3 (2009-01-06)
+* Server
+** Now tries to change to user and group "_mandos" before falling back
+   to trying the old values "mandos", "nobody:nogroup", and "65534".
+** Now does not abort on startup even if no clients are defined in
+   clients.conf.
+
+* Client
+** Plugins named "*.dpkg-bak" are now ignored.
+** Hopefully fixed compilation failure on some architectures where the
+   C compiler does not recognize the "-z" option as a linker option.
+
+Version 1.0.2 (2008-10-17)
+* mandos-keygen now signs the encrypted key blobs.  This signature is
+  not currently verified by mandos-client, but this may change in the
+  future.
+
+Version 1.0.1 (2008-10-07)
+* Server
+** Expand environment variables and ~user in clients.conf's "secfile"
+   The "secfile" option in /etc/mandos/clients.conf now expands
+   "~user/foo" and "$ENVVAR" strings.
+
+* Client (plugin-runner, plugins, etc.)
+** Manual pages for the usplash, splashy, and askpass-fifo plugins.
+   All plugins now have man pages.
+** More secure compilation and linking flags.
+   All programs are now compiled with "-fstack-protector-all -fPIE
+   -pie", and linked using "-z relro -pie" for additional security.
+
+* There is now a "NEWS" file (this one), giving a history of
+  noteworthy changes.

=== modified file 'README'
--- README	2008-09-17 00:34:09 +0000
+++ README	2009-01-12 22:02:33 +0000
@@ -130,10 +130,36 @@
   on your door and the sudden absence of all the servers in your
   server room.  Which it does nicely.
 
+* The Plugin System
+  In the early designs, the mandos-client(8mandos) program (which
+  retrieves a password from the Mandos server) also prompted for a
+  password on the terminal, in case a Mandos server could not be
+  found.  This duality of purpose was seen to be too complex to be a
+  viable way to continue.  Instead, the programs are now separated
+  into mandos-client(8mandos) and password-prompt(8mandos), and a
+  plugin-runner(8mandos) exist to run them both in parallel, allowing
+  the first plugin to succeed to provide the password.  This opened up
+  for any number of additional plugins to run, all competing to be the
+  first to find a password and provide it to the plugin runner.
+  
+  Three additional plugins are provided:
+  * usplash(8mandos)
+    This prompts for a password when using usplash(8).
+  * splashy(8mandos)
+    This prompts for a password when using splashy(8).
+  * askpass-fifo(8mandos)
+    To provide compatibility with the "askpass" program from
+    cryptsetup, this plugin listens to the same FIFO as askpass would
+    do.
+  
+  More plugins could easily be written and added by the system
+  administrator; see the section called "WRITING PLUGINS" in
+  plugin-runner(8mandos) to learn the plugin requirements.
+
 * Copyright
 
-    Copyright © 2008 Teddy Hogeborn
-                2008 Björn Påhlsson
+    Copyright © 2008,2009 Teddy Hogeborn
+    Copyright © 2008,2009 Björn Påhlsson
   
 ** License:
    

=== modified file 'TODO'
--- TODO	2008-09-19 00:00:51 +0000
+++ TODO	2009-01-23 20:09:55 +0000
@@ -1,15 +1,14 @@
 -*- org -*-
 
-* plugin-runner
-** TODO Man page  for plugin-runner.conf.5
-   link to plugin-runner.8
-
 * mandos-client
+** TODO [#A] Clean up /tmp directory on signal
 ** TODO [#B] Temporarily lower kernel log level
    for less printouts during sucessfull boot.
+   klogctl(6, NULL, 0); klogctl(7, NULL, 0);
 ** TODO [#C] IPv4 support
 
-* DONE password-prompt
+* plugin-runner
+** TODO [#B] use scandir(3) instead of readdir(3)
 
 * mandos (server)
 ** TODO [#B] Log level				:bugs:
@@ -19,6 +18,12 @@
 ** TODO [#B] Run-time communication with server :bugs:
    Probably using D-Bus
    See also [[*Mandos-tools]]
+*** Client class
+*** Main server
+    + SetLogLevel
+      syslogger.setLevel(logging.WARNING)
+    + Quit
+    + [[http://log.ometer.com/2007-05.html][Best D-Bus practices]]
 ** TODO Implement --foreground			:bugs:
    [[info:standards:Option%20Table][Table of Long Options]]
 ** TODO Implement --socket
@@ -26,58 +31,36 @@
 ** TODO Date+time on console log messages	:bugs:
    Is this the default?
 ** TODO delete hook when clients fall out by timeout
-
-* Mandos-tools/utilities
-  All of this probably using D-Bus
-** TODO List clients
+   This will not be strictly necessary when the D-Bus interface is
+   implemented.
+
+* mandos.xml
+** [[file:mandos.xml::XXX][Document D-Bus interface]]
+
+* Provide and install /etc/dbus-1/system.d/mandos.conf
+
+* mandos-list
+*** Handle "no D-Bus server" and/or "no Mandos server found" better
+*** [#B] --dump option
 ** TODO Disable client
 ** TODO Enable client
 ** TODO Reset timer
 
-* Man pages
-** TODO Use xinclude for all common sections
-   Like authors, etc.
-
-
-* Installer
-** Client-side
-*** TODO Update initrd.img after installation
-    This seems to use some kind of "trigger" system
-    [[file:/usr/share/doc/dpkg/triggers.txt.gz]]
-    dpkg-trigger(1), deb-triggers(5)
-*** mandos-keygen
-**** TODO [#A] Ask for password twice for confirmation
-**** TODO "--passfile" option
-     Using the "secfile" option instead of "secret"
-**** TODO [#B] "--test" option
-     For testing decryption before rebooting.
-** Server-side
-*** TODO [#A] Create mandos user and group for server
-
-
-* [#A] Package
+* Curses interface
+
+* mandos-keygen
+** TODO "--secfile" option
+   Using the "secfile" option instead of "secret"
+** TODO [#B] "--test" option
+   For testing decryption before rebooting.
+
+* Package
 ** /usr/share/initramfs-tools/hooks/mandos
-*** TODO Do not install in initrd.img if configured not to.
+*** TODO [#C] Do not install in initrd.img if configured not to.
     Use "/etc/initramfs-tools/conf.d/mandos"?  Definitely a debconf
     question.
-** TODO /etc/bash_completion.d/mandos
+** TODO [#C] /etc/bash_completion.d/mandos
    From XML sources directly?
-** TODO unperish
-** DONE bzr-builddeb
-** TODO mandos user/group creation
-** TODO Key creation in postinst
-
-* TODO Web site
-** DONE http://www.fukt.bsnet.se/mandos
-   Redirects to the wiki page
-** TODO http://wiki.fukt.bsnet.se/wiki/Mandos
-
-* Mailing list
-** DONE mandos-dev
-*** TODO http://gmane.org/subscribe.php
-
-* TODO Announce project on Usenet
-  [[news:comp.os.linux.announce]]
 
 
 #+STARTUP: showall

=== modified file 'clients.conf'
--- clients.conf	2008-08-27 01:18:25 +0000
+++ clients.conf	2009-01-08 03:54:06 +0000
@@ -14,7 +14,7 @@
 ;interval = 5m
 
 # What command to run as "the checker".
-;checker = fping -q -- %(host)s
+;checker = fping -q -- %%(host)s
 
 
 ;####
@@ -55,7 +55,7 @@
 ;fingerprint = 3e393aeaefb84c7e89e2f547b3a107558fca3a27
 ;
 ;# If "secret" is not specified, a file can be read for the data.
-;;secfile = /etc/mandos/bar-secret.txt.asc
+;secfile = /etc/mandos/bar-secret.bin
 ;
 ;# An IP address for host is also fine, if the checker accepts it.
 ;host = 192.0.2.3

=== added file 'common.ent'
--- common.ent	1970-01-01 00:00:00 +0000
+++ common.ent	2008-09-30 07:23:39 +0000
@@ -0,0 +1,3 @@
+<?xml version="1.0" encoding="UTF-8"?>
+<!-- Edited by Makefile -->
+<!ENTITY version "1.0">

=== modified file 'debian/changelog'
--- debian/changelog	2008-09-17 00:34:09 +0000
+++ debian/changelog	2009-01-17 02:12:05 +0000
@@ -1,5 +1,67 @@
-mandos (1.0) unstable; urgency=low
-
-  * Initial Release.
-
- -- Mandos Maintainers <mandos@fukt.bsnet.se>  Sun, 07 Sep 2008 11:55:51 +0200
+mandos (1.0.5-1) unstable; urgency=low
+
+  * New upstream release.
+
+ -- Teddy Hogeborn <teddy@fukt.bsnet.se>  Sat, 17 Jan 2009 02:26:00 +0100
+
+mandos (1.0.4-1) unstable; urgency=low
+
+  * New upstream release.
+  * debian/watch: New file.
+  * debian/mandos-client.README.Debian: Document new "mandos=off" kernel
+    parameter.
+
+ -- Teddy Hogeborn <teddy@fukt.bsnet.se>  Thu, 15 Jan 2009 05:49:22 +0100
+
+mandos (1.0.3-2) unstable; urgency=low
+
+  * Removed some now-unused debconf files.
+  * Changed postinst scripts to not source debconf/confmodule.
+  * Removed po-debconf from build-depends.
+
+ -- Teddy Hogeborn <teddy@fukt.bsnet.se>  Tue, 06 Jan 2009 21:28:20 +0100
+
+mandos (1.0.3-1) unstable; urgency=low
+
+  * New upstream release.
+  * Add -Xlinker to linker flags to fix FTBFS for some architectures.
+    Thanks to Thiemo Seufer <ths@networkno.de> for the report and
+    fix. (Closes:  #509398)
+  * Remove debconf use altogether, thereby stopping debconf abuse.  Thanks
+    to Christian Perrier <bubulle@debian.org>. (Closes:  #509653)
+  * Add NEWS file to /usr/share/doc directories.
+  * Use and create "_mandos" user+group.  Rename old user+group created by
+    older versions of this package.
+  * Fix manual pages by adding build-depend on "docbook-xml".
+
+ -- Teddy Hogeborn <teddy@fukt.bsnet.se>  Tue, 06 Jan 2009 01:21:20 +0100
+
+mandos (1.0.2-1) unstable; urgency=low
+
+  * New upstream release.
+  * debian/copyright: Rewritten to conform to
+    <http://wiki.debian.org/Proposals/CopyrightFormat?action=recall&rev=233>.
+
+ -- Teddy Hogeborn <teddy@fukt.bsnet.se>  Fri, 17 Oct 2008 20:42:12 +0200
+
+mandos (1.0.1-1) unstable; urgency=low
+
+  * New upstream release.
+  * Separate /usr/share/doc/mandos-client/README.Debian into sections with
+    headlines.  Add instructions on how to test the server and verify the
+    password.
+
+ -- Teddy Hogeborn <teddy@fukt.bsnet.se>  Tue, 07 Oct 2008 23:07:23 +0200
+
+mandos (1.0-2) unstable; urgency=low
+
+  * Added comments in debian/*.lintian-overrides files.  Added Debian
+    revison number to version number.
+
+ -- Teddy Hogeborn <teddy@fukt.bsnet.se>  Wed, 01 Oct 2008 17:23:35 +0200
+
+mandos (1.0-1) unstable; urgency=low
+
+  * Initial Release. (Closes: #500727).
+
+ -- Teddy Hogeborn <teddy@fukt.bsnet.se>  Tue, 30 Sep 2008 21:58:43 +0200

=== modified file 'debian/control'
--- debian/control	2008-09-17 00:34:09 +0000
+++ debian/control	2009-01-26 23:47:44 +0000
@@ -2,17 +2,21 @@
 Section: admin
 Priority: extra
 Maintainer: Mandos Maintainers <mandos@fukt.bsnet.se>
-Build-Depends: debhelper (>= 7), docbook-xsl, docbook (<5.0),
-	libavahi-core-dev, libgpgme11-dev, libgnutls-dev, xsltproc
+Uploaders: Teddy Hogeborn <teddy@fukt.bsnet.se>,
+	   Björn Påhlsson <belorn@fukt.bsnet.se>
+Build-Depends: debhelper (>= 7), docbook-xml, docbook-xsl,
+	libavahi-core-dev, libgpgme11-dev, libgnutls-dev, xsltproc,
+	pkg-config
 Standards-Version: 3.8.0
-Vcs-Bzr: ftp://anonymous@ftp.fukt.bsnet.se/pub/mandos/latest
+Vcs-Bzr: http://ftp.fukt.bsnet.se/pub/mandos/trunk
+Vcs-Browser: http://bzr.fukt.bsnet.se/loggerhead/mandos/trunk/files
 Homepage: http://www.fukt.bsnet.se/mandos
 
 Package: mandos
 Architecture: all
-Depends: ${shlibs:Depends}, ${misc:Depends}, python (>=2.5),
-	python-gnutls, python-dbus, python-avahi, avahi-daemon,
-	gnupg (< 2)
+Depends: ${misc:Depends}, python (>=2.5), python-gnutls, python-dbus,
+	python-avahi, python-gobject, avahi-daemon, gnupg (< 2),
+	adduser
 Recommends: fping
 Description: a server giving encrypted passwords to Mandos clients
  This is the server part of the Mandos system, which allows
@@ -31,7 +35,7 @@
 
 Package: mandos-client
 Architecture: any
-Depends: ${shlibs:Depends}, ${misc:Depends}
+Depends: ${shlibs:Depends}, ${misc:Depends}, adduser, cryptsetup
 Enhances: cryptsetup
 Description: do unattended reboots with an encrypted root file system
  This is the client part of the Mandos system, which allows

=== modified file 'debian/copyright'
--- debian/copyright	2008-09-17 00:34:09 +0000
+++ debian/copyright	2009-01-04 21:54:55 +0000
@@ -1,27 +1,26 @@
-Authors: Teddy Hogeborn, Björn Påhlsson
-
-Homepage: <http://www.fukt.bsnet.se/mandos>
-
-Copyright:
-
-    Copyright © 2008 Teddy Hogeborn
-                2008 Björn Påhlsson
-
-License:
-
+Format-Specification:
+ http://wiki.debian.org/Proposals/CopyrightFormat?action=recall&rev=233
+Upstream-Name: Mandos
+Upstream-Maintainer: Mandos Maintainers <mandos@fukt.bsnet.se>
+Upstream-Source: <http://www.fukt.bsnet.se/mandos>
+
+Files: *
+Copyright: Copyright © 2008,2009 Teddy Hogeborn
+Copyright: Copyright © 2008,2009 Björn Påhlsson
+License: GPL-3+
     This program is free software: you can redistribute it and/or
     modify it under the terms of the GNU General Public License as
     published by the Free Software Foundation, either version 3 of the
     License, or (at your option) any later version.
-
+    .
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     General Public License for more details.
-
+    .
     You should have received a copy of the GNU General Public License
     along with this program.  If not, see
     <http://www.gnu.org/licenses/>.
-
-On Debian systems, the complete text of the GNU General Public License
-can be found in "/usr/share/common-licenses/GPL".
+    .
+    On Debian systems, the complete text of the GNU General Public
+    License can be found in "/usr/share/common-licenses/GPL".

=== modified file 'debian/mandos-client.README.Debian'
--- debian/mandos-client.README.Debian	2008-09-19 00:00:51 +0000
+++ debian/mandos-client.README.Debian	2009-01-15 02:52:02 +0000
@@ -1,15 +1,51 @@
-A client key has been automatically created in /etc/keys/mandos.  The
-next step is to run "mandos-keygen --password" to get a config file
-stanza to copy and paste into /etc/mandos/clients.conf on the Mandos
-server.
-
-Also, if some other network interface than "eth0" is used, it will be
-necessary to edit /etc/mandos/plugin-runner.conf to uncomment and
-change the line there.  If this file is changed, it will be necessary
-to update the initrd image by doing "update-initramfs -k all -u".
-
-It is NOT necessary to edit /etc/crypttab to specify
-/usr/lib/mandos/plugin-runner as a keyscript for the root file system;
-if no keyscript is given for the root file system, the Mandos client
-will be the new default way for getting a password for the root file
-system when booting.
+* Configure The Server
+  
+  A client key has been automatically created in /etc/keys/mandos.
+  The next step is to run "mandos-keygen --password" to get a config
+  file section.  This should be appended to /etc/mandos/clients.conf
+  on the Mandos server.
+
+* Use the Correct Network Interface
+  
+  If some other network interface than "eth0" is used, it will be
+  necessary to edit /etc/mandos/plugin-runner.conf to uncomment and
+  change the line there.  If this is done, it will be necessary to
+  update the initrd image by doing "update-initramfs -k all -u".
+
+* Test the Server
+  
+  After the server has been started and this client's key added, it is
+  possible to verify that the correct password will be received by
+  this client by running the command, on the client:
+  
+	# /usr/lib/mandos/plugins.d/mandos-client \
+		--pubkey=/etc/keys/mandos/pubkey.txt \
+		--seckey=/etc/keys/mandos/seckey.txt; echo
+  
+  This command should retrieve the password from the server, decrypt
+  it, and output it to standard output.  It is now possible to verify
+  the correctness of the password before rebooting.
+
+* User-Supplied Plugins
+
+  Any plugins found in /etc/mandos/plugins.d will override and add to
+  the normal Mandos plugins.  When adding or changing plugins, do not
+  forget to update the initital RAM disk image:
+
+	# update-initramfs -k all -u
+
+* Do *NOT* Edit /etc/crypttab
+
+  It is NOT necessary to edit /etc/crypttab to specify
+  /usr/lib/mandos/plugin-runner as a keyscript for the root file
+  system; if no keyscript is given for the root file system, the
+  Mandos client will be the new default way for getting a password for
+  the root file system when booting.
+
+* Emergency Escape
+  
+  If it ever should be necessary, the Mandos client can be temporarily
+  prevented from running at startup by passing the parameter
+  "mandos=off" to the kernel.
+
+ -- Teddy Hogeborn <teddy@fukt.bsnet.se>, Mon, 12 Jan 2009 02:29:10 +0100

=== added file 'debian/mandos-client.docs'
--- debian/mandos-client.docs	1970-01-01 00:00:00 +0000
+++ debian/mandos-client.docs	2008-10-18 11:17:22 +0000
@@ -0,0 +1,3 @@
+NEWS
+README
+TODO

=== added file 'debian/mandos-client.links'
--- debian/mandos-client.links	1970-01-01 00:00:00 +0000
+++ debian/mandos-client.links	2008-09-19 13:50:22 +0000
@@ -0,0 +1,1 @@
+usr/share/man/man8/plugin-runner.8mandos.gz usr/share/man/man5/plugin-runner.conf.5mandos.gz

=== modified file 'debian/mandos-client.lintian-overrides'
--- debian/mandos-client.lintian-overrides	2008-09-17 00:34:09 +0000
+++ debian/mandos-client.lintian-overrides	2009-01-18 06:41:57 +0000
@@ -1,4 +1,27 @@
-mandos-client binary: manpage-has-errors-from-man usr/share/man/man8/plugin-runner.8mandos.gz 297: warning [p 4, 5.8i]: can't break line
+# This directory contains secret client key files.
+#
 mandos-client binary: non-standard-dir-perm etc/keys/mandos/ 0700 != 0755
+
+# The directory /usr/lib/mandos/plugins.d contains setuid binaries
+# which are not meant to be run outside an initial RAM disk
+# environment (except for test purposes).  It would be insecure to
+# allow anyone to run them.
+#
+mandos-client binary: non-standard-dir-perm usr/lib/mandos/plugins.d/ 0700 != 0755
+
+# These binaries must be setuid root, since they need root powers, but
+# are started by plugin-runner(8mandos), which runs all plugins as
+# user/group "_mandos".  These binaries are not run in a running
+# system, but in an initial RAM disk environment.  Here they are
+# protected from non-root access by the directory permissions, above.
+#
 mandos-client binary: setuid-binary usr/lib/mandos/plugins.d/mandos-client 4755 root/root
-mandos-client binary: non-standard-dir-perm usr/lib/mandos/plugins.d/ 0700 != 0755
+mandos-client binary: setuid-binary usr/lib/mandos/plugins.d/askpass-fifo 4755 root/root
+mandos-client binary: setuid-binary usr/lib/mandos/plugins.d/splashy 4755 root/root
+mandos-client binary: setuid-binary usr/lib/mandos/plugins.d/usplash 4755 root/root
+
+# The directory /etc/mandos/plugins.d can be used by local system
+# administrators to place plugins in, overriding and complementing
+# /usr/lib/mandos/plugins.d, and must be likewise protected.
+#
+mandos-client binary: non-standard-dir-perm etc/mandos/plugins.d/ 0700 != 0755

=== modified file 'debian/mandos-client.postinst'
--- debian/mandos-client.postinst	2008-09-17 00:34:09 +0000
+++ debian/mandos-client.postinst	2009-01-18 00:16:57 +0000
@@ -1,4 +1,4 @@
-#!/bin/bash -e
+#!/bin/sh -e
 # This script can be called in the following ways:
 #
 # After the package was installed:
@@ -15,26 +15,54 @@
 # If prerm fails during replacement due to conflict:
 #       <postinst> abort-remove in-favour <new-package> <version>
 
-. /usr/share/debconf/confmodule
-
-# Update the initramfs
+# Update the initial RAM file system image
 update_initramfs()
 {
-	if which update-initramfs >/dev/null 2>&1; then
-		update-initramfs -u
-	fi
+    if [ -x /usr/sbin/update-initramfs ]; then
+	update-initramfs -u -k all
+    fi
+}
+
+# Add user and group
+add_mandos_user(){
+    # Rename old "mandos" user and group
+    case "`getent passwd mandos`" in
+	*:Mandos\ password\ system,,,:/nonexistent:/bin/false)
+	    usermod --login _mandos mandos
+	    groupmod --new-name _mandos mandos
+	    return
+	    ;;
+    esac
+    # Create new user and group
+    if ! getent passwd _mandos >/dev/null; then
+	adduser --system --force-badname --quiet --home /nonexistent \
+	    --no-create-home --group --disabled-password \
+	    --gecos "Mandos password system" _mandos
+    fi
+}
+
+# Create client key pair
+create_key(){
+    if [ -r /etc/keys/mandos/pubkey.txt \
+	-a -r /etc/keys/mandos/seckey.txt ]; then
+	return 0
+    fi
+    if [ -x /usr/sbin/mandos-keygen ]; then
+	mandos-keygen
+    fi
 }
 
 case "$1" in
     configure)
+	add_mandos_user
+	create_key
 	update_initramfs
 	;;
-
     abort-upgrade|abort-deconfigure|abort-remove)
 	;;
 
     *)
-	echo "$0 called with unknown argument \`$1'" 1>&2
+	echo "$0 called with unknown argument '$1'" 1>&2
 	exit 1
 	;;
 esac

=== modified file 'debian/mandos-client.postrm'
--- debian/mandos-client.postrm	2008-09-19 00:00:51 +0000
+++ debian/mandos-client.postrm	2009-01-18 00:16:57 +0000
@@ -27,32 +27,30 @@
 #       <new-postrm> abort-upgrade <old-version>
 
 
-# Update the initramfs
+# Update the initial RAM file system image
 update_initramfs()
 {
-	if type update-initramfs >/dev/null 2>&1 ; then
-		update-initramfs -u -k all
-	fi
+    if [ -x /usr/sbin/update-initramfs ]; then
+	update-initramfs -u -k all
+    fi
 }
 
-
 case "$1" in
     remove)
 	update_initramfs
 	;;
 
     purge)
-	shred --remove /etc/keys/mandos/seckey.txt || :
+	shred --remove /etc/keys/mandos/seckey.txt 2>/dev/null || :
 	rm --force /etc/mandos/plugin-runner.conf \
 	    /etc/keys/mandos/pubkey.txt \
-	    /etc/keys/mandos/seckey.txt
-	rmdir /etc/keys/mandos /etc/mandos/plugins.d /etc/mandos || :
+	    /etc/keys/mandos/seckey.txt 2>/dev/null
 	;;
     upgrade|failed-upgrade|disappear|abort-install|abort-upgrade)
 	;;
 
     *)
-	echo "$0 called with unknown argument \`$1'" 1>&2
+	echo "$0 called with unknown argument '$1'" 1>&2
 	exit 1
 	;;
 esac

=== added file 'debian/mandos.README.Debian'
--- debian/mandos.README.Debian	1970-01-01 00:00:00 +0000
+++ debian/mandos.README.Debian	2009-01-04 22:15:01 +0000
@@ -0,0 +1,7 @@
+The Mandos server is useless without at least one configured client in
+/etc/mandos/clients.conf.  To create one, install the "mandos-client"
+package on a client computer, and run "mandos-keygen --password" there
+to get a config file stanza.  Append that to /etc/mandos/clients.conf
+on the Mandos server.
+
+ -- Teddy Hogeborn <teddy@fukt.bsnet.se>, Sun,  4 Jan 2009 22:59:22 +0100

=== added file 'debian/mandos.docs'
--- debian/mandos.docs	1970-01-01 00:00:00 +0000
+++ debian/mandos.docs	2008-10-18 11:17:22 +0000
@@ -0,0 +1,3 @@
+NEWS
+README
+TODO

=== modified file 'debian/mandos.lintian-overrides'
--- debian/mandos.lintian-overrides	2008-09-17 00:34:09 +0000
+++ debian/mandos.lintian-overrides	2008-10-01 15:29:01 +0000
@@ -1,1 +1,4 @@
+# This config file will normally have encrypted secret client keys in
+# it, so it must be kept unreadable for non-root users.
+#
 mandos binary: non-standard-file-perm etc/mandos/clients.conf 0600 != 0644

=== modified file 'debian/mandos.postinst'
--- debian/mandos.postinst	2008-09-19 00:54:24 +0000
+++ debian/mandos.postinst	2009-01-18 00:16:57 +0000
@@ -1,4 +1,4 @@
-#!/bin/bash -e
+#!/bin/sh -e
 # This script can be called in the following ways:
 #
 # After the package was installed:
@@ -15,14 +15,21 @@
 # If prerm fails during replacement due to conflict:
 #       <postinst> abort-remove in-favour <new-package> <version>
 
-. /usr/share/debconf/confmodule
-
 case "$1" in
     configure)
-	if ! getent passwd mandos >/dev/null; then
-	    adduser --disabled-password  --quiet --system \
-		--home /var/run/mandos --no-create-home \
-		--gecos "Mandos password daemon" --group mandos
+	# Rename old "mandos" user and group
+	case "`getent passwd mandos`" in
+	    *:Mandos\ password\ system,,,:/nonexistent:/bin/false)
+		usermod --login _mandos mandos
+		groupmod --new-name _mandos mandos
+		;;
+	esac
+	# Create new user and group
+	if ! getent passwd _mandos >/dev/null; then
+	    adduser --system --force-badname --quiet \
+		--home /nonexistent --no-create-home --group \
+		--disabled-password --gecos "Mandos password system" \
+		_mandos
 	fi
 	;;
 
@@ -30,7 +37,7 @@
 	;;
 
     *)
-	echo "$0 called with unknown argument \`$1'" 1>&2
+	echo "$0 called with unknown argument '$1'" 1>&2
 	exit 1
 	;;
 esac

=== removed file 'debian/mandos.postrm'
--- debian/mandos.postrm	2008-09-19 00:54:24 +0000
+++ debian/mandos.postrm	1970-01-01 00:00:00 +0000
@@ -1,11 +0,0 @@
-#!/bin/sh
-
-set -e
-
-#DEBHELPER#
-
-if [ "$1" = "purge" ] ; then
-        deluser --quiet --system mandos > /dev/null || true
-        delgroup --quiet --system mandos > /dev/null || true
-fi
-

=== added file 'debian/mandos.prerm'
--- debian/mandos.prerm	1970-01-01 00:00:00 +0000
+++ debian/mandos.prerm	2009-01-18 00:16:57 +0000
@@ -0,0 +1,38 @@
+#! /bin/sh
+# prerm script for mandos
+#
+# see: dh_installdeb(1)
+
+set -e
+
+# summary of how this script can be called:
+#        * <prerm> 'remove'
+#        * <old-prerm> 'upgrade' <new-version>
+#        * <new-prerm> 'failed-upgrade' <old-version>
+#        * <conflictor's-prerm> 'remove' 'in-favour' <package> <new-version>
+#        * <deconfigured's-prerm> 'deconfigure' 'in-favour'
+#          <package-being-installed> <version> 'removing'
+#          <conflicting-package> <version>
+# for details, see /usr/share/doc/packaging-manual/
+
+case "$1" in
+    remove|deconfigure)
+	if [ -x /etc/init.d/mandos ]; then
+		if [ -x /usr/sbin/invoke-rc.d ]; then
+			invoke-rc.d mandos stop
+		else
+			/etc/init.d/mandos stop
+		fi
+	fi
+	;;
+    upgrade|failed-upgrade)
+        ;;
+    *)
+        echo "prerm called with unknown argument '$1'" >&2
+        exit 0
+    ;;
+esac
+
+#DEBHELPER#
+
+exit 0

=== added directory 'debian/po'
=== modified file 'debian/rules'
--- debian/rules	2008-09-17 00:34:09 +0000
+++ debian/rules	2009-01-18 00:18:50 +0000
@@ -10,7 +10,7 @@
 # build-arch and build-indep targets  by Bill Allombert 2001
 
 # Uncomment this to turn on verbose mode.
-export DH_VERBOSE=1
+#export DH_VERBOSE=1
 
 # This has to be exported to make some magic below work.
 export DH_OPTIONS
@@ -18,21 +18,17 @@
 configure: configure-stamp
 configure-stamp:
 	dh_testdir
-# Add here commands to configure the package.
 	touch configure-stamp
 
-#Architecture
 build: build-arch build-indep
 
 build-arch: build-arch-stamp
 build-arch-stamp: configure-stamp
-# Add here commands to compile the arch part of the package.
 	dh_auto_build -- all doc
 	touch $@
 
 build-indep: build-indep-stamp
 build-indep-stamp: configure-stamp
-# Add here commands to compile the indep part of the package.
 	dh_auto_build -- doc
 	touch $@
 
@@ -40,7 +36,6 @@
 	dh_testdir
 	dh_testroot
 	rm -f build-arch-stamp build-indep-stamp configure-stamp
-# Add here commands to clean up after the build process.
 	dh_auto_clean
 	dh_clean
 
@@ -50,11 +45,10 @@
 	dh_testroot
 	dh_prep
 	dh_installdirs --indep
-# Add here commands to install the indep part of the package into
-# debian/<package>-doc.
 	$(MAKE) DESTDIR=$(CURDIR)/debian/mandos install-server
 	dh_lintian
-	dh_installinit --onlyscripts --no-start
+	dh_installinit --onlyscripts \
+		--update-rcd-params="defaults 25 15"
 	dh_install --indep
 
 install-arch:
@@ -62,46 +56,28 @@
 	dh_testroot
 	dh_prep
 	dh_installdirs --same-arch
-
-# Add here commands to install the arch part of the package into
-# debian/tmp.
 	$(MAKE) DESTDIR=$(CURDIR)/debian/mandos-client install-client-nokey
 	dh_lintian
 	dh_install --same-arch
 
-# Must not depend on anything. This is to be called by
-# binary-arch/binary-indep
-# in another 'make' thread.
 binary-common:
 	dh_testdir
 	dh_testroot
 	dh_installchangelogs
 	dh_installdocs
-#	dh_installexamples
-#	dh_installmenu
-#	dh_installdebconf
-#	dh_installlogrotate
-#	dh_installemacsen
-#	dh_installpam
-#	dh_installmime
-#	dh_pycentral
-#	dh_installinit
-#	dh_installcron
-#	dh_installinfo
-#	dh_installman
 	dh_link
 	dh_strip
 	dh_compress
 	dh_fixperms --exclude etc/keys/mandos \
 		--exclude etc/mandos/clients.conf \
+		--exclude etc/mandos/plugins.d \
 		--exclude usr/lib/mandos/plugins.d
-#	dh_perl
-	dh_makeshlibs
 	dh_installdeb
 	dh_shlibdeps
 	dh_gencontrol
 	dh_md5sums
 	dh_builddeb
+
 # Build architecture independant packages using the common target.
 binary-indep: build-indep install-indep
 	$(MAKE) -f debian/rules DH_OPTIONS=--indep binary-common

=== added file 'debian/watch'
--- debian/watch	1970-01-01 00:00:00 +0000
+++ debian/watch	2009-01-15 02:52:02 +0000
@@ -0,0 +1,2 @@
+version=3
+ftp://ftp.fukt.bsnet.se/pub/mandos/mandos[-_]([^\s]+?)(?:\.orig)?\.tar\.(?:gz|bz2|7z)

=== removed file 'etc-plugins.d-README'
--- etc-plugins.d-README	2008-09-06 16:11:50 +0000
+++ etc-plugins.d-README	1970-01-01 00:00:00 +0000
@@ -1,5 +0,0 @@
-Any plugins found here in /etc/mandos/plugins.d will override and add
-to the normal Mandos plugins.  When adding or changing plugins, do not
-forget to update the initital RAM disk image:
-
-# update-initramfs -k all -u

=== modified file 'init.d-mandos'
--- init.d-mandos	2008-09-05 16:24:33 +0000
+++ init.d-mandos	2008-09-21 12:04:02 +0000
@@ -1,7 +1,7 @@
 #! /bin/sh
 ### BEGIN INIT INFO
 # Provides:          mandos
-# Required-Start:    $remote_fs
+# Required-Start:    $remote_fs avahi-daemon
 # Required-Stop:     $remote_fs
 # Default-Start:     2 3 4 5
 # Default-Stop:      0 1 6

=== modified file 'initramfs-tools-hook'
--- initramfs-tools-hook	2008-09-19 00:54:24 +0000
+++ initramfs-tools-hook	2008-12-10 01:26:02 +0000
@@ -51,11 +51,13 @@
     exit 1
 fi
 
-mandos_user="`{ getent passwd mandos \
+mandos_user="`{ getent passwd _mandos \
+		|| getent passwd mandos \
 		|| getent passwd nobody \
 		|| echo ::65534::::; } \
 	| awk --field-separator=: '{ print $3 }'`" 
-mandos_group="`{ getent group mandos \
+mandos_group="`{ getent group _mandos \
+		|| getent group mandos \
 		|| getent group nogroup \
 		|| echo ::65534:; } \
 	| awk --field-separator=: '{ print $3 }'`"
@@ -89,7 +91,7 @@
 	continue
     fi
     case "$base" in
-	*~|.*|\#*\#|*.dpkg-old|*.dpkg-new|*.dpkg-divert) : ;;
+	*~|.*|\#*\#|*.dpkg-old|*.dpkg-bak|*.dpkg-new|*.dpkg-divert) : ;;
 	"*") :;;
 	*) copy_exec "$file" "${PLUGINDIR}";;
     esac
@@ -99,7 +101,7 @@
 for file in /etc/mandos/plugins.d/*; do
     base="`basename \"$file\"`"
     case "$base" in
-	*~|.*|\#*\#|*.dpkg-old|*.dpkg-new|*.dpkg-divert) : ;;
+	*~|.*|\#*\#|*.dpkg-old|*.dpkg-bak|*.dpkg-new|*.dpkg-divert) : ;;
 	"*") :;;
 	*) copy_exec "$file" "${PLUGINDIR}";;
     esac

=== modified file 'initramfs-tools-script'
--- initramfs-tools-script	2008-09-07 15:42:11 +0000
+++ initramfs-tools-script	2009-01-15 02:52:02 +0000
@@ -24,6 +24,12 @@
      ;;
 esac
 
+for param in `cat /proc/cmdline`; do
+    case "$param" in
+	mandos=off) exit 0;;
+    esac
+done
+
 chmod a=rwxt /tmp
 
 test -w /conf/conf.d/cryptroot

=== modified file 'mandos'
--- mandos	2008-09-05 18:37:28 +0000
+++ mandos	2009-01-23 23:17:42 +0000
@@ -11,7 +11,8 @@
 # and some lines in "main".
 # 
 # Everything else is
-# Copyright © 2007-2008 Teddy Hogeborn & Björn Påhlsson
+# Copyright © 2008,2009 Teddy Hogeborn
+# Copyright © 2008,2009 Björn Påhlsson
 # 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -30,12 +31,11 @@
 # Contact the authors at <mandos@fukt.bsnet.se>.
 # 
 
-from __future__ import division
+from __future__ import division, with_statement, absolute_import
 
 import SocketServer
 import socket
-import select
-from optparse import OptionParser
+import optparse
 import datetime
 import errno
 import gnutls.crypto
@@ -56,21 +56,24 @@
 import logging
 import logging.handlers
 import pwd
+from contextlib import closing
 
 import dbus
+import dbus.service
 import gobject
 import avahi
 from dbus.mainloop.glib import DBusGMainLoop
 import ctypes
+import ctypes.util
 
-version = "1.0"
+version = "1.0.5"
 
 logger = logging.Logger('mandos')
-syslogger = logging.handlers.SysLogHandler\
-            (facility = logging.handlers.SysLogHandler.LOG_DAEMON,
-             address = "/dev/log")
-syslogger.setFormatter(logging.Formatter\
-                        ('Mandos: %(levelname)s: %(message)s'))
+syslogger = (logging.handlers.SysLogHandler
+             (facility = logging.handlers.SysLogHandler.LOG_DAEMON,
+              address = "/dev/log"))
+syslogger.setFormatter(logging.Formatter
+                       ('Mandos: %(levelname)s: %(message)s'))
 logger.addHandler(syslogger)
 
 console = logging.StreamHandler()
@@ -79,10 +82,11 @@
 logger.addHandler(console)
 
 class AvahiError(Exception):
-    def __init__(self, value):
+    def __init__(self, value, *args, **kwargs):
         self.value = value
-    def __str__(self):
-        return repr(self.value)
+        super(AvahiError, self).__init__(value, *args, **kwargs)
+    def __unicode__(self):
+        return unicode(repr(self.value))
 
 class AvahiServiceError(AvahiError):
     pass
@@ -108,16 +112,13 @@
                   a sensible number of times
     """
     def __init__(self, interface = avahi.IF_UNSPEC, name = None,
-                 type = None, port = None, TXT = None, domain = "",
-                 host = "", max_renames = 32768):
+                 servicetype = None, port = None, TXT = None,
+                 domain = "", host = "", max_renames = 32768):
         self.interface = interface
         self.name = name
-        self.type = type
+        self.type = servicetype
         self.port = port
-        if TXT is None:
-            self.TXT = []
-        else:
-            self.TXT = TXT
+        self.TXT = TXT if TXT is not None else []
         self.domain = domain
         self.host = host
         self.rename_count = 0
@@ -127,14 +128,14 @@
         if self.rename_count >= self.max_renames:
             logger.critical(u"No suitable Zeroconf service name found"
                             u" after %i retries, exiting.",
-                            rename_count)
-            raise AvahiServiceError("Too many renames")
+                            self.rename_count)
+            raise AvahiServiceError(u"Too many renames")
         self.name = server.GetAlternativeServiceName(self.name)
         logger.info(u"Changing Zeroconf service name to %r ...",
                     str(self.name))
-        syslogger.setFormatter(logging.Formatter\
+        syslogger.setFormatter(logging.Formatter
                                ('Mandos (%s): %%(levelname)s:'
-                               ' %%(message)s' % self.name))
+                                ' %%(message)s' % self.name))
         self.remove()
         self.add()
         self.rename_count += 1
@@ -146,10 +147,10 @@
         """Derived from the Avahi example code"""
         global group
         if group is None:
-            group = dbus.Interface\
-                    (bus.get_object(avahi.DBUS_NAME,
+            group = dbus.Interface(bus.get_object
+                                   (avahi.DBUS_NAME,
                                     server.EntryGroupNew()),
-                     avahi.DBUS_INTERFACE_ENTRY_GROUP)
+                                   avahi.DBUS_INTERFACE_ENTRY_GROUP)
             group.connect_to_signal('StateChanged',
                                     entry_group_state_changed)
         logger.debug(u"Adding Zeroconf service '%s' of type '%s' ...",
@@ -169,148 +170,200 @@
 # End of Avahi example code
 
 
-class Client(object):
+def _datetime_to_dbus(dt, variant_level=0):
+    """Convert a UTC datetime.datetime() to a D-Bus type."""
+    return dbus.String(dt.isoformat(), variant_level=variant_level)
+
+
+class Client(dbus.service.Object):
     """A representation of a client host served by this server.
     Attributes:
-    name:      string; from the config file, used in log messages
+    name:       string; from the config file, used in log messages and
+                        D-Bus identifiers
     fingerprint: string (40 or 32 hexadecimal digits); used to
                  uniquely identify the client
-    secret:    bytestring; sent verbatim (over TLS) to client
-    host:      string; available for use by the checker command
-    created:   datetime.datetime(); object creation, not client host
-    last_checked_ok: datetime.datetime() or None if not yet checked OK
-    timeout:   datetime.timedelta(); How long from last_checked_ok
-                                     until this client is invalid
-    interval:  datetime.timedelta(); How often to start a new checker
-    stop_hook: If set, called by stop() as stop_hook(self)
-    checker:   subprocess.Popen(); a running checker process used
-                                   to see if the client lives.
-                                   'None' if no process is running.
+    secret:     bytestring; sent verbatim (over TLS) to client
+    host:       string; available for use by the checker command
+    created:    datetime.datetime(); (UTC) object creation
+    last_enabled: datetime.datetime(); (UTC)
+    enabled:    bool()
+    last_checked_ok: datetime.datetime(); (UTC) or None
+    timeout:    datetime.timedelta(); How long from last_checked_ok
+                                      until this client is invalid
+    interval:   datetime.timedelta(); How often to start a new checker
+    disable_hook:  If set, called by disable() as disable_hook(self)
+    checker:    subprocess.Popen(); a running checker process used
+                                    to see if the client lives.
+                                    'None' if no process is running.
     checker_initiator_tag: a gobject event source tag, or None
-    stop_initiator_tag:    - '' -
+    disable_initiator_tag:    - '' -
     checker_callback_tag:  - '' -
     checker_command: string; External command which is run to check if
                      client lives.  %() expansions are done at
                      runtime with vars(self) as dict, so that for
                      instance %(name)s can be used in the command.
-    Private attibutes:
-    _timeout: Real variable for 'timeout'
-    _interval: Real variable for 'interval'
-    _timeout_milliseconds: Used when calling gobject.timeout_add()
-    _interval_milliseconds: - '' -
+    use_dbus: bool(); Whether to provide D-Bus interface and signals
+    dbus_object_path: dbus.ObjectPath ; only set if self.use_dbus
     """
-    def _set_timeout(self, timeout):
-        "Setter function for 'timeout' attribute"
-        self._timeout = timeout
-        self._timeout_milliseconds = ((self.timeout.days
-                                       * 24 * 60 * 60 * 1000)
-                                      + (self.timeout.seconds * 1000)
-                                      + (self.timeout.microseconds
-                                         // 1000))
-    timeout = property(lambda self: self._timeout,
-                       _set_timeout)
-    del _set_timeout
-    def _set_interval(self, interval):
-        "Setter function for 'interval' attribute"
-        self._interval = interval
-        self._interval_milliseconds = ((self.interval.days
-                                        * 24 * 60 * 60 * 1000)
-                                       + (self.interval.seconds
-                                          * 1000)
-                                       + (self.interval.microseconds
-                                          // 1000))
-    interval = property(lambda self: self._interval,
-                        _set_interval)
-    del _set_interval
-    def __init__(self, name = None, stop_hook=None, config={}):
+    def timeout_milliseconds(self):
+        "Return the 'timeout' attribute in milliseconds"
+        return ((self.timeout.days * 24 * 60 * 60 * 1000)
+                + (self.timeout.seconds * 1000)
+                + (self.timeout.microseconds // 1000))
+    
+    def interval_milliseconds(self):
+        "Return the 'interval' attribute in milliseconds"
+        return ((self.interval.days * 24 * 60 * 60 * 1000)
+                + (self.interval.seconds * 1000)
+                + (self.interval.microseconds // 1000))
+    
+    def __init__(self, name = None, disable_hook=None, config=None,
+                 use_dbus=True):
         """Note: the 'checker' key in 'config' sets the
         'checker_command' attribute and *not* the 'checker'
         attribute."""
         self.name = name
+        if config is None:
+            config = {}
         logger.debug(u"Creating client %r", self.name)
+        self.use_dbus = False   # During __init__
         # Uppercase and remove spaces from fingerprint for later
         # comparison purposes with return value from the fingerprint()
         # function
-        self.fingerprint = config["fingerprint"].upper()\
-                           .replace(u" ", u"")
+        self.fingerprint = (config["fingerprint"].upper()
+                            .replace(u" ", u""))
         logger.debug(u"  Fingerprint: %s", self.fingerprint)
         if "secret" in config:
             self.secret = config["secret"].decode(u"base64")
         elif "secfile" in config:
-            sf = open(config["secfile"])
-            self.secret = sf.read()
-            sf.close()
+            with closing(open(os.path.expanduser
+                              (os.path.expandvars
+                               (config["secfile"])))) as secfile:
+                self.secret = secfile.read()
         else:
             raise TypeError(u"No secret or secfile for client %s"
                             % self.name)
         self.host = config.get("host", "")
-        self.created = datetime.datetime.now()
+        self.created = datetime.datetime.utcnow()
+        self.enabled = False
+        self.last_enabled = None
         self.last_checked_ok = None
         self.timeout = string_to_delta(config["timeout"])
         self.interval = string_to_delta(config["interval"])
-        self.stop_hook = stop_hook
+        self.disable_hook = disable_hook
         self.checker = None
         self.checker_initiator_tag = None
-        self.stop_initiator_tag = None
+        self.disable_initiator_tag = None
         self.checker_callback_tag = None
-        self.check_command = config["checker"]
-    def start(self):
+        self.checker_command = config["checker"]
+        self.last_connect = None
+        # Only now, when this client is initialized, can it show up on
+        # the D-Bus
+        self.use_dbus = use_dbus
+        if self.use_dbus:
+            self.dbus_object_path = (dbus.ObjectPath
+                                     ("/clients/"
+                                      + self.name.replace(".", "_")))
+            dbus.service.Object.__init__(self, bus,
+                                         self.dbus_object_path)
+    
+    def enable(self):
         """Start this client's checker and timeout hooks"""
+        self.last_enabled = datetime.datetime.utcnow()
         # Schedule a new checker to be started an 'interval' from now,
         # and every interval from then on.
-        self.checker_initiator_tag = gobject.timeout_add\
-                                     (self._interval_milliseconds,
-                                      self.start_checker)
+        self.checker_initiator_tag = (gobject.timeout_add
+                                      (self.interval_milliseconds(),
+                                       self.start_checker))
         # Also start a new checker *right now*.
         self.start_checker()
-        # Schedule a stop() when 'timeout' has passed
-        self.stop_initiator_tag = gobject.timeout_add\
-                                  (self._timeout_milliseconds,
-                                   self.stop)
-    def stop(self):
-        """Stop this client.
-        The possibility that a client might be restarted is left open,
-        but not currently used."""
-        # If this client doesn't have a secret, it is already stopped.
-        if hasattr(self, "secret") and self.secret:
-            logger.info(u"Stopping client %s", self.name)
-            self.secret = None
-        else:
+        # Schedule a disable() when 'timeout' has passed
+        self.disable_initiator_tag = (gobject.timeout_add
+                                   (self.timeout_milliseconds(),
+                                    self.disable))
+        self.enabled = True
+        if self.use_dbus:
+            # Emit D-Bus signals
+            self.PropertyChanged(dbus.String(u"enabled"),
+                                 dbus.Boolean(True, variant_level=1))
+            self.PropertyChanged(dbus.String(u"last_enabled"),
+                                 (_datetime_to_dbus(self.last_enabled,
+                                                    variant_level=1)))
+    
+    def disable(self):
+        """Disable this client."""
+        if not getattr(self, "enabled", False):
             return False
-        if getattr(self, "stop_initiator_tag", False):
-            gobject.source_remove(self.stop_initiator_tag)
-            self.stop_initiator_tag = None
+        logger.info(u"Disabling client %s", self.name)
+        if getattr(self, "disable_initiator_tag", False):
+            gobject.source_remove(self.disable_initiator_tag)
+            self.disable_initiator_tag = None
         if getattr(self, "checker_initiator_tag", False):
             gobject.source_remove(self.checker_initiator_tag)
             self.checker_initiator_tag = None
         self.stop_checker()
-        if self.stop_hook:
-            self.stop_hook(self)
+        if self.disable_hook:
+            self.disable_hook(self)
+        self.enabled = False
+        if self.use_dbus:
+            # Emit D-Bus signal
+            self.PropertyChanged(dbus.String(u"enabled"),
+                                 dbus.Boolean(False, variant_level=1))
         # Do not run this again if called by a gobject.timeout_add
         return False
+    
     def __del__(self):
-        self.stop_hook = None
-        self.stop()
-    def checker_callback(self, pid, condition):
+        self.disable_hook = None
+        self.disable()
+    
+    def checker_callback(self, pid, condition, command):
         """The checker has completed, so take appropriate actions."""
-        now = datetime.datetime.now()
         self.checker_callback_tag = None
         self.checker = None
-        if os.WIFEXITED(condition) \
-               and (os.WEXITSTATUS(condition) == 0):
-            logger.info(u"Checker for %(name)s succeeded",
-                        vars(self))
-            self.last_checked_ok = now
-            gobject.source_remove(self.stop_initiator_tag)
-            self.stop_initiator_tag = gobject.timeout_add\
-                                      (self._timeout_milliseconds,
-                                       self.stop)
-        elif not os.WIFEXITED(condition):
+        if self.use_dbus:
+            # Emit D-Bus signal
+            self.PropertyChanged(dbus.String(u"checker_running"),
+                                 dbus.Boolean(False, variant_level=1))
+        if os.WIFEXITED(condition):
+            exitstatus = os.WEXITSTATUS(condition)
+            if exitstatus == 0:
+                logger.info(u"Checker for %(name)s succeeded",
+                            vars(self))
+                self.checked_ok()
+            else:
+                logger.info(u"Checker for %(name)s failed",
+                            vars(self))
+            if self.use_dbus:
+                # Emit D-Bus signal
+                self.CheckerCompleted(dbus.Int16(exitstatus),
+                                      dbus.Int64(condition),
+                                      dbus.String(command))
+        else:
             logger.warning(u"Checker for %(name)s crashed?",
                            vars(self))
-        else:
-            logger.info(u"Checker for %(name)s failed",
-                        vars(self))
+            if self.use_dbus:
+                # Emit D-Bus signal
+                self.CheckerCompleted(dbus.Int16(-1),
+                                      dbus.Int64(condition),
+                                      dbus.String(command))
+    
+    def checked_ok(self):
+        """Bump up the timeout for this client.
+        This should only be called when the client has been seen,
+        alive and well.
+        """
+        self.last_checked_ok = datetime.datetime.utcnow()
+        gobject.source_remove(self.disable_initiator_tag)
+        self.disable_initiator_tag = (gobject.timeout_add
+                                      (self.timeout_milliseconds(),
+                                       self.disable))
+        if self.use_dbus:
+            # Emit D-Bus signal
+            self.PropertyChanged(
+                dbus.String(u"last_checked_ok"),
+                (_datetime_to_dbus(self.last_checked_ok,
+                                   variant_level=1)))
+    
     def start_checker(self):
         """Start a new checker subprocess if one is not running.
         If a checker already exists, leave it running and do
@@ -325,18 +378,18 @@
         # is as it should be.
         if self.checker is None:
             try:
-                # In case check_command has exactly one % operator
-                command = self.check_command % self.host
+                # In case checker_command has exactly one % operator
+                command = self.checker_command % self.host
             except TypeError:
                 # Escape attributes for the shell
                 escaped_attrs = dict((key, re.escape(str(val)))
                                      for key, val in
                                      vars(self).iteritems())
                 try:
-                    command = self.check_command % escaped_attrs
+                    command = self.checker_command % escaped_attrs
                 except TypeError, error:
                     logger.error(u'Could not format string "%s":'
-                                 u' %s', self.check_command, error)
+                                 u' %s', self.checker_command, error)
                     return True # Try again later
             try:
                 logger.info(u"Starting checker %r for %s",
@@ -348,14 +401,22 @@
                 self.checker = subprocess.Popen(command,
                                                 close_fds=True,
                                                 shell=True, cwd="/")
-                self.checker_callback_tag = gobject.child_watch_add\
-                                            (self.checker.pid,
-                                             self.checker_callback)
+                if self.use_dbus:
+                    # Emit D-Bus signal
+                    self.CheckerStarted(command)
+                    self.PropertyChanged(
+                        dbus.String("checker_running"),
+                        dbus.Boolean(True, variant_level=1))
+                self.checker_callback_tag = (gobject.child_watch_add
+                                             (self.checker.pid,
+                                              self.checker_callback,
+                                              data=command))
             except OSError, error:
                 logger.error(u"Failed to start subprocess: %s",
                              error)
         # Re-run this periodically if run by gobject.timeout_add
         return True
+    
     def stop_checker(self):
         """Force the checker process, if any, to stop."""
         if self.checker_callback_tag:
@@ -373,26 +434,174 @@
             if error.errno != errno.ESRCH: # No such process
                 raise
         self.checker = None
+        if self.use_dbus:
+            self.PropertyChanged(dbus.String(u"checker_running"),
+                                 dbus.Boolean(False, variant_level=1))
+    
     def still_valid(self):
         """Has the timeout not yet passed for this client?"""
-        now = datetime.datetime.now()
+        if not getattr(self, "enabled", False):
+            return False
+        now = datetime.datetime.utcnow()
         if self.last_checked_ok is None:
             return now < (self.created + self.timeout)
         else:
             return now < (self.last_checked_ok + self.timeout)
+    
+    ## D-Bus methods & signals
+    _interface = u"se.bsnet.fukt.Mandos.Client"
+    
+    # CheckedOK - method
+    CheckedOK = dbus.service.method(_interface)(checked_ok)
+    CheckedOK.__name__ = "CheckedOK"
+    
+    # CheckerCompleted - signal
+    @dbus.service.signal(_interface, signature="nxs")
+    def CheckerCompleted(self, exitcode, waitstatus, command):
+        "D-Bus signal"
+        pass
+    
+    # CheckerStarted - signal
+    @dbus.service.signal(_interface, signature="s")
+    def CheckerStarted(self, command):
+        "D-Bus signal"
+        pass
+    
+    # GetAllProperties - method
+    @dbus.service.method(_interface, out_signature="a{sv}")
+    def GetAllProperties(self):
+        "D-Bus method"
+        return dbus.Dictionary({
+                dbus.String("name"):
+                    dbus.String(self.name, variant_level=1),
+                dbus.String("fingerprint"):
+                    dbus.String(self.fingerprint, variant_level=1),
+                dbus.String("host"):
+                    dbus.String(self.host, variant_level=1),
+                dbus.String("created"):
+                    _datetime_to_dbus(self.created, variant_level=1),
+                dbus.String("last_enabled"):
+                    (_datetime_to_dbus(self.last_enabled,
+                                       variant_level=1)
+                     if self.last_enabled is not None
+                     else dbus.Boolean(False, variant_level=1)),
+                dbus.String("enabled"):
+                    dbus.Boolean(self.enabled, variant_level=1),
+                dbus.String("last_checked_ok"):
+                    (_datetime_to_dbus(self.last_checked_ok,
+                                       variant_level=1)
+                     if self.last_checked_ok is not None
+                     else dbus.Boolean (False, variant_level=1)),
+                dbus.String("timeout"):
+                    dbus.UInt64(self.timeout_milliseconds(),
+                                variant_level=1),
+                dbus.String("interval"):
+                    dbus.UInt64(self.interval_milliseconds(),
+                                variant_level=1),
+                dbus.String("checker"):
+                    dbus.String(self.checker_command,
+                                variant_level=1),
+                dbus.String("checker_running"):
+                    dbus.Boolean(self.checker is not None,
+                                 variant_level=1),
+                dbus.String("object_path"):
+                    dbus.ObjectPath(self.dbus_object_path,
+                                    variant_level=1)
+                }, signature="sv")
+    
+    # IsStillValid - method
+    IsStillValid = (dbus.service.method(_interface, out_signature="b")
+                    (still_valid))
+    IsStillValid.__name__ = "IsStillValid"
+    
+    # PropertyChanged - signal
+    @dbus.service.signal(_interface, signature="sv")
+    def PropertyChanged(self, property, value):
+        "D-Bus signal"
+        pass
+    
+    # SetChecker - method
+    @dbus.service.method(_interface, in_signature="s")
+    def SetChecker(self, checker):
+        "D-Bus setter method"
+        self.checker_command = checker
+        # Emit D-Bus signal
+        self.PropertyChanged(dbus.String(u"checker"),
+                             dbus.String(self.checker_command,
+                                         variant_level=1))
+    
+    # SetHost - method
+    @dbus.service.method(_interface, in_signature="s")
+    def SetHost(self, host):
+        "D-Bus setter method"
+        self.host = host
+        # Emit D-Bus signal
+        self.PropertyChanged(dbus.String(u"host"),
+                             dbus.String(self.host, variant_level=1))
+    
+    # SetInterval - method
+    @dbus.service.method(_interface, in_signature="t")
+    def SetInterval(self, milliseconds):
+        self.interval = datetime.timedelta(0, 0, 0, milliseconds)
+        # Emit D-Bus signal
+        self.PropertyChanged(dbus.String(u"interval"),
+                             (dbus.UInt64(self.interval_milliseconds(),
+                                          variant_level=1)))
+    
+    # SetSecret - method
+    @dbus.service.method(_interface, in_signature="ay",
+                         byte_arrays=True)
+    def SetSecret(self, secret):
+        "D-Bus setter method"
+        self.secret = str(secret)
+    
+    # SetTimeout - method
+    @dbus.service.method(_interface, in_signature="t")
+    def SetTimeout(self, milliseconds):
+        self.timeout = datetime.timedelta(0, 0, 0, milliseconds)
+        # Emit D-Bus signal
+        self.PropertyChanged(dbus.String(u"timeout"),
+                             (dbus.UInt64(self.timeout_milliseconds(),
+                                          variant_level=1)))
+    
+    # Enable - method
+    Enable = dbus.service.method(_interface)(enable)
+    Enable.__name__ = "Enable"
+    
+    # StartChecker - method
+    @dbus.service.method(_interface)
+    def StartChecker(self):
+        "D-Bus method"
+        self.start_checker()
+    
+    # Disable - method
+    @dbus.service.method(_interface)
+    def Disable(self):
+        "D-Bus method"
+        self.disable()
+    
+    # StopChecker - method
+    StopChecker = dbus.service.method(_interface)(stop_checker)
+    StopChecker.__name__ = "StopChecker"
+    
+    del _interface
 
 
 def peer_certificate(session):
     "Return the peer's OpenPGP certificate as a bytestring"
     # If not an OpenPGP certificate...
-    if gnutls.library.functions.gnutls_certificate_type_get\
-            (session._c_object) \
-           != gnutls.library.constants.GNUTLS_CRT_OPENPGP:
+    if (gnutls.library.functions
+        .gnutls_certificate_type_get(session._c_object)
+        != gnutls.library.constants.GNUTLS_CRT_OPENPGP):
         # ...do the normal thing
         return session.peer_certificate
-    list_size = ctypes.c_uint()
-    cert_list = gnutls.library.functions.gnutls_certificate_get_peers\
-        (session._c_object, ctypes.byref(list_size))
+    list_size = ctypes.c_uint(1)
+    cert_list = (gnutls.library.functions
+                 .gnutls_certificate_get_peers
+                 (session._c_object, ctypes.byref(list_size)))
+    if not bool(cert_list) and list_size.value != 0:
+        raise gnutls.errors.GNUTLSError("error getting peer"
+                                        " certificate")
     if list_size.value == 0:
         return None
     cert = cert_list[0]
@@ -402,50 +611,55 @@
 def fingerprint(openpgp):
     "Convert an OpenPGP bytestring to a hexdigit fingerprint string"
     # New GnuTLS "datum" with the OpenPGP public key
-    datum = gnutls.library.types.gnutls_datum_t\
-        (ctypes.cast(ctypes.c_char_p(openpgp),
-                     ctypes.POINTER(ctypes.c_ubyte)),
-         ctypes.c_uint(len(openpgp)))
+    datum = (gnutls.library.types
+             .gnutls_datum_t(ctypes.cast(ctypes.c_char_p(openpgp),
+                                         ctypes.POINTER
+                                         (ctypes.c_ubyte)),
+                             ctypes.c_uint(len(openpgp))))
     # New empty GnuTLS certificate
     crt = gnutls.library.types.gnutls_openpgp_crt_t()
-    gnutls.library.functions.gnutls_openpgp_crt_init\
-        (ctypes.byref(crt))
+    (gnutls.library.functions
+     .gnutls_openpgp_crt_init(ctypes.byref(crt)))
     # Import the OpenPGP public key into the certificate
-    gnutls.library.functions.gnutls_openpgp_crt_import\
-                    (crt, ctypes.byref(datum),
-                     gnutls.library.constants.GNUTLS_OPENPGP_FMT_RAW)
+    (gnutls.library.functions
+     .gnutls_openpgp_crt_import(crt, ctypes.byref(datum),
+                                gnutls.library.constants
+                                .GNUTLS_OPENPGP_FMT_RAW))
     # Verify the self signature in the key
-    crtverify = ctypes.c_uint();
-    gnutls.library.functions.gnutls_openpgp_crt_verify_self\
-        (crt, 0, ctypes.byref(crtverify))
+    crtverify = ctypes.c_uint()
+    (gnutls.library.functions
+     .gnutls_openpgp_crt_verify_self(crt, 0, ctypes.byref(crtverify)))
     if crtverify.value != 0:
         gnutls.library.functions.gnutls_openpgp_crt_deinit(crt)
         raise gnutls.errors.CertificateSecurityError("Verify failed")
     # New buffer for the fingerprint
-    buffer = ctypes.create_string_buffer(20)
-    buffer_length = ctypes.c_size_t()
+    buf = ctypes.create_string_buffer(20)
+    buf_len = ctypes.c_size_t()
     # Get the fingerprint from the certificate into the buffer
-    gnutls.library.functions.gnutls_openpgp_crt_get_fingerprint\
-        (crt, ctypes.byref(buffer), ctypes.byref(buffer_length))
+    (gnutls.library.functions
+     .gnutls_openpgp_crt_get_fingerprint(crt, ctypes.byref(buf),
+                                         ctypes.byref(buf_len)))
     # Deinit the certificate
     gnutls.library.functions.gnutls_openpgp_crt_deinit(crt)
     # Convert the buffer to a Python bytestring
-    fpr = ctypes.string_at(buffer, buffer_length.value)
+    fpr = ctypes.string_at(buf, buf_len.value)
     # Convert the bytestring to hexadecimal notation
     hex_fpr = u''.join(u"%02X" % ord(char) for char in fpr)
     return hex_fpr
 
 
-class tcp_handler(SocketServer.BaseRequestHandler, object):
+class TCP_handler(SocketServer.BaseRequestHandler, object):
     """A TCP request handler class.
     Instantiated by IPv6_TCPServer for each request to handle it.
     Note: This will run in its own forked process."""
     
     def handle(self):
         logger.info(u"TCP connection from: %s",
-                     unicode(self.client_address))
-        session = gnutls.connection.ClientSession\
-                  (self.request, gnutls.connection.X509Credentials())
+                    unicode(self.client_address))
+        session = (gnutls.connection
+                   .ClientSession(self.request,
+                                  gnutls.connection
+                                  .X509Credentials()))
         
         line = self.request.makefile().readline()
         logger.debug(u"Protocol version: %r", line)
@@ -464,12 +678,11 @@
         #priority = ':'.join(("NONE", "+VERS-TLS1.1", "+AES-256-CBC",
         #                "+SHA1", "+COMP-NULL", "+CTYPE-OPENPGP",
         #                "+DHE-DSS"))
-        priority = "NORMAL"             # Fallback default, since this
-                                        # MUST be set.
-        if self.server.settings["priority"]:
-            priority = self.server.settings["priority"]
-        gnutls.library.functions.gnutls_priority_set_direct\
-            (session._c_object, priority, None);
+        # Use a fallback default, since this MUST be set.
+        priority = self.server.settings.get("priority", "NORMAL")
+        (gnutls.library.functions
+         .gnutls_priority_set_direct(session._c_object,
+                                     priority, None))
         
         try:
             session.handshake()
@@ -478,6 +691,7 @@
             # Do not run session.bye() here: the session is not
             # established.  Just abandon the request.
             return
+        logger.debug(u"Handshake succeeded")
         try:
             fpr = fingerprint(peer_certificate(session))
         except (TypeError, gnutls.errors.GNUTLSError), error:
@@ -485,12 +699,11 @@
             session.bye()
             return
         logger.debug(u"Fingerprint: %s", fpr)
-        client = None
         for c in self.server.clients:
             if c.fingerprint == fpr:
                 client = c
                 break
-        if not client:
+        else:
             logger.warning(u"Client not found for fingerprint: %s",
                            fpr)
             session.bye()
@@ -503,6 +716,8 @@
                            vars(client))
             session.bye()
             return
+        ## This won't work here, since we're in a fork.
+        # client.checked_ok()
         sent_size = 0
         while sent_size < len(client.secret):
             sent = session.send(client.secret[sent_size:])
@@ -513,7 +728,8 @@
         session.bye()
 
 
-class IPv6_TCPServer(SocketServer.ForkingTCPServer, object):
+class IPv6_TCPServer(SocketServer.ForkingMixIn,
+                     SocketServer.TCPServer, object):
     """IPv6 TCP server.  Accepts 'None' as address and/or port.
     Attributes:
         settings:       Server settings
@@ -529,7 +745,7 @@
             self.clients = kwargs["clients"]
             del kwargs["clients"]
         self.enabled = False
-        return super(type(self), self).__init__(*args, **kwargs)
+        super(IPv6_TCPServer, self).__init__(*args, **kwargs)
     def server_bind(self):
         """This overrides the normal server_bind() function
         to bind to an interface if one was specified, and also NOT to
@@ -564,10 +780,10 @@
 #                                            if_nametoindex
 #                                            (self.settings
 #                                             ["interface"]))
-            return super(type(self), self).server_bind()
+            return super(IPv6_TCPServer, self).server_bind()
     def server_activate(self):
         if self.enabled:
-            return super(type(self), self).server_activate()
+            return super(IPv6_TCPServer, self).server_activate()
     def enable(self):
         self.enabled = True
 
@@ -591,8 +807,8 @@
     timevalue = datetime.timedelta(0)
     for s in interval.split():
         try:
-            suffix=unicode(s[-1])
-            value=int(s[:-1])
+            suffix = unicode(s[-1])
+            value = int(s[:-1])
             if suffix == u"d":
                 delta = datetime.timedelta(value)
             elif suffix == u"s":
@@ -632,16 +848,15 @@
     elif state == avahi.ENTRY_GROUP_FAILURE:
         logger.critical(u"Avahi: Error in group state changed %s",
                         unicode(error))
-        raise AvahiGroupError("State changed: %s", str(error))
+        raise AvahiGroupError(u"State changed: %s" % unicode(error))
 
 def if_nametoindex(interface):
     """Call the C function if_nametoindex(), or equivalent"""
     global if_nametoindex
     try:
-        if "ctypes.util" not in sys.modules:
-            import ctypes.util
-        if_nametoindex = ctypes.cdll.LoadLibrary\
-            (ctypes.util.find_library("c")).if_nametoindex
+        if_nametoindex = (ctypes.cdll.LoadLibrary
+                          (ctypes.util.find_library("c"))
+                          .if_nametoindex)
     except (OSError, AttributeError):
         if "struct" not in sys.modules:
             import struct
@@ -650,10 +865,9 @@
         def if_nametoindex(interface):
             "Get an interface index the hard way, i.e. using fcntl()"
             SIOCGIFINDEX = 0x8933  # From /usr/include/linux/sockios.h
-            s = socket.socket()
-            ifreq = fcntl.ioctl(s, SIOCGIFINDEX,
-                                struct.pack("16s16x", interface))
-            s.close()
+            with closing(socket.socket()) as s:
+                ifreq = fcntl.ioctl(s, SIOCGIFINDEX,
+                                    struct.pack("16s16x", interface))
             interface_index = struct.unpack("I", ifreq[16:20])[0]
             return interface_index
     return if_nametoindex(interface)
@@ -683,17 +897,14 @@
 
 
 def main():
-    global main_loop_started
-    main_loop_started = False
-    
-    parser = OptionParser(version = "%%prog %s" % version)
+    parser = optparse.OptionParser(version = "%%prog %s" % version)
     parser.add_option("-i", "--interface", type="string",
                       metavar="IF", help="Bind to interface IF")
     parser.add_option("-a", "--address", type="string",
                       help="Address to listen for requests on")
     parser.add_option("-p", "--port", type="int",
                       help="Port number to receive requests on")
-    parser.add_option("--check", action="store_true", default=False,
+    parser.add_option("--check", action="store_true",
                       help="Run self-test")
     parser.add_option("--debug", action="store_true",
                       help="Debug mode; run in foreground and log to"
@@ -706,7 +917,11 @@
                       default="/etc/mandos", metavar="DIR",
                       help="Directory to search for configuration"
                       " files")
-    (options, args) = parser.parse_args()
+    parser.add_option("--no-dbus", action="store_false",
+                      dest="use_dbus",
+                      help="Do not provide D-Bus system bus"
+                      " interface")
+    options = parser.parse_args()[0]
     
     if options.check:
         import doctest
@@ -721,6 +936,7 @@
                         "priority":
                         "SECURE256:!CTYPE-X.509:+CTYPE-OPENPGP",
                         "servicename": "Mandos",
+                        "use_dbus": "True",
                         }
     
     # Parse config file for server-global settings
@@ -729,29 +945,37 @@
     server_config.read(os.path.join(options.configdir, "mandos.conf"))
     # Convert the SafeConfigParser object to a dict
     server_settings = server_config.defaults()
-    # Use getboolean on the boolean config option
-    server_settings["debug"] = server_config.getboolean\
-                               ("DEFAULT", "debug")
+    # Use the appropriate methods on the non-string config options
+    server_settings["debug"] = server_config.getboolean("DEFAULT",
+                                                        "debug")
+    server_settings["use_dbus"] = server_config.getboolean("DEFAULT",
+                                                           "use_dbus")
+    if server_settings["port"]:
+        server_settings["port"] = server_config.getint("DEFAULT",
+                                                       "port")
     del server_config
     
     # Override the settings from the config file with command line
     # options, if set.
     for option in ("interface", "address", "port", "debug",
-                   "priority", "servicename", "configdir"):
+                   "priority", "servicename", "configdir",
+                   "use_dbus"):
         value = getattr(options, option)
         if value is not None:
             server_settings[option] = value
     del options
     # Now we have our good server settings in "server_settings"
     
+    # For convenience
     debug = server_settings["debug"]
+    use_dbus = server_settings["use_dbus"]
     
     if not debug:
         syslogger.setLevel(logging.WARNING)
         console.setLevel(logging.WARNING)
     
     if server_settings["servicename"] != "Mandos":
-        syslogger.setFormatter(logging.Formatter\
+        syslogger.setFormatter(logging.Formatter
                                ('Mandos (%s): %%(levelname)s:'
                                 ' %%(message)s'
                                 % server_settings["servicename"]))
@@ -759,7 +983,7 @@
     # Parse config file with clients
     client_defaults = { "timeout": "1h",
                         "interval": "5m",
-                        "checker": "fping -q -- %(host)s",
+                        "checker": "fping -q -- %%(host)s",
                         "host": "",
                         }
     client_config = ConfigParser.SafeConfigParser(client_defaults)
@@ -769,7 +993,7 @@
     clients = Set()
     tcp_server = IPv6_TCPServer((server_settings["address"],
                                  server_settings["port"]),
-                                tcp_handler,
+                                TCP_handler,
                                 settings=server_settings,
                                 clients=clients)
     pidfilename = "/var/run/mandos.pid"
@@ -778,22 +1002,20 @@
     except IOError, error:
         logger.error("Could not open file %r", pidfilename)
     
-    uid = 65534
-    gid = 65534
-    try:
-        uid = pwd.getpwnam("mandos").pw_uid
-    except KeyError:
-        try:
-            uid = pwd.getpwnam("nobody").pw_uid
-        except KeyError:
-            pass
-    try:
-        gid = pwd.getpwnam("mandos").pw_gid
-    except KeyError:
-        try:
-            gid = pwd.getpwnam("nogroup").pw_gid
-        except KeyError:
-            pass
+    try:
+        uid = pwd.getpwnam("_mandos").pw_uid
+        gid = pwd.getpwnam("_mandos").pw_gid
+    except KeyError:
+        try:
+            uid = pwd.getpwnam("mandos").pw_uid
+            gid = pwd.getpwnam("mandos").pw_gid
+        except KeyError:
+            try:
+                uid = pwd.getpwnam("nobody").pw_uid
+                gid = pwd.getpwnam("nogroup").pw_gid
+            except KeyError:
+                uid = 65534
+                gid = 65534
     try:
         os.setuid(uid)
         os.setgid(gid)
@@ -803,10 +1025,10 @@
     
     global service
     service = AvahiService(name = server_settings["servicename"],
-                           type = "_mandos._tcp", );
+                           servicetype = "_mandos._tcp", )
     if server_settings["interface"]:
-        service.interface = if_nametoindex\
-                            (server_settings["interface"])
+        service.interface = (if_nametoindex
+                             (server_settings["interface"]))
     
     global main_loop
     global bus
@@ -819,21 +1041,16 @@
                                            avahi.DBUS_PATH_SERVER),
                             avahi.DBUS_INTERFACE_SERVER)
     # End of Avahi example code
-    
-    def remove_from_clients(client):
-        clients.remove(client)
-        if not clients:
-            logger.critical(u"No clients left, exiting")
-            sys.exit()
+    if use_dbus:
+        bus_name = dbus.service.BusName(u"se.bsnet.fukt.Mandos", bus)
     
     clients.update(Set(Client(name = section,
-                              stop_hook = remove_from_clients,
                               config
-                              = dict(client_config.items(section)))
+                              = dict(client_config.items(section)),
+                              use_dbus = use_dbus)
                        for section in client_config.sections()))
     if not clients:
-        logger.critical(u"No clients defined")
-        sys.exit(1)
+        logger.warning(u"No clients defined")
     
     if debug:
         # Redirect stdin so all checkers get /dev/null
@@ -852,7 +1069,7 @@
         pidfile.write(str(pid) + "\n")
         pidfile.close()
         del pidfile
-    except IOError, err:
+    except IOError:
         logger.error(u"Could not write to file %r with PID %d",
                      pidfilename, pid)
     except NameError:
@@ -871,8 +1088,8 @@
         
         while clients:
             client = clients.pop()
-            client.stop_hook = None
-            client.stop()
+            client.disable_hook = None
+            client.disable()
     
     atexit.register(cleanup)
     
@@ -881,8 +1098,60 @@
     signal.signal(signal.SIGHUP, lambda signum, frame: sys.exit())
     signal.signal(signal.SIGTERM, lambda signum, frame: sys.exit())
     
+    if use_dbus:
+        class MandosServer(dbus.service.Object):
+            """A D-Bus proxy object"""
+            def __init__(self):
+                dbus.service.Object.__init__(self, bus, "/")
+            _interface = u"se.bsnet.fukt.Mandos"
+            
+            @dbus.service.signal(_interface, signature="oa{sv}")
+            def ClientAdded(self, objpath, properties):
+                "D-Bus signal"
+                pass
+            
+            @dbus.service.signal(_interface, signature="os")
+            def ClientRemoved(self, objpath, name):
+                "D-Bus signal"
+                pass
+            
+            @dbus.service.method(_interface, out_signature="ao")
+            def GetAllClients(self):
+                "D-Bus method"
+                return dbus.Array(c.dbus_object_path for c in clients)
+            
+            @dbus.service.method(_interface, out_signature="a{oa{sv}}")
+            def GetAllClientsWithProperties(self):
+                "D-Bus method"
+                return dbus.Dictionary(
+                    ((c.dbus_object_path, c.GetAllProperties())
+                     for c in clients),
+                    signature="oa{sv}")
+            
+            @dbus.service.method(_interface, in_signature="o")
+            def RemoveClient(self, object_path):
+                "D-Bus method"
+                for c in clients:
+                    if c.dbus_object_path == object_path:
+                        clients.remove(c)
+                        # Don't signal anything except ClientRemoved
+                        c.use_dbus = False
+                        c.disable()
+                        # Emit D-Bus signal
+                        self.ClientRemoved(object_path, c.name)
+                        return
+                raise KeyError
+            
+            del _interface
+        
+        mandos_server = MandosServer()
+    
     for client in clients:
-        client.start()
+        if use_dbus:
+            # Emit D-Bus signal
+            mandos_server.ClientAdded(client.dbus_object_path,
+                                      client.GetAllProperties())
+        client.enable()
     
     tcp_server.enable()
     tcp_server.server_activate()
@@ -906,14 +1175,13 @@
         
         gobject.io_add_watch(tcp_server.fileno(), gobject.IO_IN,
                              lambda *args, **kwargs:
-                             tcp_server.handle_request\
-                             (*args[2:], **kwargs) or True)
+                             (tcp_server.handle_request
+                              (*args[2:], **kwargs) or True))
         
         logger.debug(u"Starting main loop")
-        main_loop_started = True
         main_loop.run()
     except AvahiError, error:
-        logger.critical(u"AvahiError: %s" + unicode(error))
+        logger.critical(u"AvahiError: %s", error)
         sys.exit(1)
     except KeyboardInterrupt:
         if debug:

=== modified file 'mandos-clients.conf.xml'
--- mandos-clients.conf.xml	2008-09-12 19:12:40 +0000
+++ mandos-clients.conf.xml	2009-01-08 03:54:06 +0000
@@ -1,10 +1,11 @@
 <?xml version="1.0" encoding="UTF-8"?>
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
-<!ENTITY VERSION "1.0">
 <!ENTITY CONFNAME "mandos-clients.conf">
 <!ENTITY CONFPATH "<filename>/etc/mandos/clients.conf</filename>">
-<!ENTITY TIMESTAMP "2008-09-12">
+<!ENTITY TIMESTAMP "2009-01-08">
+<!ENTITY % common SYSTEM "common.ent">
+%common;
 ]>
 
 <refentry xmlns:xi="http://www.w3.org/2001/XInclude">
@@ -12,7 +13,7 @@
     <title>Mandos Manual</title>
     <!-- NWalsh’s docbook scripts use this to generate the footer: -->
     <productname>Mandos</productname>
-    <productnumber>&VERSION;</productnumber>
+    <productnumber>&version;</productnumber>
     <date>&TIMESTAMP;</date>
     <authorgroup>
       <author>
@@ -32,6 +33,7 @@
     </authorgroup>
     <copyright>
       <year>2008</year>
+      <year>2009</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>
@@ -168,7 +170,7 @@
 	    <varname>PATH</varname> will be searched.  The default
 	    value for the checker command is <quote><literal
 	    ><command>fping</command> <option>-q</option> <option
-	    >--</option> %(host)s</literal></quote>.
+	    >--</option> %%(host)s</literal></quote>.
 	  </para>
 	  <para>
 	    In addition to normal start time expansion, this option
@@ -239,6 +241,11 @@
 	    should <emphasis>not</emphasis> be base64-encoded, but
 	    will be sent to clients verbatim.
 	  </para>
+	  <para>
+	    File names of the form <filename>~user/foo/bar</filename>
+	    and <filename>$<envar>ENVVAR</envar>/foo/bar</filename>
+	    are supported.
+	  </para>
 	</listitem>
       </varlistentry>
       
@@ -344,7 +351,7 @@
 [DEFAULT]
 timeout = 1h
 interval = 5m
-checker = fping -q -- %(host)s
+checker = fping -q -- %%(host)s
 
 # Client "foo"
 [foo]

=== added file 'mandos-ctl'
--- mandos-ctl	1970-01-01 00:00:00 +0000
+++ mandos-ctl	2009-01-17 02:12:05 +0000
@@ -0,0 +1,187 @@
+#!/usr/bin/python
+# -*- mode: python; coding: utf-8 -*-
+
+from __future__ import division
+import sys
+import dbus
+from optparse import OptionParser
+import locale
+import datetime
+import re
+
+locale.setlocale(locale.LC_ALL, u'')
+
+tablewords = {
+    'name': u'Name',
+    'enabled': u'Enabled',
+    'timeout': u'Timeout',
+    'last_checked_ok': u'Last Successful Check',
+    'created': u'Created',
+    'interval': u'Interval',
+    'host': u'Host',
+    'fingerprint': u'Fingerprint',
+    'checker_running': u'Check Is Running',
+    'last_enabled': u'Last Enabled',
+    'checker': u'Checker',
+    }
+defaultkeywords = ('name', 'enabled', 'timeout', 'last_checked_ok',
+                   'checker')
+busname = 'org.mandos-system.Mandos'
+server_path = '/Mandos'
+server_interface = 'org.mandos_system.Mandos'
+client_interface = 'org.mandos_system.Mandos.Client'
+version = "1.0.5"
+
+bus = dbus.SystemBus()
+mandos_dbus_objc = bus.get_object(busname, server_path)
+mandos_serv = dbus.Interface(mandos_dbus_objc,
+                             dbus_interface = server_interface)
+mandos_clients = mandos_serv.GetAllClientsWithProperties()
+
+def datetime_to_milliseconds(dt):
+    "Return the 'timeout' attribute in milliseconds"
+    return ((dt.days * 24 * 60 * 60 * 1000)
+            + (dt.seconds * 1000)
+            + (dt.microseconds // 1000))
+
+def milliseconds_to_string(ms):
+    td = datetime.timedelta(0, 0, 0, ms)
+    return "%s%02d:%02d:%02d" % (("%dT" % td.days) if td.days else "", # days
+                           td.seconds // 3600,        # hours
+                           (td.seconds % 3600) // 60, # minutes
+                           (td.seconds % 60))         # seconds
+
+
+def string_to_delta(interval):
+    """Parse a string and return a datetime.timedelta
+
+    >>> string_to_delta('7d')
+    datetime.timedelta(7)
+    >>> string_to_delta('60s')
+    datetime.timedelta(0, 60)
+    >>> string_to_delta('60m')
+    datetime.timedelta(0, 3600)
+    >>> string_to_delta('24h')
+    datetime.timedelta(1)
+    >>> string_to_delta(u'1w')
+    datetime.timedelta(7)
+    >>> string_to_delta('5m 30s')
+    datetime.timedelta(0, 330)
+    """
+    timevalue = datetime.timedelta(0)
+    regexp = re.compile("\d+[dsmhw]")
+    
+    for s in regexp.findall(interval):
+        try:
+            suffix = unicode(s[-1])
+            value = int(s[:-1])
+            if suffix == u"d":
+                delta = datetime.timedelta(value)
+            elif suffix == u"s":
+                delta = datetime.timedelta(0, value)
+            elif suffix == u"m":
+                delta = datetime.timedelta(0, 0, 0, 0, value)
+            elif suffix == u"h":
+                delta = datetime.timedelta(0, 0, 0, 0, 0, value)
+            elif suffix == u"w":
+                delta = datetime.timedelta(0, 0, 0, 0, 0, 0, value)
+            else:
+                raise ValueError
+        except (ValueError, IndexError):
+            raise ValueError
+        timevalue += delta
+    return timevalue
+
+def print_clients(clients):
+    def valuetostring(value, keyword):
+        if type(value) is dbus.Boolean:
+            return u"Yes" if value else u"No"
+        if keyword in ("timeout", "interval"):
+            return milliseconds_to_string(value)
+        return unicode(value)
+    
+    format_string = u' '.join(u'%%-%ds' %
+                              max(len(tablewords[key]),
+                                  max(len(valuetostring(client[key], key))
+                                      for client in
+                                      clients))
+                              for key in keywords)
+    print format_string % tuple(tablewords[key] for key in keywords) 
+    for client in clients:
+        print format_string % tuple(valuetostring(client[key], key)
+                                    for key in keywords)
+
+parser = OptionParser(version = "%%prog %s" % version)
+parser.add_option("-a", "--all", action="store_true",
+                  help="Print all fields")
+parser.add_option("-e", "--enable", action="store_true",
+                  help="Enable specified client")
+parser.add_option("-d", "--disable", action="store_true",
+                  help="disable specified client")
+parser.add_option("-b", "--bump-timeout", action="store_true",
+                  help="Bump timeout of specified client")
+parser.add_option("--start-checker", action="store_true",
+                  help="Start checker for specified client")
+parser.add_option("--stop-checker", action="store_true",
+                  help="Stop checker for specified client")
+parser.add_option("-v", "--is-valid", action="store_true",
+                  help="Stop checker for specified client")
+parser.add_option("-c", "--checker", type="string",
+                  help="Set checker command for specified client")
+parser.add_option("-t", "--timeout", type="string",
+                  help="Set timeout for specified client")
+parser.add_option("-i", "--interval", type="string",
+                  help="Set checker interval for specified client")
+parser.add_option("-H", "--host", type="string",
+                  help="Set host for specified client")
+parser.add_option("-s", "--secret", type="string",
+                  help="Set password blob (file) for specified client")
+options, client_names = parser.parse_args()
+
+clients=[]
+for name in client_names:
+    for path, client in mandos_clients.iteritems():
+        if client['name'] == name:
+            client_objc = bus.get_object(busname, path)
+            clients.append(dbus.Interface(client_objc,
+                                          dbus_interface
+                                          = client_interface))
+            break
+    else:
+        print >> sys.stderr, "Client not found on server: %r" % name
+        sys.exit(1)
+
+if not clients:
+    keywords = defaultkeywords
+    if options.all:
+        keywords = ('name', 'enabled', 'timeout', 'last_checked_ok',
+                    'created', 'interval', 'host', 'fingerprint',
+                    'checker_running', 'last_enabled', 'checker')
+    print_clients(mandos_clients.values())
+    
+for client in clients:
+    if options.enable:
+        client.Enable()
+    if options.disable:
+        client.Disable()
+    if options.bump_timeout:
+        client.BumpTimeout()
+    if options.start_checker:
+        client.StartChecker()
+    if options.stop_checker:
+        client.StopChecker()
+    if options.is_valid:
+        sys.exit(0 if client.IsStillValid() else 1)
+    if options.checker:
+        client.SetChecker(options.checker)
+    if options.host:
+        client.SetHost(options.host)
+    if options.interval:
+        client.SetInterval(datetime_to_milliseconds
+                           (string_to_delta(options.interval)))
+    if options.timeout:
+        client.SetTimeout(datetime_to_milliseconds
+                          (string_to_delta(options.timeout)))
+    if options.secret:
+        client.SetSecret(dbus.ByteArray(open(options.secret, 'rb').read()))
+    

=== modified file 'mandos-keygen'
--- mandos-keygen	2008-09-08 12:03:16 +0000
+++ mandos-keygen	2009-01-17 09:46:54 +0000
@@ -2,7 +2,8 @@
 # 
 # Mandos key generator - create a new OpenPGP key for a Mandos client
 # 
-# Copyright © 2007-2008 Teddy Hogeborn & Björn Påhlsson
+# Copyright © 2008,2009 Teddy Hogeborn
+# Copyright © 2008,2009 Björn Påhlsson
 # 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,14 +21,14 @@
 # Contact the authors at <mandos@fukt.bsnet.se>.
 # 
 
-VERSION="1.0"
+VERSION="1.0.5"
 
 KEYDIR="/etc/keys/mandos"
 KEYTYPE=DSA
 KEYLENGTH=2048
 SUBKEYTYPE=ELG-E
 SUBKEYLENGTH=2048
-KEYNAME="`hostname --fqdn`"
+KEYNAME="`hostname --fqdn 2>/dev/null || hostname`"
 KEYEMAIL=""
 KEYCOMMENT="Mandos client key"
 KEYEXPIRE=0
@@ -40,8 +41,8 @@
 fi
 
 # Parse options
-TEMP=`getopt --options vhd:t:l:n:e:c:x:f \
-    --longoptions version,help,password,dir:,type:,length:,subtype:,sublength:,name:,email:,comment:,expire:,force \
+TEMP=`getopt --options vhpF:d:t:l:s:L:n:e:c:x:f \
+    --longoptions version,help,password,passfile:,dir:,type:,length:,subtype:,sublength:,name:,email:,comment:,expire:,force \
     --name "$0" -- "$@"`
 
 help(){
@@ -53,6 +54,7 @@
        $basename [ OPTIONS ]
    Encrypted password creation:
        $basename { -p | --password } [ --name NAME ] [ --dir DIR]
+       $basename { -F | --passfile } FILE [ --name NAME ] [ --dir DIR]
 
 Key creation options:
   -v, --version         Show program's version number and exit
@@ -74,10 +76,14 @@
   -x TIME, --expire TIME
                         Key expire time.  Default is no expiration.
                         See gpg(1) for syntax.
-  -f, --force           Force overwriting old keys.
+  -f, --force           Force overwriting old key files.
 
 Password creation options:
-  -p, --password        Create an encrypted password using the keys in
+  -p, --password        Create an encrypted password using the key in
+                        the key directory.  All options other than
+                        --dir and --name are ignored.
+  -F FILE, --passfile FILE
+                        Encrypt a password from FILE using the key in
                         the key directory.  All options other than
                         --dir and --name are ignored.
 EOF
@@ -87,6 +93,7 @@
 while :; do
     case "$1" in
 	-p|--password) mode=password; shift;;
+	-F|--passfile) mode=password; PASSFILE="$2"; shift 2;;
 	-d|--dir) KEYDIR="$2"; shift 2;;
 	-t|--type) KEYTYPE="$2"; shift 2;;
 	-s|--subtype) SUBKEYTYPE="$2"; shift 2;;
@@ -140,7 +147,7 @@
 	echo "Invalid key length" >&2
 	exit 1
     fi
-
+    
     if [ -z "$KEYEXPIRE" ]; then
 	echo "Empty key expiration" >&2
 	exit 1
@@ -165,7 +172,7 @@
     if [ -n "$KEYEMAIL" ]; then
 	KEYEMAILLINE="Name-Email: $KEYEMAIL"
     fi
-
+    
     # Create temporary gpg batch file
     BATCHFILE="`mktemp -t mandos-keygen-batch.XXXXXXXXXX`"
 fi
@@ -235,7 +242,7 @@
 	FILECOMMENT="$FILECOMMENT <$KEYEMAIL>"
     fi
     
-    # Export keys from key rings to key files
+    # Export key from key rings to key files
     gpg --quiet --batch --no-tty --no-options --enable-dsa2 \
 	--homedir "$RINGDIR" --armor --export-options export-minimal \
 	--comment "$FILECOMMENT" --output "$SECKEYFILE" \
@@ -246,7 +253,7 @@
 fi
 
 if [ "$mode" = password ]; then
-    # Import keys into temporary key rings
+    # Import key into temporary key rings
     gpg --quiet --batch --no-tty --no-options --enable-dsa2 \
 	--homedir "$RINGDIR" --trust-model always --armor \
 	--import "$SECKEYFILE"
@@ -265,22 +272,37 @@
     
     FILECOMMENT="Encrypted password for a Mandos client"
     
-    stty -echo
-    echo -n "Enter passphrase: " >&2
-    head --lines=1 | tr --delete '\n' \
-	| gpg --quiet --batch --no-tty --no-options --enable-dsa2 \
+    if [ -n "$PASSFILE" ]; then
+	cat "$PASSFILE"
+    else
+	stty -echo
+	echo -n "Enter passphrase: " >&2
+	first="$(head --lines=1 | tr --delete '\n')"
+	echo -n -e "\nRepeat passphrase: " >&2
+	second="$(head --lines=1 | tr --delete '\n')"
+	echo >&2
+	stty echo
+	if [ "$first" != "$second" ]; then
+	    echo -e "Passphrase mismatch" >&2
+	    touch "$RINGDIR"/mismatch
+	else
+	    echo -n "$first"
+	fi
+    fi | gpg --quiet --batch --no-tty --no-options --enable-dsa2 \
 	--homedir "$RINGDIR" --trust-model always --armor --encrypt \
-	--recipient "$FINGERPRINT" --comment "$FILECOMMENT" \
+	--sign --recipient "$FINGERPRINT" --comment "$FILECOMMENT" \
 	> "$SECFILE"
-    echo >&2
-    stty echo
+    if [ -e "$RINGDIR"/mismatch ]; then
+	rm --force "$RINGDIR"/mismatch
+	exit 1
+    fi
     
     cat <<-EOF
 	[$KEYNAME]
 	host = $KEYNAME
 	fingerprint = $FINGERPRINT
 	secret =
-EOF
+	EOF
     sed --quiet --expression='
 	/^-----BEGIN PGP MESSAGE-----$/,/^-----END PGP MESSAGE-----$/{
 	    /^$/,${

=== modified file 'mandos-keygen.xml'
--- mandos-keygen.xml	2008-09-12 19:12:40 +0000
+++ mandos-keygen.xml	2009-01-04 21:54:55 +0000
@@ -1,9 +1,10 @@
 <?xml version="1.0" encoding="UTF-8"?>
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
-<!ENTITY VERSION "1.0">
 <!ENTITY COMMANDNAME "mandos-keygen">
-<!ENTITY TIMESTAMP "2008-09-12">
+<!ENTITY TIMESTAMP "2009-01-04">
+<!ENTITY % common SYSTEM "common.ent">
+%common;
 ]>
 
 <refentry xmlns:xi="http://www.w3.org/2001/XInclude">
@@ -11,7 +12,7 @@
     <title>Mandos Manual</title>
     <!-- NWalsh’s docbook scripts use this to generate the footer: -->
     <productname>Mandos</productname>
-    <productnumber>&VERSION;</productnumber>
+    <productnumber>&version;</productnumber>
     <date>&TIMESTAMP;</date>
     <authorgroup>
       <author>
@@ -31,6 +32,7 @@
     </authorgroup>
     <copyright>
       <year>2008</year>
+      <year>2009</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>
@@ -122,6 +124,10 @@
       <group choice="req">
 	<arg choice="plain"><option>--password</option></arg>
 	<arg choice="plain"><option>-p</option></arg>
+	<arg choice="plain"><option>--passfile
+	<replaceable>FILE</replaceable></option></arg>
+	<arg choice="plain"><option>-F</option>
+	<replaceable>FILE</replaceable></arg>
       </group>
       <sbr/>
       <group>
@@ -167,8 +173,9 @@
     </para>
     <para>
       This program can also be used with the
-      <option>--password</option> option to generate a ready-made
-      section for <filename>clients.conf</filename> (see
+      <option>--password</option> or <option>--passfile</option>
+      options to generate a ready-made section for
+      <filename>clients.conf</filename> (see
       <citerefentry><refentrytitle>mandos-clients.conf</refentrytitle>
       <manvolnum>5</manvolnum></citerefentry>).
     </para>
@@ -326,6 +333,18 @@
 	  </para>
 	</listitem>
       </varlistentry>
+      <varlistentry>
+	<term><option>--passfile
+	<replaceable>FILE</replaceable></option></term>
+	<term><option>-F
+	<replaceable>FILE</replaceable></option></term>
+	<listitem>
+	  <para>
+	    The same as <option>--password</option>, but read from
+	    <replaceable>FILE</replaceable>, not the terminal.
+	  </para>
+	</listitem>
+      </varlistentry>
     </variablelist>
   </refsect1>
   
@@ -364,7 +383,7 @@
     </variablelist>
   </refsect1>
   
-  <refsect1 id="file">
+  <refsect1 id="files">
     <title>FILES</title>
     <para>
       Use the <option>--dir</option> option to change where

=== modified file 'mandos-options.xml'
--- mandos-options.xml	2008-09-06 16:31:49 +0000
+++ mandos-options.xml	2008-12-29 02:44:54 +0000
@@ -58,12 +58,18 @@
   <para id="servicename">
     Zeroconf service name.  The default is
     <quote><literal>Mandos</literal></quote>.  This only needs to be
-    changed this if it, for some reason, is necessary to run more than
-    one server on the same <emphasis>host</emphasis>, which would not
+    changed if for some reason is would be necessary to run more than
+    one server on the same <emphasis>host</emphasis>.  This would not
     normally be useful.  If there are name collisions on the same
     <emphasis>network</emphasis>, the newer server will automatically
     rename itself to <quote><literal>Mandos #2</literal></quote>, and
     so on; therefore, this option is not needed in that case.
   </para>
+
+  <para id="dbus">
+    This option controls whether the server will provide a D-Bus
+    system bus interface.  The default is to provide such an
+    interface.
+  </para>
   
 </section>

=== modified file 'mandos.conf'
--- mandos.conf	2008-08-18 23:55:28 +0000
+++ mandos.conf	2008-12-29 02:44:54 +0000
@@ -36,3 +36,6 @@
 # If there are name collisions on the same *network*, the server will
 # rename itself to "Mandos #2", etc.
 ;servicename = Mandos
+
+# Whether to provide a D-Bus system bus interface or not
+;use_dbus = True

=== modified file 'mandos.conf.xml'
--- mandos.conf.xml	2008-09-12 19:12:40 +0000
+++ mandos.conf.xml	2009-01-04 21:54:55 +0000
@@ -1,10 +1,11 @@
 <?xml version="1.0" encoding="UTF-8"?>
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
-<!ENTITY VERSION "1.0">
 <!ENTITY CONFNAME "mandos.conf">
 <!ENTITY CONFPATH "<filename>/etc/mandos/mandos.conf</filename>">
-<!ENTITY TIMESTAMP "2008-09-12">
+<!ENTITY TIMESTAMP "2009-01-04">
+<!ENTITY % common SYSTEM "common.ent">
+%common;
 ]>
 
 <refentry xmlns:xi="http://www.w3.org/2001/XInclude">
@@ -12,7 +13,7 @@
     <title>Mandos Manual</title>
     <!-- NWalsh’s docbook scripts use this to generate the footer: -->
     <productname>Mandos</productname>
-    <productnumber>&VERSION;</productnumber>
+    <productnumber>&version;</productnumber>
     <date>&TIMESTAMP;</date>
     <authorgroup>
       <author>
@@ -32,6 +33,7 @@
     </authorgroup>
     <copyright>
       <year>2008</year>
+      <year>2009</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>
@@ -129,6 +131,17 @@
 	</listitem>
       </varlistentry>
       
+      <varlistentry>
+	<term><option>use_dbus<literal> = </literal>{ <literal
+	  >1</literal> | <literal>yes</literal> | <literal
+	  >true</literal> | <literal>on</literal> | <literal
+	  >0</literal> | <literal>no</literal> | <literal
+	  >false</literal> | <literal>off</literal> }</option></term>
+	<listitem>
+	  <xi:include href="mandos-options.xml" xpointer="dbus"/>
+	</listitem>
+      </varlistentry>
+      
     </variablelist>
   </refsect1>
   
@@ -171,6 +184,7 @@
 debug = true
 priority = SECURE256:!CTYPE-X.509:+CTYPE-OPENPGP
 servicename = Daena
+use_dbus = False
       </programlisting>
     </informalexample>
   </refsect1>

=== added file 'mandos.lsm'
--- mandos.lsm	1970-01-01 00:00:00 +0000
+++ mandos.lsm	2009-01-17 02:12:05 +0000
@@ -0,0 +1,22 @@
+Begin4
+Title:		Mandos
+Version:	1.0.5
+Entered-date:	2009-01-17
+Description:	The Mandos system allows computers to have encrypted
+root file systems and at the same time be capable of remote and/or
+unattended reboots.
+Keywords:	boot, encryption, luks, cryptsetup, network, openpgp,
+tls, dm-crypt
+Author:		teddy@fukt.bsnet.se (Teddy Hogeborn),
+		belorn@fukt.bsnet.se (Björn Påhlsson)
+Maintained-by:	teddy@fukt.bsnet.se (Teddy Hogeborn),
+		belorn@fukt.bsnet.se (Björn Påhlsson)
+Primary-site: http://www.fukt.bsnet.se/mandos
+	      94K mandos_1.0.5.orig.tar.gz
+Alternate-site: ftp://ftp.fukt.bsnet.se/pub/mandos
+		94K mandos_1.0.5.orig.tar.gz
+Platforms:	Requires GCC, GNU libC, Avahi, GnuPG, Python 2.4, and
+various other libraries.  While made for Debian GNU/Linux, it is
+probably portable to other distributions, but not other Unixes.
+Copying-policy: GNU General Public License version 3.0 or later
+End

=== modified file 'mandos.xml'
--- mandos.xml	2008-09-12 19:12:40 +0000
+++ mandos.xml	2009-01-15 05:27:55 +0000
@@ -1,17 +1,18 @@
 <?xml version="1.0" encoding="UTF-8"?>
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 "http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
-<!ENTITY VERSION "1.0">
 <!ENTITY COMMANDNAME "mandos">
-<!ENTITY TIMESTAMP "2008-09-12">
+<!ENTITY TIMESTAMP "2009-01-04">
+<!ENTITY % common SYSTEM "common.ent">
+%common;
 ]>
 
 <refentry xmlns:xi="http://www.w3.org/2001/XInclude">
-  <refentryinfo>
+   <refentryinfo>
     <title>Mandos Manual</title>
     <!-- NWalsh’s docbook scripts use this to generate the footer: -->
     <productname>Mandos</productname>
-    <productnumber>&VERSION;</productnumber>
+    <productnumber>&version;</productnumber>
     <date>&TIMESTAMP;</date>
     <authorgroup>
       <author>
@@ -31,6 +32,7 @@
     </authorgroup>
     <copyright>
       <year>2008</year>
+      <year>2009</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>
@@ -83,6 +85,8 @@
       <replaceable>DIRECTORY</replaceable></option></arg>
       <sbr/>
       <arg><option>--debug</option></arg>
+      <sbr/>
+      <arg><option>--no-dbus</option></arg>
     </cmdsynopsis>
     <cmdsynopsis>
       <command>&COMMANDNAME;</command>
@@ -227,6 +231,16 @@
 	  </para>
 	</listitem>
       </varlistentry>
+      
+      <varlistentry>
+	<term><option>--no-dbus</option></term>
+	<listitem>
+	  <xi:include href="mandos-options.xml" xpointer="dbus"/>
+	  <para>
+	    See also <xref linkend="dbus_interface"/>.
+	  </para>
+	</listitem>
+      </varlistentry>
     </variablelist>
   </refsect1>
   
@@ -322,6 +336,16 @@
     </para>
   </refsect1>
   
+  <refsect1 id="dbus_interface">
+    <title>D-BUS INTERFACE</title>
+    <para>
+      The server will by default provide a D-Bus system bus interface.
+      This interface will only be accessible by the root user or a
+      Mandos-specific user, if such a user exists.
+      <!-- XXX -->
+    </para>
+  </refsect1>
+
   <refsect1 id="exit_status">
     <title>EXIT STATUS</title>
     <para>
@@ -350,7 +374,7 @@
     </variablelist>
   </refsect1>
   
-  <refsect1 id="file">
+  <refsect1 id="files">
     <title>FILES</title>
     <para>
       Use the <option>--configdir</option> option to change where
@@ -420,7 +444,7 @@
       Currently, if a client is declared <quote>invalid</quote> due to
       having timed out, the server does not record this fact onto
       permanent storage.  This has some security implications, see
-      <xref linkend="CLIENTS"/>.
+      <xref linkend="clients"/>.
     </para>
     <para>
       There is currently no way of querying the server of the current
@@ -482,7 +506,7 @@
   
   <refsect1 id="security">
     <title>SECURITY</title>
-    <refsect2 id="SERVER">
+    <refsect2 id="server">
       <title>SERVER</title>
       <para>
 	Running this <command>&COMMANDNAME;</command> server program
@@ -491,7 +515,7 @@
 	soon after startup.
       </para>
     </refsect2>
-    <refsect2 id="CLIENTS">
+    <refsect2 id="clients">
       <title>CLIENTS</title>
       <para>
 	The server only gives out its stored data to clients which
@@ -504,7 +528,7 @@
 	<citerefentry><refentrytitle>mandos-clients.conf</refentrytitle>
 	<manvolnum>5</manvolnum></citerefentry>)
 	<emphasis>must</emphasis> be made non-readable by anyone
-	except the user running the server.
+	except the user starting the server (usually root).
       </para>
       <para>
 	As detailed in <xref linkend="checking"/>, the status of all

=== modified file 'plugin-runner.c'
--- plugin-runner.c	2008-09-19 00:00:51 +0000
+++ plugin-runner.c	2009-01-17 02:34:57 +0000
@@ -2,7 +2,8 @@
 /*
  * Mandos plugin runner - Run Mandos plugins
  *
- * Copyright © 2007-2008 Teddy Hogeborn & Björn Påhlsson
+ * Copyright © 2008,2009 Teddy Hogeborn
+ * Copyright © 2008,2009 Björn Påhlsson
  * 
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -27,8 +28,8 @@
 #include <stdlib.h>		/* malloc(), exit(), EXIT_FAILURE,
 				   EXIT_SUCCESS, realloc() */
 #include <stdbool.h>		/* bool, true, false */
-#include <stdio.h>		/* perror, popen(), fileno(),
-				   fprintf(), stderr, STDOUT_FILENO */
+#include <stdio.h>		/* perror, fileno(), fprintf(),
+				   stderr, STDOUT_FILENO */
 #include <sys/types.h>	        /* DIR, opendir(), stat(), struct
 				   stat, waitpid(), WIFEXITED(),
 				   WEXITSTATUS(), wait(), pid_t,
@@ -46,7 +47,7 @@
 				   fcntl(), setuid(), setgid(),
 				   F_GETFD, F_SETFD, FD_CLOEXEC,
 				   access(), pipe(), fork(), close()
-				   dup2, STDOUT_FILENO, _exit(),
+				   dup2(), STDOUT_FILENO, _exit(),
 				   execv(), write(), read(),
 				   close() */
 #include <fcntl.h>		/* fcntl(), F_GETFD, F_SETFD,
@@ -63,13 +64,14 @@
 				   sigprocmask(), SIG_BLOCK, SIGCHLD,
 				   SIG_UNBLOCK, kill() */
 #include <errno.h>		/* errno, EBADF */
+#include <inttypes.h>		/* intmax_t, SCNdMAX, PRIdMAX,  */
 
 #define BUFFER_SIZE 256
 
 #define PDIR "/lib/mandos/plugins.d"
 #define AFILE "/conf/conf.d/mandos/plugin-runner.conf"
 
-const char *argp_program_version = "plugin-runner 1.0";
+const char *argp_program_version = "plugin-runner " VERSION;
 const char *argp_program_bug_address = "<mandos@fukt.bsnet.se>";
 
 typedef struct plugin{
@@ -98,21 +100,22 @@
    or if none is found, creates a new one */
 static plugin *getplugin(char *name){
   /* Check for exiting plugin with that name */
-  for (plugin *p = plugin_list; p != NULL; p = p->next){
-    if ((p->name == name)
-	or (p->name and name and (strcmp(p->name, name) == 0))){
+  for(plugin *p = plugin_list; p != NULL; p = p->next){
+    if((p->name == name)
+       or (p->name and name and (strcmp(p->name, name) == 0))){
       return p;
     }
   }
   /* Create a new plugin */
   plugin *new_plugin = malloc(sizeof(plugin));
-  if (new_plugin == NULL){
+  if(new_plugin == NULL){
     return NULL;
   }
   char *copy_name = NULL;
   if(name != NULL){
     copy_name = strdup(name);
     if(copy_name == NULL){
+      free(new_plugin);
       return NULL;
     }
   }
@@ -123,7 +126,7 @@
 			   .next = plugin_list };
   
   new_plugin->argv = malloc(sizeof(char *) * 2);
-  if (new_plugin->argv == NULL){
+  if(new_plugin->argv == NULL){
     free(copy_name);
     free(new_plugin);
     return NULL;
@@ -229,7 +232,7 @@
       break;
     }
     if(pid == -1){
-      if (errno != ECHILD){
+      if(errno != ECHILD){
 	perror("waitpid");
       }
       /* No child processes */
@@ -307,7 +310,9 @@
   struct dirent *dirst;
   struct stat st;
   fd_set rfds_all;
-  int ret, maxfd = 0;
+  int ret, numchars, maxfd = 0;
+  ssize_t sret;
+  intmax_t tmpmax;
   uid_t uid = 65534;
   gid_t gid = 65534;
   bool debug = false;
@@ -370,11 +375,11 @@
     { .name = NULL }
   };
   
-  error_t parse_opt (int key, char *arg, __attribute__((unused))
-		     struct argp_state *state) {
-    switch (key) {
+  error_t parse_opt(int key, char *arg, __attribute__((unused))
+		    struct argp_state *state) {
+    switch(key) {
     case 'g': 			/* --global-options */
-      if (arg != NULL){
+      if(arg != NULL){
 	char *p;
 	while((p = strsep(&arg, ",")) != NULL){
 	  if(p[0] == '\0'){
@@ -396,7 +401,7 @@
       }
       break;
     case 'o':			/* --options-for */
-      if (arg != NULL){
+      if(arg != NULL){
 	char *p_name = strsep(&arg, ":");
 	if(p_name[0] == '\0' or arg == NULL){
 	  break;
@@ -433,7 +438,7 @@
       }
       break;
     case 'd':			/* --disable */
-      if (arg != NULL){
+      if(arg != NULL){
 	plugin *p = getplugin(arg);
 	if(p == NULL){
 	  return ARGP_ERR_UNKNOWN;
@@ -442,7 +447,7 @@
       }
       break;
     case 'e':			/* --enable */
-      if (arg != NULL){
+      if(arg != NULL){
 	plugin *p = getplugin(arg);
 	if(p == NULL){
 	  return ARGP_ERR_UNKNOWN;
@@ -461,14 +466,32 @@
       /* This is already done by parse_opt_config_file() */
       break;
     case 130:			/* --userid */
-      uid = (uid_t)strtol(arg, NULL, 10);
+      ret = sscanf(arg, "%" SCNdMAX "%n", &tmpmax, &numchars);
+      if(ret < 1 or tmpmax != (uid_t)tmpmax
+	 or arg[numchars] != '\0'){
+	fprintf(stderr, "Bad user ID number: \"%s\", using %"
+		PRIdMAX "\n", arg, (intmax_t)uid);
+      } else {
+	uid = (uid_t)tmpmax;
+      }
       break;
     case 131:			/* --groupid */
-      gid = (gid_t)strtol(arg, NULL, 10);
+      ret = sscanf(arg, "%" SCNdMAX "%n", &tmpmax, &numchars);
+      if(ret < 1 or tmpmax != (gid_t)tmpmax
+	 or arg[numchars] != '\0'){
+	fprintf(stderr, "Bad group ID number: \"%s\", using %"
+		PRIdMAX "\n", arg, (intmax_t)gid);
+      } else {
+	gid = (gid_t)tmpmax;
+      }
       break;
     case 132:			/* --debug */
       debug = true;
       break;
+/*
+ * When adding more options before this line, remember to also add a
+ * "case" to the "parse_opt_config_file" function below.
+ */
     case ARGP_KEY_ARG:
       /* Cryptsetup always passes an argument, which is an empty
 	 string if "none" was specified in /etc/crypttab.  So if
@@ -487,10 +510,10 @@
   
   /* This option parser is the same as parse_opt() above, except it
      ignores everything but the --config-file option. */
-  error_t parse_opt_config_file (int key, char *arg,
-				 __attribute__((unused))
-				 struct argp_state *state) {
-    switch (key) {
+  error_t parse_opt_config_file(int key, char *arg,
+				__attribute__((unused))
+				struct argp_state *state) {
+    switch(key) {
     case 'g': 			/* --global-options */
     case 'G':			/* --global-env */
     case 'o':			/* --options-for */
@@ -523,10 +546,10 @@
 		       .args_doc = "",
 		       .doc = "Mandos plugin runner -- Run plugins" };
   
-  /* Parse using the parse_opt_config_file in order to get the custom
+  /* Parse using parse_opt_config_file() in order to get the custom
      config file location, if any. */
-  ret = argp_parse (&argp, argc, argv, ARGP_IN_ORDER, 0, NULL);
-  if (ret == ARGP_ERR_UNKNOWN){
+  ret = argp_parse(&argp, argc, argv, ARGP_IN_ORDER, 0, NULL);
+  if(ret == ARGP_ERR_UNKNOWN){
     fprintf(stderr, "Unknown error while parsing arguments\n");
     exitstatus = EXIT_FAILURE;
     goto fallback;
@@ -536,7 +559,7 @@
   argp.parser = parse_opt;
   
   /* Open the configfile if available */
-  if (argfile == NULL){
+  if(argfile == NULL){
     conffp = fopen(AFILE, "r");
   } else {
     conffp = fopen(argfile, "r");
@@ -545,7 +568,6 @@
     char *org_line = NULL;
     char *p, *arg, *new_arg, *line;
     size_t size = 0;
-    ssize_t sret;
     const char whitespace_delims[] = " \r\t\f\v\n";
     const char comment_delim[] = "#";
 
@@ -598,7 +620,7 @@
   } else {
     /* Check for harmful errors and go to fallback. Other errors might
        not affect opening plugins */
-    if (errno == EMFILE or errno == ENFILE or errno == ENOMEM){
+    if(errno == EMFILE or errno == ENFILE or errno == ENOMEM){
       perror("fopen");
       exitstatus = EXIT_FAILURE;
       goto fallback;
@@ -607,9 +629,9 @@
   /* If there was any arguments from configuration file,
      pass them to parser as command arguments */
   if(custom_argv != NULL){
-    ret = argp_parse (&argp, custom_argc, custom_argv, ARGP_IN_ORDER,
-		      0, NULL);
-    if (ret == ARGP_ERR_UNKNOWN){
+    ret = argp_parse(&argp, custom_argc, custom_argv, ARGP_IN_ORDER,
+		     0, NULL);
+    if(ret == ARGP_ERR_UNKNOWN){
       fprintf(stderr, "Unknown error while parsing arguments\n");
       exitstatus = EXIT_FAILURE;
       goto fallback;
@@ -618,8 +640,8 @@
   
   /* Parse actual command line arguments, to let them override the
      config file */
-  ret = argp_parse (&argp, argc, argv, ARGP_IN_ORDER, 0, NULL);
-  if (ret == ARGP_ERR_UNKNOWN){
+  ret = argp_parse(&argp, argc, argv, ARGP_IN_ORDER, 0, NULL);
+  if(ret == ARGP_ERR_UNKNOWN){
     fprintf(stderr, "Unknown error while parsing arguments\n");
     exitstatus = EXIT_FAILURE;
     goto fallback;
@@ -632,7 +654,7 @@
       for(char **a = p->argv; *a != NULL; a++){
 	fprintf(stderr, "\tArg: %s\n", *a);
       }
-      fprintf(stderr, "...and %u environment variables\n", p->envc);
+      fprintf(stderr, "...and %d environment variables\n", p->envc);
       for(char **a = p->environ; *a != NULL; a++){
 	fprintf(stderr, "\t%s\n", *a);
       }
@@ -641,15 +663,15 @@
   
   /* Strip permissions down to nobody */
   ret = setuid(uid);
-  if (ret == -1){
+  if(ret == -1){
     perror("setuid");
   }  
   setgid(gid);
-  if (ret == -1){
+  if(ret == -1){
     perror("setgid");
   }
   
-  if (plugindir == NULL){
+  if(plugindir == NULL){
     dir = opendir(PDIR);
   } else {
     dir = opendir(plugindir);
@@ -682,7 +704,7 @@
     
     /* All directory entries have been processed */
     if(dirst == NULL){
-      if (errno == EBADF){
+      if(errno == EBADF){
 	perror("readdir");
 	exitstatus = EXIT_FAILURE;
 	goto fallback;
@@ -700,6 +722,7 @@
       
       const char const *bad_suffixes[] = { "~", "#", ".dpkg-new",
 					   ".dpkg-old",
+					   ".dpkg-bak",
 					   ".dpkg-divert", NULL };
       for(const char **pre = bad_prefixes; *pre != NULL; pre++){
 	size_t pre_len = strlen(*pre);
@@ -747,14 +770,14 @@
     }
     
     ret = stat(filename, &st);
-    if (ret == -1){
+    if(ret == -1){
       perror("stat");
       free(filename);
       continue;
     }
 
     /* Ignore non-executable files */
-    if (not S_ISREG(st.st_mode)	or (access(filename, X_OK) != 0)){
+    if(not S_ISREG(st.st_mode) or (access(filename, X_OK) != 0)){
       if(debug){
 	fprintf(stderr, "Ignoring plugin dir entry \"%s\""
 		" with bad type or mode\n", filename);
@@ -807,7 +830,7 @@
     
     int pipefd[2];
     ret = pipe(pipefd);
-    if (ret == -1){
+    if(ret == -1){
       perror("pipe");
       exitstatus = EXIT_FAILURE;
       goto fallback;
@@ -826,7 +849,7 @@
       goto fallback;
     }
     /* Block SIGCHLD until process is safely in process list */
-    ret = sigprocmask (SIG_BLOCK, &sigchld_action.sa_mask, NULL);
+    ret = sigprocmask(SIG_BLOCK, &sigchld_action.sa_mask, NULL);
     if(ret < 0){
       perror("sigprocmask");
       exitstatus = EXIT_FAILURE;
@@ -846,12 +869,12 @@
 	perror("sigaction");
 	_exit(EXIT_FAILURE);
       }
-      ret = sigprocmask (SIG_UNBLOCK, &sigchld_action.sa_mask, NULL);
+      ret = sigprocmask(SIG_UNBLOCK, &sigchld_action.sa_mask, NULL);
       if(ret < 0){
 	perror("sigprocmask");
 	_exit(EXIT_FAILURE);
       }
-
+      
       ret = dup2(pipefd[1], STDOUT_FILENO); /* replace our stdout */
       if(ret == -1){
 	perror("dup2");
@@ -880,9 +903,9 @@
     close(pipefd[1]);		/* Close unused write end of pipe */
     free(filename);
     plugin *new_plugin = getplugin(dirst->d_name);
-    if (new_plugin == NULL){
+    if(new_plugin == NULL){
       perror("getplugin");
-      ret = sigprocmask (SIG_UNBLOCK, &sigchld_action.sa_mask, NULL);
+      ret = sigprocmask(SIG_UNBLOCK, &sigchld_action.sa_mask, NULL);
       if(ret < 0){
         perror("sigprocmask");
       }
@@ -895,7 +918,7 @@
     
     /* Unblock SIGCHLD so signal handler can be run if this process
        has already completed */
-    ret = sigprocmask (SIG_UNBLOCK, &sigchld_action.sa_mask, NULL);
+    ret = sigprocmask(SIG_UNBLOCK, &sigchld_action.sa_mask, NULL);
     if(ret < 0){
       perror("sigprocmask");
       exitstatus = EXIT_FAILURE;
@@ -904,15 +927,14 @@
     
     FD_SET(new_plugin->fd, &rfds_all);
     
-    if (maxfd < new_plugin->fd){
+    if(maxfd < new_plugin->fd){
       maxfd = new_plugin->fd;
     }
-    
   }
   
   closedir(dir);
   dir = NULL;
-
+  
   for(plugin *p = plugin_list; p != NULL; p = p->next){
     if(p->pid != 0){
       break;
@@ -923,12 +945,12 @@
       free_plugin_list();
     }
   }
-
+  
   /* Main loop while running plugins exist */
   while(plugin_list){
     fd_set rfds = rfds_all;
     int select_ret = select(maxfd+1, &rfds, NULL, NULL, NULL);
-    if (select_ret == -1){
+    if(select_ret == -1){
       perror("select");
       exitstatus = EXIT_FAILURE;
       goto fallback;
@@ -945,16 +967,16 @@
 
 	  if(debug){
 	    if(WIFEXITED(proc->status)){
-	      fprintf(stderr, "Plugin %u exited with status %d\n",
-		      (unsigned int) (proc->pid),
+	      fprintf(stderr, "Plugin %" PRIdMAX " exited with status"
+		      " %d\n", (intmax_t) (proc->pid),
 		      WEXITSTATUS(proc->status));
 	    } else if(WIFSIGNALED(proc->status)) {
-	      fprintf(stderr, "Plugin %u killed by signal %d\n",
-		      (unsigned int) (proc->pid),
+	      fprintf(stderr, "Plugin %" PRIdMAX " killed by signal"
+		      " %d\n", (intmax_t) (proc->pid),
 		      WTERMSIG(proc->status));
 	    } else if(WCOREDUMP(proc->status)){
-	      fprintf(stderr, "Plugin %d dumped core\n",
-		      (unsigned int) (proc->pid));
+	      fprintf(stderr, "Plugin %" PRIdMAX " dumped core\n",
+		      (intmax_t) (proc->pid));
 	    }
 	  }
 	  
@@ -969,9 +991,13 @@
 	    goto fallback;
 	  }
 	  
+	  plugin *next_plugin = proc->next;
+	  free_plugin(proc);
+	  proc = next_plugin;
+	  
 	  /* We are done modifying process list, so unblock signal */
-	  ret = sigprocmask (SIG_UNBLOCK, &sigchld_action.sa_mask,
-			     NULL);
+	  ret = sigprocmask(SIG_UNBLOCK, &sigchld_action.sa_mask,
+			    NULL);
 	  if(ret < 0){
 	    perror("sigprocmask");
 	    exitstatus = EXIT_FAILURE;
@@ -982,14 +1008,11 @@
 	    break;
 	  }
 	  
-	  plugin *next_plugin = proc->next;
-	  free_plugin(proc);
-	  proc = next_plugin;
 	  continue;
 	}
 	
 	/* This process exited nicely, so print its buffer */
-
+	
 	bool bret = print_out_password(proc->buffer,
 				       proc->buffer_length);
 	if(not bret){
@@ -1009,7 +1032,7 @@
       if(proc->buffer_length + BUFFER_SIZE > proc->buffer_size){
 	proc->buffer = realloc(proc->buffer, proc->buffer_size
 			       + (size_t) BUFFER_SIZE);
-	if (proc->buffer == NULL){
+	if(proc->buffer == NULL){
 	  perror("malloc");
 	  exitstatus = EXIT_FAILURE;
 	  goto fallback;
@@ -1017,18 +1040,18 @@
 	proc->buffer_size += BUFFER_SIZE;
       }
       /* Read from the process */
-      ret = read(proc->fd, proc->buffer + proc->buffer_length,
-		 BUFFER_SIZE);
-      if(ret < 0){
+      sret = read(proc->fd, proc->buffer + proc->buffer_length,
+		  BUFFER_SIZE);
+      if(sret < 0){
 	/* Read error from this process; ignore the error */
 	proc = proc->next;
 	continue;
       }
-      if(ret == 0){
+      if(sret == 0){
 	/* got EOF */
 	proc->eof = true;
       } else {
-	proc->buffer_length += (size_t) ret;
+	proc->buffer_length += (size_t) sret;
       }
     }
   }
@@ -1061,7 +1084,7 @@
     perror("sigaction");
     exitstatus = EXIT_FAILURE;
   }
-
+  
   if(custom_argv != NULL){
     for(char **arg = custom_argv+1; *arg != NULL; arg++){
       free(*arg);
@@ -1073,7 +1096,7 @@
     closedir(dir);
   }
   
-  /* Free the process list and kill the processes */
+  /* Kill the processes */
   for(plugin *p = plugin_list; p != NULL; p = p->next){
     if(p->pid != 0){
       close(p->fd);
@@ -1092,7 +1115,7 @@
   if(errno != ECHILD){
     perror("wait");
   }
-
+  
   free_plugin_list();
   
   free(plugindir);

=== modified file 'plugin-runner.conf'
--- plugin-runner.conf	2008-09-06 16:31:49 +0000
+++ plugin-runner.conf	2008-10-05 17:38:31 +0000
@@ -1,9 +1,11 @@
 ## This is the configuration file for plugin-runner.  It should be
 ## installed as "/etc/mandos/plugin-runner.conf", which will be copied
 ## to "/conf/conf.d/mandos/plugin-runner.conf" in the initrd.img file.
+##
+## After editing this file, the initrd image file must be updated for
+## the changes to take effect!
 ## 
 ## The default network interface for mandos-client(8mandos) is
 ## "eth0".  Uncomment this line and change it if necessary.
 ## 
-
 #--options-for=mandos-client:--interface=eth0

=== modified file 'plugin-runner.xml'
--- plugin-runner.xml	2008-09-19 00:00:51 +0000
+++ plugin-runner.xml	2009-01-18 06:41:57 +0000
@@ -1,9 +1,10 @@
 <?xml version="1.0" encoding="UTF-8"?>
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
-<!ENTITY VERSION "1.0">
 <!ENTITY COMMANDNAME "plugin-runner">
-<!ENTITY TIMESTAMP "2008-09-19">
+<!ENTITY TIMESTAMP "2009-01-17">
+<!ENTITY % common SYSTEM "common.ent">
+%common;
 ]>
 
 <refentry xmlns:xi="http://www.w3.org/2001/XInclude">
@@ -11,7 +12,7 @@
     <title>Mandos Manual</title>
     <!-- Nwalsh’s docbook scripts use this to generate the footer: -->
     <productname>Mandos</productname>
-    <productnumber>&VERSION;</productnumber>
+    <productnumber>&version;</productnumber>
     <date>&TIMESTAMP;</date>
     <authorgroup>
       <author>
@@ -31,6 +32,7 @@
     </authorgroup>
     <copyright>
       <year>2008</year>
+      <year>2009</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>
@@ -577,7 +579,7 @@
       <para>
 
 <!-- do not wrap this line -->
-<userinput>&COMMANDNAME;  --config-file=/etc/mandos/plugin-runner.conf --plugin-dir /usr/lib/mandos/plugins.d --options-for=mandos-client:--pubkey=/etc/keys/mandos/pubkey.txt,--seckey=/etc/keys/mandos/seckey.txt</userinput>
+<userinput>cd /etc/keys/mandos; &COMMANDNAME;  --config-file=/etc/mandos/plugin-runner.conf --plugin-dir /usr/lib/mandos/plugins.d --options-for=mandos-client:--pubkey=pubkey.txt,--seckey=seckey.txt</userinput>
 
       </para>
     </informalexample>

=== added file 'plugins.d/askpass-fifo.c'
--- plugins.d/askpass-fifo.c	1970-01-01 00:00:00 +0000
+++ plugins.d/askpass-fifo.c	2009-01-10 06:00:50 +0000
@@ -0,0 +1,105 @@
+/*  -*- coding: utf-8 -*- */
+/*
+ * Askpass-FIFO - Read a password from a FIFO and output it
+ * 
+ * Copyright © 2008,2009 Teddy Hogeborn
+ * Copyright © 2008,2009 Björn Påhlsson
+ * 
+ * This program is free software: you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see
+ * <http://www.gnu.org/licenses/>.
+ * 
+ * Contact the authors at <https://www.fukt.bsnet.se/~belorn/> and
+ * <https://www.fukt.bsnet.se/~teddy/>.
+ */
+
+#define _GNU_SOURCE		/* TEMP_FAILURE_RETRY() */
+#include <sys/types.h>		/* ssize_t */
+#include <sys/stat.h>		/* mkfifo(), S_IRUSR, S_IWUSR */
+#include <iso646.h>		/* and */
+#include <errno.h>		/* errno, EEXIST */
+#include <stdio.h>		/* perror() */
+#include <stdlib.h>		/* EXIT_FAILURE, NULL, size_t, free(), 
+				   realloc(), EXIT_SUCCESS */
+#include <fcntl.h>		/* open(), O_RDONLY */
+#include <unistd.h>		/* read(), close(), write(),
+				   STDOUT_FILENO */
+
+
+int main(__attribute__((unused))int argc,
+	 __attribute__((unused))char **argv){
+  int ret = 0;
+  ssize_t sret;
+  
+  /* Create FIFO */
+  const char passfifo[] = "/lib/cryptsetup/passfifo";
+  ret = (int)TEMP_FAILURE_RETRY(mkfifo(passfifo, S_IRUSR | S_IWUSR));
+  if(ret == -1 and errno != EEXIST){
+    perror("mkfifo");
+    return EXIT_FAILURE;
+  }
+  
+  /* Open FIFO */
+  int fifo_fd = (int)TEMP_FAILURE_RETRY(open(passfifo, O_RDONLY));
+  if(fifo_fd == -1){
+    perror("open");
+    return EXIT_FAILURE;
+  }
+  
+  /* Read from FIFO */
+  char *buf = NULL;
+  size_t buf_len = 0;
+  {
+    size_t buf_allocated = 0;
+    const size_t blocksize = 1024;
+    do{
+      if(buf_len + blocksize > buf_allocated){
+	char *tmp = realloc(buf, buf_allocated + blocksize);
+	if(tmp == NULL){
+	  perror("realloc");
+	  free(buf);
+	  return EXIT_FAILURE;
+	}
+	buf = tmp;
+	buf_allocated += blocksize;
+      }
+      sret = TEMP_FAILURE_RETRY(read(fifo_fd, buf + buf_len,
+				     buf_allocated - buf_len));
+      if(sret == -1){
+	perror("read");
+	free(buf);
+	return EXIT_FAILURE;
+      }
+      buf_len += (size_t)sret;
+    }while(sret != 0);
+  }
+  
+  /* Close FIFO */
+  TEMP_FAILURE_RETRY(close(fifo_fd));
+  
+  /* Print password to stdout */
+  size_t written = 0;
+  while(written < buf_len){
+    sret = TEMP_FAILURE_RETRY(write(STDOUT_FILENO, buf + written,
+				    buf_len - written));
+    if(sret == -1){
+      perror("write");
+      free(buf);
+      return EXIT_FAILURE;
+    }
+    written += (size_t)sret;
+  }
+  free(buf);
+  
+  return EXIT_SUCCESS;
+}

=== added file 'plugins.d/askpass-fifo.xml'
--- plugins.d/askpass-fifo.xml	1970-01-01 00:00:00 +0000
+++ plugins.d/askpass-fifo.xml	2009-01-04 21:54:55 +0000
@@ -0,0 +1,162 @@
+<?xml version="1.0" encoding="UTF-8"?>
+<!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
+	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
+<!ENTITY COMMANDNAME "askpass-fifo">
+<!ENTITY TIMESTAMP "2009-01-04">
+<!ENTITY % common SYSTEM "../common.ent">
+%common;
+]>
+
+<refentry xmlns:xi="http://www.w3.org/2001/XInclude">
+  <refentryinfo>
+    <title>Mandos Manual</title>
+    <!-- NWalsh’s docbook scripts use this to generate the footer: -->
+    <productname>Mandos</productname>
+    <productnumber>&version;</productnumber>
+    <date>&TIMESTAMP;</date>
+    <authorgroup>
+      <author>
+	<firstname>Björn</firstname>
+	<surname>Påhlsson</surname>
+	<address>
+	  <email>belorn@fukt.bsnet.se</email>
+	</address>
+      </author>
+      <author>
+	<firstname>Teddy</firstname>
+	<surname>Hogeborn</surname>
+	<address>
+	  <email>teddy@fukt.bsnet.se</email>
+	</address>
+      </author>
+    </authorgroup>
+    <copyright>
+      <year>2008</year>
+      <year>2009</year>
+      <holder>Teddy Hogeborn</holder>
+      <holder>Björn Påhlsson</holder>
+    </copyright>
+    <xi:include href="../legalnotice.xml"/>
+  </refentryinfo>
+  
+  <refmeta>
+    <refentrytitle>&COMMANDNAME;</refentrytitle>
+    <manvolnum>8mandos</manvolnum>
+  </refmeta>
+  
+  <refnamediv>
+    <refname><command>&COMMANDNAME;</command></refname>
+    <refpurpose>Mandos plugin to get a password from a
+    FIFO.</refpurpose>
+  </refnamediv>
+  
+  <refsynopsisdiv>
+    <cmdsynopsis>
+      <command>&COMMANDNAME;</command>
+    </cmdsynopsis>
+  </refsynopsisdiv>
+  
+  <refsect1 id="description">
+    <title>DESCRIPTION</title>
+    <para>
+      This program reads a password from a <acronym>FIFO</acronym> and
+      outputs it to standard output.
+    </para>
+    <para>
+      This program is not very useful on its own.  This program is
+      really meant to run as a plugin in the <application
+      >Mandos</application> client-side system, where it is used as a
+      fallback and alternative to retrieving passwords from a
+      <application >Mandos</application> server.
+    </para>
+    <para>
+      This program is meant to be imitate a feature of the
+      <command>askpass</command> program, so that programs written to
+      interface with it can keep working under the
+      <application>Mandos</application> system.
+    </para>
+  </refsect1>
+  
+  <refsect1 id="options">
+    <title>OPTIONS</title>
+    <para>
+      This program takes no options.
+    </para>
+  </refsect1>
+  
+  <refsect1 id="exit_status">
+    <title>EXIT STATUS</title>
+    <para>
+      If exit status is 0, the output from the program is the password
+      as it was read.  Otherwise, if exit status is other than 0, the
+      program was interrupted or encountered an error, and any output
+      so far could be corrupt and/or truncated, and should therefore
+      be ignored.
+    </para>
+  </refsect1>
+  
+  <refsect1 id="files">
+    <title>FILES</title>
+    <variablelist>
+      <varlistentry>
+	<term><filename>/lib/cryptsetup/passfifo</filename></term>
+	<listitem>
+	  <para>
+	    This is the <acronym>FIFO</acronym> where this program
+	    will read the password.  If it does not exist, it will be
+	    created.
+	  </para>
+	</listitem>
+      </varlistentry>
+    </variablelist>
+  </refsect1>
+  
+  <refsect1 id="example">
+    <title>EXAMPLE</title>
+    <para>
+      Note that normally, this program will not be invoked directly,
+      but instead started by the Mandos <citerefentry><refentrytitle
+      >plugin-runner</refentrytitle><manvolnum>8mandos</manvolnum>
+      </citerefentry>.
+    </para>
+    <informalexample>
+      <para>
+	This program takes no options.
+      </para>
+      <para>
+	<userinput>&COMMANDNAME;</userinput>
+      </para>
+    </informalexample>
+  </refsect1>
+  
+  <refsect1 id="security">
+    <title>SECURITY</title>
+    <para>
+      The only thing that could be considered worthy of note is
+      this:  This program is meant to be run by <citerefentry>
+      <refentrytitle>plugin-runner</refentrytitle><manvolnum
+      >8mandos</manvolnum></citerefentry>, and will, when run
+      standalone, outside, in a normal environment, immediately output
+      on its standard output any presumably secret password it just
+      received.  Therefore, when running this program standalone
+      (which should never normally be done), take care not to type in
+      any real secret password by force of habit, since it would then
+      immediately be shown as output.
+    </para>
+  </refsect1>
+  
+  <refsect1 id="see_also">
+    <title>SEE ALSO</title>
+    <para>
+      <citerefentry><refentrytitle>fifo</refentrytitle>
+      <manvolnum>7</manvolnum></citerefentry>,
+      <citerefentry><refentrytitle>plugin-runner</refentrytitle>
+      <manvolnum>8mandos</manvolnum></citerefentry>
+    </para>
+  </refsect1>
+</refentry>
+<!-- Local Variables: -->
+<!-- time-stamp-start: "<!ENTITY TIMESTAMP [\"']" -->
+<!-- time-stamp-end: "[\"']>" -->
+<!-- time-stamp-format: "%:y-%02m-%02d" -->
+<!-- End: -->

=== modified file 'plugins.d/mandos-client.c'
--- plugins.d/mandos-client.c	2008-09-07 01:44:44 +0000
+++ plugins.d/mandos-client.c	2009-01-27 15:42:41 +0000
@@ -1,6 +1,6 @@
 /*  -*- coding: utf-8 -*- */
 /*
- * Mandos client - get and decrypt data from a Mandos server
+ * Mandos-client - get and decrypt data from a Mandos server
  *
  * This program is partly derived from an example program for an Avahi
  * service browser, downloaded from
@@ -9,7 +9,8 @@
  * "browse_callback", and parts of "main".
  * 
  * Everything else is
- * Copyright © 2007-2008 Teddy Hogeborn & Björn Påhlsson
+ * Copyright © 2008,2009 Teddy Hogeborn
+ * Copyright © 2008,2009 Björn Påhlsson
  * 
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -35,7 +36,8 @@
 #define _GNU_SOURCE		/* TEMP_FAILURE_RETRY(), asprintf() */
 
 #include <stdio.h>		/* fprintf(), stderr, fwrite(),
-				   stdout, ferror() */
+				   stdout, ferror(), sscanf(),
+				   remove() */
 #include <stdint.h> 		/* uint16_t, uint32_t */
 #include <stddef.h>		/* NULL, size_t, ssize_t */
 #include <stdlib.h> 		/* free(), EXIT_SUCCESS, EXIT_FAILURE,
@@ -47,14 +49,16 @@
 #include <sys/types.h>		/* socket(), inet_pton(), sockaddr,
 				   sockaddr_in6, PF_INET6,
 				   SOCK_STREAM, INET6_ADDRSTRLEN,
-				   uid_t, gid_t, open(), opendir(), DIR */
+				   uid_t, gid_t, open(), opendir(),
+				   DIR */
 #include <sys/stat.h>		/* open() */
 #include <sys/socket.h>		/* socket(), struct sockaddr_in6,
 				   struct in6_addr, inet_pton(),
 				   connect() */
 #include <fcntl.h>		/* open() */
-#include <dirent.h>		/* opendir(), struct dirent, readdir() */
-#include <inttypes.h>		/* PRIu16 */
+#include <dirent.h>		/* opendir(), struct dirent, readdir()
+				 */
+#include <inttypes.h>		/* PRIu16, intmax_t, SCNdMAX */
 #include <assert.h>		/* assert() */
 #include <errno.h>		/* perror(), errno */
 #include <time.h>		/* time() */
@@ -66,7 +70,7 @@
 				   getuid(), getgid(), setuid(),
 				   setgid() */
 #include <arpa/inet.h>		/* inet_pton(), htons */
-#include <iso646.h>		/* not, and */
+#include <iso646.h>		/* not, and, or */
 #include <argp.h>		/* struct argp_option, error_t, struct
 				   argp_state, struct argp,
 				   argp_parse(), ARGP_KEY_ARG,
@@ -89,7 +93,8 @@
 				   gnutls_*
 				   init_gnutls_session(),
 				   GNUTLS_* */
-#include <gnutls/openpgp.h>     /* gnutls_certificate_set_openpgp_key_file(),
+#include <gnutls/openpgp.h>
+			  /* gnutls_certificate_set_openpgp_key_file(),
 				   GNUTLS_OPENPGP_FMT_BASE64 */
 
 /* GPGME */
@@ -101,17 +106,13 @@
 
 #define BUFFER_SIZE 256
 
-/*
-  #define PATHDIR "/conf/conf.d/mandos"
-*/
-
 #define PATHDIR "/conf/conf.d/mandos"
 #define SECKEY "seckey.txt"
 #define PUBKEY "pubkey.txt"
 
 bool debug = false;
 static const char mandos_protocol_version[] = "1";
-const char *argp_program_version = "mandos-client 1.0";
+const char *argp_program_version = "mandos-client " VERSION;
 const char *argp_program_bug_address = "<mandos@fukt.bsnet.se>";
 
 /* Used for passing in values through the Avahi callback functions */
@@ -132,9 +133,9 @@
  */
 size_t adjustbuffer(char **buffer, size_t buffer_length,
 		  size_t buffer_capacity){
-  if (buffer_length + BUFFER_SIZE > buffer_capacity){
+  if(buffer_length + BUFFER_SIZE > buffer_capacity){
     *buffer = realloc(*buffer, buffer_capacity + BUFFER_SIZE);
-    if (buffer == NULL){
+    if(buffer == NULL){
       return 0;
     }
     buffer_capacity += BUFFER_SIZE;
@@ -153,33 +154,33 @@
   
   
   /*
-   * Helper function to insert pub and seckey to the enigne keyring.
+   * Helper function to insert pub and seckey to the engine keyring.
    */
   bool import_key(const char *filename){
     int fd;
     gpgme_data_t pgp_data;
     
-    fd = TEMP_FAILURE_RETRY(open(filename, O_RDONLY));
+    fd = (int)TEMP_FAILURE_RETRY(open(filename, O_RDONLY));
     if(fd == -1){
       perror("open");
       return false;
     }
     
     rc = gpgme_data_new_from_fd(&pgp_data, fd);
-    if (rc != GPG_ERR_NO_ERROR){
+    if(rc != GPG_ERR_NO_ERROR){
       fprintf(stderr, "bad gpgme_data_new_from_fd: %s: %s\n",
 	      gpgme_strsource(rc), gpgme_strerror(rc));
       return false;
     }
     
     rc = gpgme_op_import(mc->ctx, pgp_data);
-    if (rc != GPG_ERR_NO_ERROR){
+    if(rc != GPG_ERR_NO_ERROR){
       fprintf(stderr, "bad gpgme_op_import: %s: %s\n",
 	      gpgme_strsource(rc), gpgme_strerror(rc));
       return false;
     }
     
-    ret = TEMP_FAILURE_RETRY(close(fd));
+    ret = (int)TEMP_FAILURE_RETRY(close(fd));
     if(ret == -1){
       perror("close");
     }
@@ -187,22 +188,22 @@
     return true;
   }
   
-  if (debug){
+  if(debug){
     fprintf(stderr, "Initialize gpgme\n");
   }
   
   /* Init GPGME */
   gpgme_check_version(NULL);
   rc = gpgme_engine_check_version(GPGME_PROTOCOL_OpenPGP);
-  if (rc != GPG_ERR_NO_ERROR){
+  if(rc != GPG_ERR_NO_ERROR){
     fprintf(stderr, "bad gpgme_engine_check_version: %s: %s\n",
 	    gpgme_strsource(rc), gpgme_strerror(rc));
     return false;
   }
   
     /* Set GPGME home directory for the OpenPGP engine only */
-  rc = gpgme_get_engine_info (&engine_info);
-  if (rc != GPG_ERR_NO_ERROR){
+  rc = gpgme_get_engine_info(&engine_info);
+  if(rc != GPG_ERR_NO_ERROR){
     fprintf(stderr, "bad gpgme_get_engine_info: %s: %s\n",
 	    gpgme_strsource(rc), gpgme_strerror(rc));
     return false;
@@ -222,13 +223,13 @@
   
   /* Create new GPGME "context" */
   rc = gpgme_new(&(mc->ctx));
-  if (rc != GPG_ERR_NO_ERROR){
+  if(rc != GPG_ERR_NO_ERROR){
     fprintf(stderr, "bad gpgme_new: %s: %s\n",
 	    gpgme_strsource(rc), gpgme_strerror(rc));
     return false;
   }
   
-  if (not import_key(pubkey) or not import_key(seckey)){
+  if(not import_key(pubkey) or not import_key(seckey)){
     return false;
   }
   
@@ -239,24 +240,24 @@
  * Decrypt OpenPGP data.
  * Returns -1 on error
  */
-static ssize_t pgp_packet_decrypt (const mandos_context *mc,
-				   const char *cryptotext,
-				   size_t crypto_size,
-				   char **plaintext){
+static ssize_t pgp_packet_decrypt(const mandos_context *mc,
+				  const char *cryptotext,
+				  size_t crypto_size,
+				  char **plaintext){
   gpgme_data_t dh_crypto, dh_plain;
   gpgme_error_t rc;
   ssize_t ret;
   size_t plaintext_capacity = 0;
   ssize_t plaintext_length = 0;
   
-  if (debug){
+  if(debug){
     fprintf(stderr, "Trying to decrypt OpenPGP data\n");
   }
   
   /* Create new GPGME data buffer from memory cryptotext */
   rc = gpgme_data_new_from_mem(&dh_crypto, cryptotext, crypto_size,
 			       0);
-  if (rc != GPG_ERR_NO_ERROR){
+  if(rc != GPG_ERR_NO_ERROR){
     fprintf(stderr, "bad gpgme_data_new_from_mem: %s: %s\n",
 	    gpgme_strsource(rc), gpgme_strerror(rc));
     return -1;
@@ -264,7 +265,7 @@
   
   /* Create new empty GPGME data buffer for the plaintext */
   rc = gpgme_data_new(&dh_plain);
-  if (rc != GPG_ERR_NO_ERROR){
+  if(rc != GPG_ERR_NO_ERROR){
     fprintf(stderr, "bad gpgme_data_new: %s: %s\n",
 	    gpgme_strsource(rc), gpgme_strerror(rc));
     gpgme_data_release(dh_crypto);
@@ -274,14 +275,14 @@
   /* Decrypt data from the cryptotext data buffer to the plaintext
      data buffer */
   rc = gpgme_op_decrypt(mc->ctx, dh_crypto, dh_plain);
-  if (rc != GPG_ERR_NO_ERROR){
+  if(rc != GPG_ERR_NO_ERROR){
     fprintf(stderr, "bad gpgme_op_decrypt: %s: %s\n",
 	    gpgme_strsource(rc), gpgme_strerror(rc));
     plaintext_length = -1;
-    if (debug){
+    if(debug){
       gpgme_decrypt_result_t result;
       result = gpgme_op_decrypt_result(mc->ctx);
-      if (result == NULL){
+      if(result == NULL){
 	fprintf(stderr, "gpgme_op_decrypt_result failed\n");
       } else {
 	fprintf(stderr, "Unsupported algorithm: %s\n",
@@ -314,7 +315,7 @@
   }
   
   /* Seek back to the beginning of the GPGME plaintext data buffer */
-  if (gpgme_data_seek(dh_plain, (off_t) 0, SEEK_SET) == -1){
+  if(gpgme_data_seek(dh_plain, (off_t)0, SEEK_SET) == -1){
     perror("gpgme_data_seek");
     plaintext_length = -1;
     goto decrypt_end;
@@ -325,7 +326,7 @@
     plaintext_capacity = adjustbuffer(plaintext,
 				      (size_t)plaintext_length,
 				      plaintext_capacity);
-    if (plaintext_capacity == 0){
+    if(plaintext_capacity == 0){
 	perror("adjustbuffer");
 	plaintext_length = -1;
 	goto decrypt_end;
@@ -334,7 +335,7 @@
     ret = gpgme_data_read(dh_plain, *plaintext + plaintext_length,
 			  BUFFER_SIZE);
     /* Print the data, if any */
-    if (ret == 0){
+    if(ret == 0){
       /* EOF */
       break;
     }
@@ -364,9 +365,10 @@
   return plaintext_length;
 }
 
-static const char * safer_gnutls_strerror (int value) {
-  const char *ret = gnutls_strerror (value); /* Spurious warning */
-  if (ret == NULL)
+static const char * safer_gnutls_strerror(int value) {
+  const char *ret = gnutls_strerror(value); /* Spurious warning from
+					       -Wunreachable-code */
+  if(ret == NULL)
     ret = "(unknown)";
   return ret;
 }
@@ -387,13 +389,13 @@
   }
   
   ret = gnutls_global_init();
-  if (ret != GNUTLS_E_SUCCESS) {
-    fprintf (stderr, "GnuTLS global_init: %s\n",
-	     safer_gnutls_strerror(ret));
+  if(ret != GNUTLS_E_SUCCESS) {
+    fprintf(stderr, "GnuTLS global_init: %s\n",
+	    safer_gnutls_strerror(ret));
     return -1;
   }
   
-  if (debug){
+  if(debug){
     /* "Use a log level over 10 to enable all debugging options."
      * - GnuTLS manual
      */
@@ -403,11 +405,13 @@
   
   /* OpenPGP credentials */
   gnutls_certificate_allocate_credentials(&mc->cred);
-  if (ret != GNUTLS_E_SUCCESS){
-    fprintf (stderr, "GnuTLS memory error: %s\n", /* Spurious
-						     warning */
-	     safer_gnutls_strerror(ret));
-    gnutls_global_deinit ();
+  if(ret != GNUTLS_E_SUCCESS){
+    fprintf(stderr, "GnuTLS memory error: %s\n", /* Spurious warning
+						  * from
+						  * -Wunreachable-code
+						  */
+	    safer_gnutls_strerror(ret));
+    gnutls_global_deinit();
     return -1;
   }
   
@@ -420,7 +424,7 @@
   ret = gnutls_certificate_set_openpgp_key_file
     (mc->cred, pubkeyfilename, seckeyfilename,
      GNUTLS_OPENPGP_FMT_BASE64);
-  if (ret != GNUTLS_E_SUCCESS) {
+  if(ret != GNUTLS_E_SUCCESS) {
     fprintf(stderr,
 	    "Error[%d] while reading the OpenPGP key pair ('%s',"
 	    " '%s')\n", ret, pubkeyfilename, seckeyfilename);
@@ -431,15 +435,15 @@
   
   /* GnuTLS server initialization */
   ret = gnutls_dh_params_init(&mc->dh_params);
-  if (ret != GNUTLS_E_SUCCESS) {
-    fprintf (stderr, "Error in GnuTLS DH parameter initialization:"
-	     " %s\n", safer_gnutls_strerror(ret));
+  if(ret != GNUTLS_E_SUCCESS) {
+    fprintf(stderr, "Error in GnuTLS DH parameter initialization:"
+	    " %s\n", safer_gnutls_strerror(ret));
     goto globalfail;
   }
   ret = gnutls_dh_params_generate2(mc->dh_params, mc->dh_bits);
-  if (ret != GNUTLS_E_SUCCESS) {
-    fprintf (stderr, "Error in GnuTLS prime generation: %s\n",
-	     safer_gnutls_strerror(ret));
+  if(ret != GNUTLS_E_SUCCESS) {
+    fprintf(stderr, "Error in GnuTLS prime generation: %s\n",
+	    safer_gnutls_strerror(ret));
     goto globalfail;
   }
   
@@ -460,7 +464,7 @@
   int ret;
   /* GnuTLS session creation */
   ret = gnutls_init(session, GNUTLS_SERVER);
-  if (ret != GNUTLS_E_SUCCESS){
+  if(ret != GNUTLS_E_SUCCESS){
     fprintf(stderr, "Error in GnuTLS session initialization: %s\n",
 	    safer_gnutls_strerror(ret));
   }
@@ -468,29 +472,29 @@
   {
     const char *err;
     ret = gnutls_priority_set_direct(*session, mc->priority, &err);
-    if (ret != GNUTLS_E_SUCCESS) {
+    if(ret != GNUTLS_E_SUCCESS) {
       fprintf(stderr, "Syntax error at: %s\n", err);
       fprintf(stderr, "GnuTLS error: %s\n",
 	      safer_gnutls_strerror(ret));
-      gnutls_deinit (*session);
+      gnutls_deinit(*session);
       return -1;
     }
   }
   
   ret = gnutls_credentials_set(*session, GNUTLS_CRD_CERTIFICATE,
 			       mc->cred);
-  if (ret != GNUTLS_E_SUCCESS) {
+  if(ret != GNUTLS_E_SUCCESS) {
     fprintf(stderr, "Error setting GnuTLS credentials: %s\n",
 	    safer_gnutls_strerror(ret));
-    gnutls_deinit (*session);
+    gnutls_deinit(*session);
     return -1;
   }
   
   /* ignore client certificate if any. */
-  gnutls_certificate_server_set_request (*session,
-					 GNUTLS_CERT_IGNORE);
+  gnutls_certificate_server_set_request(*session,
+					GNUTLS_CERT_IGNORE);
   
-  gnutls_dh_set_prime_bits (*session, mc->dh_bits);
+  gnutls_dh_set_prime_bits(*session, mc->dh_bits);
   
   return 0;
 }
@@ -504,6 +508,7 @@
 				      AvahiIfIndex if_index,
 				      mandos_context *mc){
   int ret, tcp_sd;
+  ssize_t sret;
   union { struct sockaddr in; struct sockaddr_in6 in6; } to;
   char *buffer = NULL;
   char *decrypted_buffer;
@@ -515,8 +520,8 @@
   char interface[IF_NAMESIZE];
   gnutls_session_t session;
   
-  ret = init_gnutls_session (mc, &session);
-  if (ret != 0){
+  ret = init_gnutls_session(mc, &session);
+  if(ret != 0){
     return -1;
   }
   
@@ -544,7 +549,7 @@
   /* It would be nice to have a way to detect if we were passed an
      IPv4 address here.   Now we assume an IPv6 address. */
   ret = inet_pton(AF_INET6, ip, &to.in6.sin6_addr);
-  if (ret < 0 ){
+  if(ret < 0 ){
     perror("inet_pton");
     return -1;
   }
@@ -552,7 +557,9 @@
     fprintf(stderr, "Bad address: %s\n", ip);
     return -1;
   }
-  to.in6.sin6_port = htons(port); /* Spurious warning */
+  to.in6.sin6_port = htons(port); /* Spurious warnings from
+				     -Wconversion and
+				     -Wunreachable-code */
   
   to.in6.sin6_scope_id = (uint32_t)if_index;
   
@@ -571,18 +578,18 @@
   }
   
   ret = connect(tcp_sd, &to.in, sizeof(to));
-  if (ret < 0){
+  if(ret < 0){
     perror("connect");
     return -1;
   }
   
   const char *out = mandos_protocol_version;
   written = 0;
-  while (true){
+  while(true){
     size_t out_size = strlen(out);
-    ret = TEMP_FAILURE_RETRY(write(tcp_sd, out + written,
+    ret = (int)TEMP_FAILURE_RETRY(write(tcp_sd, out + written,
 				   out_size - written));
-    if (ret == -1){
+    if(ret == -1){
       perror("write");
       retval = -1;
       goto mandos_end;
@@ -591,7 +598,7 @@
     if(written < out_size){
       continue;
     } else {
-      if (out == mandos_protocol_version){
+      if(out == mandos_protocol_version){
 	written = 0;
 	out = "\r\n";
       } else {
@@ -604,16 +611,16 @@
     fprintf(stderr, "Establishing TLS session with %s\n", ip);
   }
   
-  gnutls_transport_set_ptr (session, (gnutls_transport_ptr_t) tcp_sd);
+  gnutls_transport_set_ptr(session, (gnutls_transport_ptr_t) tcp_sd);
   
   do{
-    ret = gnutls_handshake (session);
+    ret = gnutls_handshake(session);
   } while(ret == GNUTLS_E_AGAIN or ret == GNUTLS_E_INTERRUPTED);
   
-  if (ret != GNUTLS_E_SUCCESS){
+  if(ret != GNUTLS_E_SUCCESS){
     if(debug){
       fprintf(stderr, "*** GnuTLS Handshake failed ***\n");
-      gnutls_perror (ret);
+      gnutls_perror(ret);
     }
     retval = -1;
     goto mandos_end;
@@ -629,29 +636,29 @@
   while(true){
     buffer_capacity = adjustbuffer(&buffer, buffer_length,
 				   buffer_capacity);
-    if (buffer_capacity == 0){
+    if(buffer_capacity == 0){
       perror("adjustbuffer");
       retval = -1;
       goto mandos_end;
     }
     
-    ret = gnutls_record_recv(session, buffer+buffer_length,
-			     BUFFER_SIZE);
-    if (ret == 0){
+    sret = gnutls_record_recv(session, buffer+buffer_length,
+			      BUFFER_SIZE);
+    if(sret == 0){
       break;
     }
-    if (ret < 0){
-      switch(ret){
+    if(sret < 0){
+      switch(sret){
       case GNUTLS_E_INTERRUPTED:
       case GNUTLS_E_AGAIN:
 	break;
       case GNUTLS_E_REHANDSHAKE:
 	do{
-	  ret = gnutls_handshake (session);
+	  ret = gnutls_handshake(session);
 	} while(ret == GNUTLS_E_AGAIN or ret == GNUTLS_E_INTERRUPTED);
-	if (ret < 0){
+	if(ret < 0){
 	  fprintf(stderr, "*** GnuTLS Re-handshake failed ***\n");
-	  gnutls_perror (ret);
+	  gnutls_perror(ret);
 	  retval = -1;
 	  goto mandos_end;
 	}
@@ -660,11 +667,11 @@
 	fprintf(stderr, "Unknown error while reading data from"
 		" encrypted session with Mandos server\n");
 	retval = -1;
-	gnutls_bye (session, GNUTLS_SHUT_RDWR);
+	gnutls_bye(session, GNUTLS_SHUT_RDWR);
 	goto mandos_end;
       }
     } else {
-      buffer_length += (size_t) ret;
+      buffer_length += (size_t) sret;
     }
   }
   
@@ -672,18 +679,18 @@
     fprintf(stderr, "Closing TLS session\n");
   }
   
-  gnutls_bye (session, GNUTLS_SHUT_RDWR);
+  gnutls_bye(session, GNUTLS_SHUT_RDWR);
   
-  if (buffer_length > 0){
+  if(buffer_length > 0){
     decrypted_buffer_size = pgp_packet_decrypt(mc, buffer,
 					       buffer_length,
 					       &decrypted_buffer);
-    if (decrypted_buffer_size >= 0){
+    if(decrypted_buffer_size >= 0){
       written = 0;
       while(written < (size_t) decrypted_buffer_size){
-	ret = (int)fwrite (decrypted_buffer + written, 1,
-			   (size_t)decrypted_buffer_size - written,
-			   stdout);
+	ret = (int)fwrite(decrypted_buffer + written, 1,
+			  (size_t)decrypted_buffer_size - written,
+			  stdout);
 	if(ret == 0 and ferror(stdout)){
 	  if(debug){
 	    fprintf(stderr, "Error writing encrypted data: %s\n",
@@ -706,11 +713,11 @@
   
  mandos_end:
   free(buffer);
-  ret = TEMP_FAILURE_RETRY(close(tcp_sd));
+  ret = (int)TEMP_FAILURE_RETRY(close(tcp_sd));
   if(ret == -1){
     perror("close");
   }
-  gnutls_deinit (session);
+  gnutls_deinit(session);
   return retval;
 }
 
@@ -734,7 +741,7 @@
   /* Called whenever a service has been resolved successfully or
      timed out */
   
-  switch (event) {
+  switch(event) {
   default:
   case AVAHI_RESOLVER_FAILURE:
     fprintf(stderr, "(Avahi Resolver) Failed to resolve service '%s'"
@@ -748,11 +755,11 @@
       avahi_address_snprint(ip, sizeof(ip), address);
       if(debug){
 	fprintf(stderr, "Mandos server \"%s\" found on %s (%s, %"
-		PRIu16 ") on port %d\n", name, host_name, ip,
-		interface, port);
+		PRIdMAX ") on port %" PRIu16 "\n", name, host_name,
+		ip, (intmax_t)interface, port);
       }
       int ret = start_mandos_communication(ip, port, interface, mc);
-      if (ret == 0){
+      if(ret == 0){
 	avahi_simple_poll_quit(mc->simple_poll);
       }
     }
@@ -776,7 +783,7 @@
   /* Called whenever a new services becomes available on the LAN or
      is removed from the LAN */
   
-  switch (event) {
+  switch(event) {
   default:
   case AVAHI_BROWSER_FAILURE:
     
@@ -791,7 +798,7 @@
        the callback function is called the Avahi server will free the
        resolver for us. */
     
-    if (!(avahi_s_service_resolver_new(mc->server, interface,
+    if(!(avahi_s_service_resolver_new(mc->server, interface,
 				       protocol, name, type, domain,
 				       AVAHI_PROTO_INET6, 0,
 				       resolve_callback, mc)))
@@ -815,6 +822,8 @@
     AvahiSServiceBrowser *sb = NULL;
     int error;
     int ret;
+    intmax_t tmpmax;
+    int numchars;
     int exitcode = EXIT_SUCCESS;
     const char *interface = "eth0";
     struct ifreq network;
@@ -823,6 +832,7 @@
     gid_t gid;
     char *connect_to = NULL;
     char tempdir[] = "/tmp/mandosXXXXXX";
+    bool tempdir_created = false;
     AvahiIfIndex if_index = AVAHI_IF_UNSPEC;
     const char *seckey = PATHDIR "/" SECKEY;
     const char *pubkey = PATHDIR "/" PUBKEY;
@@ -830,8 +840,8 @@
     mandos_context mc = { .simple_poll = NULL, .server = NULL,
 			  .dh_bits = 1024, .priority = "SECURE256"
 			  ":!CTYPE-X.509:+CTYPE-OPENPGP" };
-    bool gnutls_initalized = false;
-    bool gpgme_initalized = false;
+    bool gnutls_initialized = false;
+    bool gpgme_initialized = false;
     
     {
       struct argp_option options[] = {
@@ -866,11 +876,9 @@
 	{ .name = NULL }
       };
       
-      error_t parse_opt (int key, char *arg,
-			 struct argp_state *state) {
-	/* Get the INPUT argument from `argp_parse', which we know is
-	   a pointer to our plugin list pointer. */
-	switch (key) {
+      error_t parse_opt(int key, char *arg,
+			struct argp_state *state) {
+	switch(key) {
 	case 128:		/* --debug */
 	  debug = true;
 	  break;
@@ -887,18 +895,19 @@
 	  pubkey = arg;
 	  break;
 	case 129:		/* --dh-bits */
-	  errno = 0;
-	  mc.dh_bits = (unsigned int) strtol(arg, NULL, 10);
-	  if (errno){
-	    perror("strtol");
+	  ret = sscanf(arg, "%" SCNdMAX "%n", &tmpmax, &numchars);
+	  if(ret < 1 or tmpmax != (typeof(mc.dh_bits))tmpmax
+	     or arg[numchars] != '\0'){
+	    fprintf(stderr, "Bad number of DH bits\n");
 	    exit(EXIT_FAILURE);
 	  }
+	  mc.dh_bits = (typeof(mc.dh_bits))tmpmax;
 	  break;
 	case 130:		/* --priority */
 	  mc.priority = arg;
 	  break;
 	case ARGP_KEY_ARG:
-	  argp_usage (state);
+	  argp_usage(state);
 	case ARGP_KEY_END:
 	  break;
 	default:
@@ -911,8 +920,8 @@
 			   .args_doc = "",
 			   .doc = "Mandos client -- Get and decrypt"
 			   " passwords from a Mandos server" };
-      ret = argp_parse (&argp, argc, argv, 0, 0, NULL);
-      if (ret == ARGP_ERR_UNKNOWN){
+      ret = argp_parse(&argp, argc, argv, 0, 0, NULL);
+      if(ret == ARGP_ERR_UNKNOWN){
 	fprintf(stderr, "Unknown error while parsing arguments\n");
 	exitcode = EXIT_FAILURE;
 	goto end;
@@ -943,7 +952,7 @@
 	  goto end;
 	}
       }
-      ret = TEMP_FAILURE_RETRY(close(sd));
+      ret = (int)TEMP_FAILURE_RETRY(close(sd));
       if(ret == -1){
 	perror("close");
       }
@@ -953,42 +962,43 @@
     gid = getgid();
     
     ret = setuid(uid);
-    if (ret == -1){
+    if(ret == -1){
       perror("setuid");
     }
     
     setgid(gid);
-    if (ret == -1){
+    if(ret == -1){
       perror("setgid");
     }
     
     ret = init_gnutls_global(&mc, pubkey, seckey);
-    if (ret == -1){
+    if(ret == -1){
       fprintf(stderr, "init_gnutls_global failed\n");
       exitcode = EXIT_FAILURE;
       goto end;
     } else {
-      gnutls_initalized = true;
+      gnutls_initialized = true;
     }
     
     if(mkdtemp(tempdir) == NULL){
       perror("mkdtemp");
-      tempdir[0] = '\0';
       goto end;
     }
+    tempdir_created = true;
     
     if(not init_gpgme(&mc, pubkey, seckey, tempdir)){
-      fprintf(stderr, "gpgme_initalized failed\n");
+      fprintf(stderr, "init_gpgme failed\n");
       exitcode = EXIT_FAILURE;
       goto end;
     } else {
-      gpgme_initalized = true;
+      gpgme_initialized = true;
     }
     
     if_index = (AvahiIfIndex) if_nametoindex(interface);
     if(if_index == 0){
       fprintf(stderr, "No such interface: \"%s\"\n", interface);
-      exit(EXIT_FAILURE);
+      exitcode = EXIT_FAILURE;
+      goto end;
     }
     
     if(connect_to != NULL){
@@ -1000,13 +1010,15 @@
 	exitcode = EXIT_FAILURE;
 	goto end;
       }
-      errno = 0;
-      uint16_t port = (uint16_t) strtol(address+1, NULL, 10);
-      if(errno){
-	perror("Bad port number");
+      uint16_t port;
+      ret = sscanf(address+1, "%" SCNdMAX "%n", &tmpmax, &numchars);
+      if(ret < 1 or tmpmax != (uint16_t)tmpmax
+	 or address[numchars+1] != '\0'){
+	fprintf(stderr, "Bad port number\n");
 	exitcode = EXIT_FAILURE;
 	goto end;
       }
+      port = (uint16_t)tmpmax;
       *address = '\0';
       address = connect_to;
       ret = start_mandos_communication(address, port, if_index, &mc);
@@ -1018,7 +1030,7 @@
       goto end;
     }
     
-    if (not debug){
+    if(not debug){
       avahi_set_log_function(empty_log);
     }
     
@@ -1027,7 +1039,7 @@
     
     /* Allocate main Avahi loop object */
     mc.simple_poll = avahi_simple_poll_new();
-    if (mc.simple_poll == NULL) {
+    if(mc.simple_poll == NULL) {
         fprintf(stderr, "Avahi: Failed to create simple poll"
 		" object.\n");
 	exitcode = EXIT_FAILURE;
@@ -1053,7 +1065,7 @@
     }
     
     /* Check if creating the Avahi server object succeeded */
-    if (mc.server == NULL) {
+    if(mc.server == NULL) {
         fprintf(stderr, "Failed to create Avahi server: %s\n",
 		avahi_strerror(error));
 	exitcode = EXIT_FAILURE;
@@ -1065,7 +1077,7 @@
 				     AVAHI_PROTO_INET6,
 				     "_mandos._tcp", NULL, 0,
 				     browse_callback, &mc);
-    if (sb == NULL) {
+    if(sb == NULL) {
         fprintf(stderr, "Failed to create service browser: %s\n",
 		avahi_strerror(avahi_server_errno(mc.server)));
 	exitcode = EXIT_FAILURE;
@@ -1074,7 +1086,7 @@
     
     /* Run the main loop */
     
-    if (debug){
+    if(debug){
       fprintf(stderr, "Starting Avahi loop search\n");
     }
     
@@ -1082,66 +1094,73 @@
     
  end:
     
-    if (debug){
+    if(debug){
       fprintf(stderr, "%s exiting\n", argv[0]);
     }
     
     /* Cleanup things */
-    if (sb != NULL)
+    if(sb != NULL)
         avahi_s_service_browser_free(sb);
     
-    if (mc.server != NULL)
+    if(mc.server != NULL)
         avahi_server_free(mc.server);
     
-    if (mc.simple_poll != NULL)
+    if(mc.simple_poll != NULL)
         avahi_simple_poll_free(mc.simple_poll);
     
-    if (gnutls_initalized){
+    if(gnutls_initialized){
       gnutls_certificate_free_credentials(mc.cred);
-      gnutls_global_deinit ();
+      gnutls_global_deinit();
       gnutls_dh_params_deinit(mc.dh_params);
     }
     
-    if(gpgme_initalized){
+    if(gpgme_initialized){
       gpgme_release(mc.ctx);
     }
     
     /* Removes the temp directory used by GPGME */
-    if(tempdir[0] != '\0'){
+    if(tempdir_created){
       DIR *d;
       struct dirent *direntry;
       d = opendir(tempdir);
       if(d == NULL){
-	perror("opendir");
+	if(errno != ENOENT){
+	  perror("opendir");
+	}
       } else {
 	while(true){
 	  direntry = readdir(d);
 	  if(direntry == NULL){
 	    break;
 	  }
-	  if (direntry->d_type == DT_REG){
-	    char *fullname = NULL;
-	    ret = asprintf(&fullname, "%s/%s", tempdir,
-			   direntry->d_name);
-	    if(ret < 0){
-	      perror("asprintf");
-	      continue;
-	    }
-	    ret = unlink(fullname);
-	    if(ret == -1){
-	      fprintf(stderr, "unlink(\"%s\"): %s",
-		      fullname, strerror(errno));
-	    }
-	    free(fullname);
-	  }
+	  /* Skip "." and ".." */
+	  if(direntry->d_name[0] == '.'
+	     and (direntry->d_name[1] == '\0'
+		  or (direntry->d_name[1] == '.'
+		      and direntry->d_name[2] == '\0'))){
+	    continue;
+	  }
+	  char *fullname = NULL;
+	  ret = asprintf(&fullname, "%s/%s", tempdir,
+			 direntry->d_name);
+	  if(ret < 0){
+	    perror("asprintf");
+	    continue;
+	  }
+	  ret = remove(fullname);
+	  if(ret == -1){
+	    fprintf(stderr, "remove(\"%s\"): %s\n", fullname,
+		    strerror(errno));
+	  }
+	  free(fullname);
 	}
 	closedir(d);
       }
       ret = rmdir(tempdir);
-      if(ret == -1){
+      if(ret == -1 and errno != ENOENT){
 	perror("rmdir");
       }
     }
-	  
+    
     return exitcode;
 }

=== modified file 'plugins.d/mandos-client.xml'
--- plugins.d/mandos-client.xml	2008-09-12 19:12:40 +0000
+++ plugins.d/mandos-client.xml	2009-01-24 15:26:43 +0000
@@ -1,17 +1,18 @@
 <?xml version="1.0" encoding="UTF-8"?>
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
-<!ENTITY VERSION "1.0">
 <!ENTITY COMMANDNAME "mandos-client">
-<!ENTITY TIMESTAMP "2008-09-12">
+<!ENTITY TIMESTAMP "2009-01-24">
+<!ENTITY % common SYSTEM "../common.ent">
+%common;
 ]>
 
 <refentry xmlns:xi="http://www.w3.org/2001/XInclude">
   <refentryinfo>
     <title>Mandos Manual</title>
-    <!-- Nwalsh’s docbook scripts use this to generate the footer: -->
+    <!-- NWalsh’s docbook scripts use this to generate the footer: -->
     <productname>Mandos</productname>
-    <productnumber>&VERSION;</productnumber>
+    <productnumber>&version;</productnumber>
     <date>&TIMESTAMP;</date>
     <authorgroup>
       <author>
@@ -31,6 +32,7 @@
     </authorgroup>
     <copyright>
       <year>2008</year>
+      <year>2009</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>
@@ -120,11 +122,16 @@
       <command>&COMMANDNAME;</command> is a client program that
       communicates with <citerefentry><refentrytitle
       >mandos</refentrytitle><manvolnum>8</manvolnum></citerefentry>
-      to get a password.  It uses IPv6 link-local addresses to get
-      network connectivity, Zeroconf to find servers, and TLS with an
-      OpenPGP key to ensure authenticity and confidentiality.  It
-      keeps running, trying all servers on the network, until it
-      receives a satisfactory reply or a TERM signal is received.
+      to get a password.  In slightly more detail, this client program
+      brings up a network interface, uses the interface’s IPv6
+      link-local address to get network connectivity, uses Zeroconf to
+      find servers on the local network, and communicates with servers
+      using TLS with an OpenPGP key to ensure authenticity and
+      confidentiality.  This client program keeps running, trying all
+      servers on the network, until it receives a satisfactory reply
+      or a TERM signal is received.  If no servers are found, or after
+      all servers have been tried, it waits indefinitely for new
+      servers to appear.
     </para>
     <para>
       This program is not meant to be run directly; it is really meant
@@ -199,6 +206,15 @@
 	    specifies the interface to use to connect to the address
 	    given.
 	  </para>
+	  <para>
+	    Note that since this program will normally run in the
+	    initial RAM disk environment, the interface must be an
+	    interface which exists at that stage.  Thus, the interface
+	    can not be a pseudo-interface such as <quote>br0</quote>
+	    or <quote>tun0</quote>; such interfaces will not exist
+	    until much later in the boot process, and can not be used
+	    by this program.
+	  </para>
 	</listitem>
       </varlistentry>
       
@@ -343,7 +359,7 @@
     </para>
   </refsect1>
   
-  <refsect1 id="file">
+  <refsect1 id="files">
     <title>FILES</title>
     <variablelist>
       <varlistentry>
@@ -449,11 +465,11 @@
       The only remaining weak point is that someone with physical
       access to the client hard drive might turn off the client
       computer, read the OpenPGP keys directly from the hard drive,
-      and communicate with the server.  The defense against this is
-      that the server is supposed to notice the client disappearing
-      and will stop giving out the encrypted data.  Therefore, it is
-      important to set the timeout and checker interval values tightly
-      on the server.  See <citerefentry><refentrytitle
+      and communicate with the server.  To safeguard against this, the
+      server is supposed to notice the client disappearing and stop
+      giving out the encrypted data.  Therefore, it is important to
+      set the timeout and checker interval values tightly on the
+      server.  See <citerefentry><refentrytitle
       >mandos</refentrytitle><manvolnum>8</manvolnum></citerefentry>.
     </para>
     <para>

=== modified file 'plugins.d/password-prompt.c'
--- plugins.d/password-prompt.c	2008-09-07 01:44:44 +0000
+++ plugins.d/password-prompt.c	2009-01-13 04:35:19 +0000
@@ -1,8 +1,9 @@
 /*  -*- coding: utf-8 -*- */
 /*
- * Passprompt - Read a password from the terminal and print it
- *
- * Copyright © 2007-2008 Teddy Hogeborn & Björn Påhlsson
+ * Password-prompt - Read a password from the terminal and print it
+ * 
+ * Copyright © 2008,2009 Teddy Hogeborn
+ * Copyright © 2008,2009 Björn Påhlsson
  * 
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -52,7 +53,7 @@
 
 volatile bool quit_now = false;
 bool debug = false;
-const char *argp_program_version = "password-prompt 1.0";
+const char *argp_program_version = "password-prompt " VERSION;
 const char *argp_program_bug_address = "<mandos@fukt.bsnet.se>";
 
 static void termination_handler(__attribute__((unused))int signum){
@@ -78,10 +79,8 @@
 	.doc = "Debug mode", .group = 3 },
       { .name = NULL }
     };
-  
+    
     error_t parse_opt (int key, char *arg, struct argp_state *state) {
-      /* Get the INPUT argument from `argp_parse', which we know is a
-	 pointer to our plugin list pointer. */
       switch (key) {
       case 'p':
 	prefix = arg;
@@ -90,7 +89,7 @@
 	debug = true;
 	break;
       case ARGP_KEY_ARG:
-	argp_usage (state);
+	argp_usage(state);
 	break;
       case ARGP_KEY_END:
 	break;
@@ -99,26 +98,26 @@
       }
       return 0;
     }
-  
+    
     struct argp argp = { .options = options, .parser = parse_opt,
 			 .args_doc = "",
 			 .doc = "Mandos password-prompt -- Read and"
 			 " output a password" };
-    ret = argp_parse (&argp, argc, argv, 0, 0, NULL);
-    if (ret == ARGP_ERR_UNKNOWN){
+    ret = argp_parse(&argp, argc, argv, 0, 0, NULL);
+    if(ret == ARGP_ERR_UNKNOWN){
       fprintf(stderr, "Unknown error while parsing arguments\n");
       return EXIT_FAILURE;
     }
   }
-    
-  if (debug){
+  
+  if(debug){
     fprintf(stderr, "Starting %s\n", argv[0]);
   }
-  if (debug){
+  if(debug){
     fprintf(stderr, "Storing current terminal attributes\n");
   }
   
-  if (tcgetattr(STDIN_FILENO, &t_old) != 0){
+  if(tcgetattr(STDIN_FILENO, &t_old) != 0){
     perror("tcgetattr");
     return EXIT_FAILURE;
   }
@@ -132,7 +131,7 @@
     perror("sigaction");
     return EXIT_FAILURE;
   }
-  if (old_action.sa_handler != SIG_IGN){
+  if(old_action.sa_handler != SIG_IGN){
     ret = sigaction(SIGINT, &new_action, NULL);
     if(ret == -1){
       perror("sigaction");
@@ -144,7 +143,7 @@
     perror("sigaction");
     return EXIT_FAILURE;
   }
-  if (old_action.sa_handler != SIG_IGN){
+  if(old_action.sa_handler != SIG_IGN){
     ret = sigaction(SIGHUP, &new_action, NULL);
     if(ret == -1){
       perror("sigaction");
@@ -156,7 +155,7 @@
     perror("sigaction");
     return EXIT_FAILURE;
   }
-  if (old_action.sa_handler != SIG_IGN){
+  if(old_action.sa_handler != SIG_IGN){
     ret = sigaction(SIGTERM, &new_action, NULL);
     if(ret == -1){
       perror("sigaction");
@@ -165,22 +164,22 @@
   }
   
   
-  if (debug){
+  if(debug){
     fprintf(stderr, "Removing echo flag from terminal attributes\n");
   }
   
   t_new = t_old;
   t_new.c_lflag &= ~ECHO;
-  if (tcsetattr(STDIN_FILENO, TCSAFLUSH, &t_new) != 0){
+  if(tcsetattr(STDIN_FILENO, TCSAFLUSH, &t_new) != 0){
     perror("tcsetattr-echo");
     return EXIT_FAILURE;
   }
 
-  if (debug){
+  if(debug){
     fprintf(stderr, "Waiting for input from stdin \n");
   }
   while(true){
-    if (quit_now){
+    if(quit_now){
       if(debug){
 	fprintf(stderr, "Interrupted by signal, exiting.\n");
       }
@@ -212,7 +211,7 @@
       }
     }
     ret = getline(&buffer, &n, stdin);
-    if (ret > 0){
+    if(ret > 0){
       status = EXIT_SUCCESS;
       /* Make n = data size instead of allocated buffer size */
       n = (size_t)ret;
@@ -233,8 +232,8 @@
       }
       break;
     }
-    if (ret < 0){
-      if (errno != EINTR and not feof(stdin)){
+    if(ret < 0){
+      if(errno != EINTR and not feof(stdin)){
 	perror("getline");
 	status = EXIT_FAILURE;
 	break;
@@ -249,17 +248,17 @@
       fprintf(stderr, "getline() returned 0, retrying.\n");
     }
   }
-
+  
   free(buffer);
   
-  if (debug){
+  if(debug){
     fprintf(stderr, "Restoring terminal attributes\n");
   }
-  if (tcsetattr(STDIN_FILENO, TCSAFLUSH, &t_old) != 0){
+  if(tcsetattr(STDIN_FILENO, TCSAFLUSH, &t_old) != 0){
     perror("tcsetattr+echo");
   }
   
-  if (debug){
+  if(debug){
     fprintf(stderr, "%s is exiting with status %d\n", argv[0],
 	    status);
   }

=== modified file 'plugins.d/password-prompt.xml'
--- plugins.d/password-prompt.xml	2008-09-06 16:31:49 +0000
+++ plugins.d/password-prompt.xml	2009-01-04 21:54:55 +0000
@@ -1,9 +1,10 @@
 <?xml version="1.0" encoding="UTF-8"?>
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
-<!ENTITY VERSION "1.0">
 <!ENTITY COMMANDNAME "password-prompt">
-<!ENTITY TIMESTAMP "2008-09-06">
+<!ENTITY TIMESTAMP "2009-01-04">
+<!ENTITY % common SYSTEM "../common.ent">
+%common;
 ]>
 
 <refentry xmlns:xi="http://www.w3.org/2001/XInclude">
@@ -11,7 +12,7 @@
     <title>Mandos Manual</title>
     <!-- NWalsh’s docbook scripts use this to generate the footer: -->
     <productname>Mandos</productname>
-    <productnumber>&VERSION;</productnumber>
+    <productnumber>&version;</productnumber>
     <date>&TIMESTAMP;</date>
     <authorgroup>
       <author>
@@ -31,6 +32,7 @@
     </authorgroup>
     <copyright>
       <year>2008</year>
+      <year>2009</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>
@@ -83,12 +85,14 @@
     <title>DESCRIPTION</title>
     <para>
       All <command>&COMMANDNAME;</command> does is prompt for a
-      password and output any given password to standard output.  This
-      is not very useful on its own.  This program is really meant to
-      run as a plugin in the <application>Mandos</application>
-      client-side system, where it is used as a fallback and
-      alternative to retrieving passwords from a <application
-      >Mandos</application> server.
+      password and output any given password to standard output.
+    </para>
+    <para>
+      This program is not very useful on its own.  This program is
+      really meant to run as a plugin in the <application
+      >Mandos</application> client-side system, where it is used as a
+      fallback and alternative to retrieving passwords from a
+      <application >Mandos</application> server.
     </para>
     <para>
       This program is little more than a <citerefentry><refentrytitle

=== added file 'plugins.d/splashy.c'
--- plugins.d/splashy.c	1970-01-01 00:00:00 +0000
+++ plugins.d/splashy.c	2009-01-14 14:20:17 +0000
@@ -0,0 +1,304 @@
+/*  -*- coding: utf-8 -*- */
+/*
+ * Splashy - Read a password from splashy and output it
+ * 
+ * Copyright © 2008,2009 Teddy Hogeborn
+ * Copyright © 2008,2009 Björn Påhlsson
+ * 
+ * This program is free software: you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see
+ * <http://www.gnu.org/licenses/>.
+ * 
+ * Contact the authors at <https://www.fukt.bsnet.se/~belorn/> and
+ * <https://www.fukt.bsnet.se/~teddy/>.
+ */
+
+#define _GNU_SOURCE		/* asprintf() */
+#include <signal.h>		/* sig_atomic_t, struct sigaction,
+				   sigemptyset(), sigaddset(), SIGINT,
+				   SIGHUP, SIGTERM, sigaction,
+				   SIG_IGN, kill(), SIGKILL */
+#include <stddef.h>		/* NULL */
+#include <stdlib.h>		/* getenv() */
+#include <stdio.h>		/* asprintf(), perror(), sscanf() */
+#include <stdlib.h>		/* EXIT_FAILURE, free(),
+				   EXIT_SUCCESS */
+#include <sys/types.h>		/* pid_t, DIR, struct dirent,
+				   ssize_t */
+#include <dirent.h>		/* opendir(), readdir(), closedir() */
+#include <inttypes.h>		/* intmax_t, SCNdMAX */
+#include <sys/stat.h>		/* struct stat, lstat(), S_ISLNK */
+#include <iso646.h>		/* not, or, and */
+#include <unistd.h>		/* readlink(), fork(), execl(),
+				   sleep(), dup2() STDERR_FILENO,
+				   STDOUT_FILENO, _exit() */
+#include <string.h>		/* memcmp() */
+#include <errno.h>		/* errno */
+#include <sys/wait.h>		/* waitpid(), WIFEXITED(),
+				   WEXITSTATUS() */
+
+sig_atomic_t interrupted_by_signal = 0;
+
+static void termination_handler(__attribute__((unused))int signum){
+  interrupted_by_signal = 1;
+}
+
+int main(__attribute__((unused))int argc,
+	 __attribute__((unused))char **argv){
+  int ret = 0;
+  
+  /* Create prompt string */
+  char *prompt = NULL;
+  {
+    const char *const cryptsource = getenv("cryptsource");
+    const char *const crypttarget = getenv("crypttarget");
+    const char *const prompt_start = "getpass "
+      "Enter passphrase to unlock the disk";
+    
+    if(cryptsource == NULL){
+      if(crypttarget == NULL){
+	ret = asprintf(&prompt, "%s: ", prompt_start);
+      } else {
+	ret = asprintf(&prompt, "%s (%s): ", prompt_start,
+		       crypttarget);
+      }
+    } else {
+      if(crypttarget == NULL){
+	ret = asprintf(&prompt, "%s %s: ", prompt_start, cryptsource);
+      } else {
+	ret = asprintf(&prompt, "%s %s (%s): ", prompt_start,
+		       cryptsource, crypttarget);
+      }
+    }
+    if(ret == -1){
+      return EXIT_FAILURE;
+    }
+  }
+  
+  /* Find splashy process */
+  pid_t splashy_pid = 0;
+  {
+    const char splashy_name[] = "/sbin/splashy";
+    DIR *proc_dir = opendir("/proc");
+    if(proc_dir == NULL){
+      free(prompt);
+      perror("opendir");
+      return EXIT_FAILURE;
+    }
+    for(struct dirent *proc_ent = readdir(proc_dir);
+	proc_ent != NULL;
+	proc_ent = readdir(proc_dir)){
+      pid_t pid;
+      {
+	intmax_t tmpmax;
+	int numchars;
+	ret = sscanf(proc_ent->d_name, "%" SCNdMAX "%n", &tmpmax,
+		     &numchars);
+	if(ret < 1 or tmpmax != (pid_t)tmpmax
+	   or proc_ent->d_name[numchars] != '\0'){
+	  /* Not a process */
+	  continue;
+	}
+	pid = (pid_t)tmpmax;
+      }
+      /* Find the executable name by doing readlink() on the
+	 /proc/<pid>/exe link */
+      char exe_target[sizeof(splashy_name)];
+      ssize_t sret;
+      {
+	char *exe_link;
+	ret = asprintf(&exe_link, "/proc/%s/exe", proc_ent->d_name);
+	if(ret == -1){
+	  perror("asprintf");
+	  free(prompt);
+	  closedir(proc_dir);
+	  return EXIT_FAILURE;
+	}
+	
+	/* Check that it refers to a symlink owned by root:root */
+	struct stat exe_stat;
+	ret = lstat(exe_link, &exe_stat);
+	if(ret == -1){
+	  if(errno == ENOENT){
+	    free(exe_link);
+	    continue;
+	  }
+	  perror("lstat");
+	  free(exe_link);
+	  free(prompt);
+	  closedir(proc_dir);
+	  return EXIT_FAILURE;
+	}
+	if(not S_ISLNK(exe_stat.st_mode)
+	   or exe_stat.st_uid != 0
+	   or exe_stat.st_gid != 0){
+	  free(exe_link);
+	  continue;
+	}
+	
+	sret = readlink(exe_link, exe_target, sizeof(exe_target));
+	free(exe_link);
+      }
+      if((sret == ((ssize_t)sizeof(exe_target)-1))
+	 and (memcmp(splashy_name, exe_target,
+		     sizeof(exe_target)-1) == 0)){
+	splashy_pid = pid;
+	break;
+      }
+    }
+    closedir(proc_dir);
+  }
+  if(splashy_pid == 0){
+    free(prompt);
+    return EXIT_FAILURE;
+  }
+  
+  /* Set up the signal handler */
+  {
+    struct sigaction old_action,
+      new_action = { .sa_handler = termination_handler,
+		     .sa_flags = 0 };
+    sigemptyset(&new_action.sa_mask);
+    sigaddset(&new_action.sa_mask, SIGINT);
+    sigaddset(&new_action.sa_mask, SIGHUP);
+    sigaddset(&new_action.sa_mask, SIGTERM);
+    ret = sigaction(SIGINT, NULL, &old_action);
+    if(ret == -1){
+      perror("sigaction");
+      free(prompt);
+      return EXIT_FAILURE;
+    }
+    if(old_action.sa_handler != SIG_IGN){
+      ret = sigaction(SIGINT, &new_action, NULL);
+      if(ret == -1){
+	perror("sigaction");
+	free(prompt);
+	return EXIT_FAILURE;
+      }
+    }
+    ret = sigaction(SIGHUP, NULL, &old_action);
+    if(ret == -1){
+      perror("sigaction");
+      free(prompt);
+      return EXIT_FAILURE;
+    }
+    if(old_action.sa_handler != SIG_IGN){
+      ret = sigaction(SIGHUP, &new_action, NULL);
+      if(ret == -1){
+	perror("sigaction");
+	free(prompt);
+	return EXIT_FAILURE;
+      }
+    }
+    ret = sigaction(SIGTERM, NULL, &old_action);
+    if(ret == -1){
+      perror("sigaction");
+      free(prompt);
+      return EXIT_FAILURE;
+    }
+    if(old_action.sa_handler != SIG_IGN){
+      ret = sigaction(SIGTERM, &new_action, NULL);
+      if(ret == -1){
+	perror("sigaction");
+	free(prompt);
+	return EXIT_FAILURE;
+      }
+    }
+  }
+  
+  /* Fork off the splashy command to prompt for password */
+  pid_t splashy_command_pid = 0;
+  if(not interrupted_by_signal){
+    splashy_command_pid = fork();
+    if(splashy_command_pid == -1){
+      if(not interrupted_by_signal){
+	perror("fork");
+      }
+      return EXIT_FAILURE;
+    }
+    /* Child */
+    if(splashy_command_pid == 0){
+      const char splashy_command[] = "/sbin/splashy_update";
+      ret = execl(splashy_command, splashy_command, prompt,
+		  (char *)NULL);
+      if(not interrupted_by_signal){
+	perror("execl");
+      }
+      free(prompt);
+      _exit(EXIT_FAILURE);
+    }
+  }
+  
+  /* Parent */
+  free(prompt);
+  
+  /* Wait for command to complete */
+  if(not interrupted_by_signal and splashy_command_pid != 0){
+    int status;
+    ret = waitpid(splashy_command_pid, &status, 0);
+    if(ret == -1){
+      if(errno != EINTR){
+	perror("waitpid");
+      }
+      if(errno == ECHILD){
+	splashy_command_pid = 0;
+      }
+    } else {
+      /* The child process has exited */
+      splashy_command_pid = 0;
+      if(not interrupted_by_signal and WIFEXITED(status)
+	 and WEXITSTATUS(status)==0){
+	return EXIT_SUCCESS;
+      }
+    }
+  }
+  kill(splashy_pid, SIGTERM);
+  if(interrupted_by_signal and splashy_command_pid != 0){
+    kill(splashy_command_pid, SIGTERM);
+  }
+  sleep(2);
+  while(kill(splashy_pid, 0) == 0){
+    kill(splashy_pid, SIGKILL);
+    sleep(1);
+  }
+  pid_t new_splashy_pid = fork();
+  if(new_splashy_pid == 0){
+    /* Child; will become new splashy process */
+    
+    /* Make the effective user ID (root) the only user ID instead of
+       the real user ID (_mandos) */
+    ret = setuid(geteuid());
+    if(ret == -1){
+      perror("setuid");
+    }
+    
+    setsid();
+    ret = chdir("/");
+/*     if(fork() != 0){ */
+/*       _exit(EXIT_SUCCESS); */
+/*     } */
+    ret = dup2(STDERR_FILENO, STDOUT_FILENO); /* replace our stdout */
+    if(ret == -1){
+      perror("dup2");
+      _exit(EXIT_FAILURE);
+    }
+    
+    execl("/sbin/splashy", "/sbin/splashy", "boot", (char *)NULL);
+    if(not interrupted_by_signal){
+      perror("execl");
+    }
+    _exit(EXIT_FAILURE);
+  }
+  
+  return EXIT_FAILURE;
+}

=== added file 'plugins.d/splashy.xml'
--- plugins.d/splashy.xml	1970-01-01 00:00:00 +0000
+++ plugins.d/splashy.xml	2009-01-04 21:54:55 +0000
@@ -0,0 +1,283 @@
+<?xml version="1.0" encoding="UTF-8"?>
+<!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
+	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
+<!ENTITY COMMANDNAME "splashy">
+<!ENTITY TIMESTAMP "2009-01-04">
+<!ENTITY % common SYSTEM "../common.ent">
+%common;
+]>
+
+<refentry xmlns:xi="http://www.w3.org/2001/XInclude">
+  <refentryinfo>
+    <title>Mandos Manual</title>
+    <!-- NWalsh’s docbook scripts use this to generate the footer: -->
+    <productname>Mandos</productname>
+    <productnumber>&version;</productnumber>
+    <date>&TIMESTAMP;</date>
+    <authorgroup>
+      <author>
+	<firstname>Björn</firstname>
+	<surname>Påhlsson</surname>
+	<address>
+	  <email>belorn@fukt.bsnet.se</email>
+	</address>
+      </author>
+      <author>
+	<firstname>Teddy</firstname>
+	<surname>Hogeborn</surname>
+	<address>
+	  <email>teddy@fukt.bsnet.se</email>
+	</address>
+      </author>
+    </authorgroup>
+    <copyright>
+      <year>2008</year>
+      <year>2009</year>
+      <holder>Teddy Hogeborn</holder>
+      <holder>Björn Påhlsson</holder>
+    </copyright>
+    <xi:include href="../legalnotice.xml"/>
+  </refentryinfo>
+  
+  <refmeta>
+    <refentrytitle>&COMMANDNAME;</refentrytitle>
+    <manvolnum>8mandos</manvolnum>
+  </refmeta>
+  
+  <refnamediv>
+    <refname><command>&COMMANDNAME;</command></refname>
+    <refpurpose>Mandos plugin to use splashy to get a
+    password.</refpurpose>
+  </refnamediv>
+  
+  <refsynopsisdiv>
+    <cmdsynopsis>
+      <command>&COMMANDNAME;</command>
+    </cmdsynopsis>
+  </refsynopsisdiv>
+  
+  <refsect1 id="description">
+    <title>DESCRIPTION</title>
+    <para>
+      This program prompts for a password using <citerefentry>
+      <refentrytitle>splashy_update</refentrytitle>
+      <manvolnum>8</manvolnum></citerefentry> and outputs any given
+      password to standard output.  If no <citerefentry><refentrytitle
+      >splashy</refentrytitle><manvolnum>8</manvolnum></citerefentry>
+      process can be found, this program will immediately exit with an
+      exit code indicating failure.
+    </para>
+    <para>
+      This program is not very useful on its own.  This program is
+      really meant to run as a plugin in the <application
+      >Mandos</application> client-side system, where it is used as a
+      fallback and alternative to retrieving passwords from a
+      <application >Mandos</application> server.
+    </para>
+    <para>
+      If this program is killed (presumably by
+      <citerefentry><refentrytitle>plugin-runner</refentrytitle>
+      <manvolnum>8mandos</manvolnum></citerefentry> because some other
+      plugin provided the password), it cannot tell <citerefentry>
+      <refentrytitle>splashy</refentrytitle><manvolnum>8</manvolnum>
+      </citerefentry> to abort requesting a password, because
+      <citerefentry><refentrytitle>splashy</refentrytitle>
+      <manvolnum>8</manvolnum></citerefentry> does not support this.
+      Therefore, this program will then <emphasis>kill</emphasis> the
+      running <citerefentry><refentrytitle>splashy</refentrytitle>
+      <manvolnum>8</manvolnum></citerefentry> process and start a
+      <emphasis>new</emphasis> one, using <quote><literal
+      >boot</literal></quote> as the only argument.
+    </para>
+  </refsect1>
+  
+  <refsect1 id="options">
+    <title>OPTIONS</title>
+    <para>
+      This program takes no options.
+    </para>
+  </refsect1>
+  
+  <refsect1 id="exit_status">
+    <title>EXIT STATUS</title>
+    <para>
+      If exit status is 0, the output from the program is the password
+      as it was read.  Otherwise, if exit status is other than 0, the
+      program was interrupted or encountered an error, and any output
+      so far could be corrupt and/or truncated, and should therefore
+      be ignored.
+    </para>
+  </refsect1>
+  
+  <refsect1 id="environment">
+    <title>ENVIRONMENT</title>
+    <variablelist>
+      <varlistentry>
+	<term><envar>cryptsource</envar></term>
+	<term><envar>crypttarget</envar></term>
+	<listitem>
+	  <para>
+	    If set, these environment variables will be assumed to
+	    contain the source device name and the target device
+	    mapper name, respectively, and will be shown as part of
+	    the prompt.
+	</para>
+	<para>
+	  These variables will normally be inherited from
+	  <citerefentry><refentrytitle>plugin-runner</refentrytitle>
+	  <manvolnum>8mandos</manvolnum></citerefentry>, which will
+	  normally have inherited them from
+	  <filename>/scripts/local-top/cryptroot</filename> in the
+	  initial <acronym>RAM</acronym> disk environment, which will
+	  have set them from parsing kernel arguments and
+	  <filename>/conf/conf.d/cryptroot</filename> (also in the
+	  initial RAM disk environment), which in turn will have been
+	  created when the initial RAM disk image was created by
+	  <filename
+	  >/usr/share/initramfs-tools/hooks/cryptroot</filename>, by
+	  extracting the information of the root file system from
+	  <filename >/etc/crypttab</filename>.
+	</para>
+	<para>
+	  This behavior is meant to exactly mirror the behavior of
+	  <command>askpass</command>, the default password prompter.
+	</para>
+	</listitem>
+      </varlistentry>
+    </variablelist>
+  </refsect1>
+  
+  <refsect1 id="files">
+    <title>FILES</title>
+    <variablelist>
+      <varlistentry>
+	<term><filename>/sbin/splashy_update</filename></term>
+	<listitem>
+	  <para>
+	    This is the command run to retrieve a password from
+	    <citerefentry><refentrytitle>splashy</refentrytitle>
+	    <manvolnum>8</manvolnum></citerefentry>.  See
+	    <citerefentry><refentrytitle
+	    >splashy_update</refentrytitle><manvolnum>8</manvolnum>
+	    </citerefentry>.
+	  </para>
+	</listitem>
+      </varlistentry>
+      <varlistentry>
+	<term><filename>/proc</filename></term>
+	<listitem>
+	  <para>
+	    To find the running <citerefentry><refentrytitle
+	    >splashy</refentrytitle><manvolnum>8</manvolnum>
+	    </citerefentry>, this directory will be searched for
+	    numeric entries which will be assumed to be directories.
+	    In all those directories, the <filename>exe</filename>
+	    entry will be used to determine the name of the running
+	    binary and the effective user and group
+	    <abbrev>ID</abbrev> of the process.  See <citerefentry>
+	    <refentrytitle>proc</refentrytitle><manvolnum
+	    >5</manvolnum></citerefentry>.
+	  </para>
+	</listitem>
+      </varlistentry>
+      <varlistentry>
+	<term><filename>/sbin/splashy</filename></term>
+	<listitem>
+	  <para>
+	    This is the name of the binary which will be searched for
+	    in the process list.  See <citerefentry><refentrytitle
+	    >splashy</refentrytitle><manvolnum>8</manvolnum>
+	  </citerefentry>.
+	  </para>
+	</listitem>
+      </varlistentry>
+    </variablelist>
+  </refsect1>
+  
+  <refsect1 id="bugs">
+    <title>BUGS</title>
+    <para>
+      Killing <citerefentry><refentrytitle>splashy</refentrytitle>
+      <manvolnum>8</manvolnum></citerefentry> and starting a new one
+      is ugly, but necessary as long as it does not support aborting a
+      password request.
+    </para>
+  </refsect1>
+  
+  <refsect1 id="example">
+    <title>EXAMPLE</title>
+    <para>
+      Note that normally, this program will not be invoked directly,
+      but instead started by the Mandos <citerefentry><refentrytitle
+      >plugin-runner</refentrytitle><manvolnum>8mandos</manvolnum>
+      </citerefentry>.
+    </para>
+    <informalexample>
+      <para>
+	This program takes no options.
+      </para>
+      <para>
+	<userinput>&COMMANDNAME;</userinput>
+      </para>
+    </informalexample>
+  </refsect1>
+  
+  <refsect1 id="security">
+    <title>SECURITY</title>
+    <para>
+      If this program is killed by a signal, it will kill the process
+      <abbrev>ID</abbrev> which at the start of this program was
+      determined to run <citerefentry><refentrytitle
+      >splashy</refentrytitle><manvolnum>8</manvolnum></citerefentry>
+      as root (see also <xref linkend="files"/>).  There is a very
+      slight risk that, in the time between those events, that process
+      <abbrev>ID</abbrev> was freed and then taken up by another
+      process; the wrong process would then be killed.  Now, this
+      program can only be killed by the user who started it; see
+      <citerefentry><refentrytitle>plugin-runner</refentrytitle>
+      <manvolnum>8mandos</manvolnum></citerefentry>.  This program
+      should therefore be started by a completely separate
+      non-privileged user, and no other programs should be allowed to
+      run as that special user.  This means that it is not recommended
+      to use the user "nobody" to start this program, as other
+      possibly less trusted programs could be running as "nobody", and
+      they would then be able to kill this program, triggering the
+      killing of the process <abbrev>ID</abbrev> which may or may not
+      be <citerefentry><refentrytitle>splashy</refentrytitle>
+      <manvolnum>8</manvolnum></citerefentry>.
+    </para>
+    <para>
+      The only other thing that could be considered worthy of note is
+      this:  This program is meant to be run by <citerefentry>
+      <refentrytitle>plugin-runner</refentrytitle><manvolnum
+      >8mandos</manvolnum></citerefentry>, and will, when run
+      standalone, outside, in a normal environment, immediately output
+      on its standard output any presumably secret password it just
+      received.  Therefore, when running this program standalone
+      (which should never normally be done), take care not to type in
+      any real secret password by force of habit, since it would then
+      immediately be shown as output.
+    </para>
+  </refsect1>
+  
+  <refsect1 id="see_also">
+    <title>SEE ALSO</title>
+    <para>
+      <citerefentry><refentrytitle>crypttab</refentrytitle>
+      <manvolnum>5</manvolnum></citerefentry>,
+      <citerefentry><refentrytitle>plugin-runner</refentrytitle>
+      <manvolnum>8mandos</manvolnum></citerefentry>,
+      <citerefentry><refentrytitle>proc</refentrytitle>
+      <manvolnum>5</manvolnum></citerefentry>,
+      <citerefentry><refentrytitle>splashy</refentrytitle>
+      <manvolnum>8</manvolnum></citerefentry>,
+      <citerefentry><refentrytitle>splashy_update</refentrytitle>
+      <manvolnum>8</manvolnum></citerefentry>
+    </para>
+  </refsect1>
+</refentry>
+<!-- Local Variables: -->
+<!-- time-stamp-start: "<!ENTITY TIMESTAMP [\"']" -->
+<!-- time-stamp-end: "[\"']>" -->
+<!-- time-stamp-format: "%:y-%02m-%02d" -->
+<!-- End: -->

=== removed file 'plugins.d/usplash'
--- plugins.d/usplash	2008-08-14 02:24:59 +0000
+++ plugins.d/usplash	1970-01-01 00:00:00 +0000
@@ -1,42 +0,0 @@
-#!/bin/sh -e
-
-# If not on a tty, then get rid of possibly disrupting stderr output
-if ! tty -s; then
-    exec 2>/dev/null
-fi
-
-test -x /sbin/usplash
-
-usplash="`pidof usplash -o $$`"
-test -n "$usplash"
-
-# We get some variables from cryptsetup:
-# $cryptsource	the device node, like "/dev/sda3"
-# $crypttarget	the device mapper name, like "sda3_crypt".
-
-prompt="Enter passphrase to unlock"
-if [ -n "$crypttarget" ]; then
-    prompt="$prompt the disk $crypttarget"
-fi
-if [ -n "$cryptsource" ]; then
-    prompt="$prompt ($cryptsource)"
-fi
-
-splash_input_password(){
-    test -p /dev/.initramfs/usplash_outfifo || return 1
-    /sbin/usplash_write "INPUTQUIET $1" || return 1
-    cat /dev/.initramfs/usplash_outfifo 2> /dev/null || return 1
-}
-
-# Usplash keeps waiting for input even if some other plugin provided
-# the password, so we must kill it
-trap "kill -TERM $usplash; sleep 2; kill -KILL $usplash; 
-	kill -TERM $$" TERM HUP
-
-password="`splash_input_password \"$prompt: \" password`"
-
-trap - TERM
-
-/sbin/usplash_write "TIMEOUT 15"
-
-echo -n "$password"

=== added file 'plugins.d/usplash.c'
--- plugins.d/usplash.c	1970-01-01 00:00:00 +0000
+++ plugins.d/usplash.c	2009-01-14 14:20:17 +0000
@@ -0,0 +1,531 @@
+/*  -*- coding: utf-8 -*- */
+/*
+ * Usplash - Read a password from usplash and output it
+ * 
+ * Copyright © 2008,2009 Teddy Hogeborn
+ * Copyright © 2008,2009 Björn Påhlsson
+ * 
+ * This program is free software: you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see
+ * <http://www.gnu.org/licenses/>.
+ * 
+ * Contact the authors at <https://www.fukt.bsnet.se/~belorn/> and
+ * <https://www.fukt.bsnet.se/~teddy/>.
+ */
+
+#define _GNU_SOURCE		/* asprintf() */
+#include <signal.h>		/* sig_atomic_t, struct sigaction,
+				   sigemptyset(), sigaddset(), SIGINT,
+				   SIGHUP, SIGTERM, sigaction(),
+				   SIG_IGN, kill(), SIGKILL */
+#include <stdbool.h>		/* bool, false, true */
+#include <fcntl.h>		/* open(), O_WRONLY, O_RDONLY */
+#include <iso646.h>		/* and, or, not*/
+#include <errno.h>		/* errno, EINTR */
+#include <sys/types.h>		/* size_t, ssize_t, pid_t, DIR, struct
+				   dirent */
+#include <stddef.h>		/* NULL */
+#include <string.h>		/* strlen(), memcmp() */
+#include <stdio.h>		/* asprintf(), perror(), sscanf() */
+#include <unistd.h>		/* close(), write(), readlink(),
+				   read(), STDOUT_FILENO, sleep(),
+				   fork(), setuid(), geteuid(),
+				   setsid(), chdir(), dup2(),
+				   STDERR_FILENO, execv() */
+#include <stdlib.h>		/* free(), EXIT_FAILURE, realloc(),
+				   EXIT_SUCCESS, malloc(), _exit() */
+#include <stdlib.h>		/* getenv() */
+#include <dirent.h>		/* opendir(), readdir(), closedir() */
+#include <inttypes.h>		/* intmax_t, SCNdMAX */
+#include <sys/stat.h>		/* struct stat, lstat(), S_ISLNK */
+
+sig_atomic_t interrupted_by_signal = 0;
+
+static void termination_handler(__attribute__((unused))int signum){
+  interrupted_by_signal = 1;
+}
+
+static bool usplash_write(const char *cmd, const char *arg){
+  /* 
+   * usplash_write("TIMEOUT", "15") will write "TIMEOUT 15\0"
+   * usplash_write("PULSATE", NULL) will write "PULSATE\0"
+   * SEE ALSO
+   *         usplash_write(8)
+   */
+  int ret;
+  int fifo_fd;
+  do{
+    fifo_fd = open("/dev/.initramfs/usplash_fifo", O_WRONLY);
+    if(fifo_fd == -1 and (errno != EINTR or interrupted_by_signal)){
+      return false;
+    }
+  }while(fifo_fd == -1);
+  
+  const char *cmd_line;
+  size_t cmd_line_len;
+  char *cmd_line_alloc = NULL;
+  if(arg == NULL){
+    cmd_line = cmd;
+    cmd_line_len = strlen(cmd);
+  }else{
+    do{
+      ret = asprintf(&cmd_line_alloc, "%s %s", cmd, arg);
+      if(ret == -1 and (errno != EINTR or interrupted_by_signal)){
+	int e = errno;
+	close(fifo_fd);
+	errno = e;
+	return false;
+      }
+    }while(ret == -1);
+    cmd_line = cmd_line_alloc;
+    cmd_line_len = (size_t)ret + 1;
+  }
+  
+  size_t written = 0;
+  ssize_t sret = 0;
+  while(not interrupted_by_signal and written < cmd_line_len){
+    sret = write(fifo_fd, cmd_line + written,
+		 cmd_line_len - written);
+    if(sret == -1){
+      if(errno != EINTR or interrupted_by_signal){
+	int e = errno;
+	close(fifo_fd);
+	free(cmd_line_alloc);
+	errno = e;
+	return false;
+      } else {
+	continue;
+      }
+    }
+    written += (size_t)sret;
+  }
+  free(cmd_line_alloc);
+  do{
+    ret = close(fifo_fd);
+    if(ret == -1 and (errno != EINTR or interrupted_by_signal)){
+      return false;
+    }
+  }while(ret == -1);
+  if(interrupted_by_signal){
+    return false;
+  }
+  return true;
+}
+
+int main(__attribute__((unused))int argc,
+	 __attribute__((unused))char **argv){
+  int ret = 0;
+  ssize_t sret;
+  bool an_error_occured = false;
+  
+  /* Create prompt string */
+  char *prompt = NULL;
+  {
+    const char *const cryptsource = getenv("cryptsource");
+    const char *const crypttarget = getenv("crypttarget");
+    const char prompt_start[] = "Enter passphrase to unlock the disk";
+    
+    if(cryptsource == NULL){
+      if(crypttarget == NULL){
+	ret = asprintf(&prompt, "%s: ", prompt_start);
+      } else {
+	ret = asprintf(&prompt, "%s (%s): ", prompt_start,
+		       crypttarget);
+      }
+    } else {
+      if(crypttarget == NULL){
+	ret = asprintf(&prompt, "%s %s: ", prompt_start, cryptsource);
+      } else {
+	ret = asprintf(&prompt, "%s %s (%s): ", prompt_start,
+		       cryptsource, crypttarget);
+      }
+    }
+    if(ret == -1){
+      return EXIT_FAILURE;
+    }
+  }
+  
+  /* Find usplash process */
+  pid_t usplash_pid = 0;
+  char *cmdline = NULL;
+  size_t cmdline_len = 0;
+  const char usplash_name[] = "/sbin/usplash";
+  {
+    DIR *proc_dir = opendir("/proc");
+    if(proc_dir == NULL){
+      free(prompt);
+      perror("opendir");
+      return EXIT_FAILURE;
+    }
+    for(struct dirent *proc_ent = readdir(proc_dir);
+	proc_ent != NULL;
+	proc_ent = readdir(proc_dir)){
+      pid_t pid;
+      {
+	intmax_t tmpmax;
+	int numchars;
+	ret = sscanf(proc_ent->d_name, "%" SCNdMAX "%n", &tmpmax,
+		     &numchars);
+	if(ret < 1 or tmpmax != (pid_t)tmpmax
+	   or proc_ent->d_name[numchars] != '\0'){
+	  /* Not a process */
+	  continue;
+	}
+	pid = (pid_t)tmpmax;
+      }
+      /* Find the executable name by doing readlink() on the
+	 /proc/<pid>/exe link */
+      char exe_target[sizeof(usplash_name)];
+      {
+	/* create file name string */
+	char *exe_link;
+	ret = asprintf(&exe_link, "/proc/%s/exe", proc_ent->d_name);
+	if(ret == -1){
+	  perror("asprintf");
+	  free(prompt);
+	  closedir(proc_dir);
+	  return EXIT_FAILURE;
+	}
+	
+	/* Check that it refers to a symlink owned by root:root */
+	struct stat exe_stat;
+	ret = lstat(exe_link, &exe_stat);
+	if(ret == -1){
+	  if(errno == ENOENT){
+	    free(exe_link);
+	    continue;
+	  }
+	  perror("lstat");
+	  free(exe_link);
+	  free(prompt);
+	  closedir(proc_dir);
+	  return EXIT_FAILURE;
+	}
+	if(not S_ISLNK(exe_stat.st_mode)
+	   or exe_stat.st_uid != 0
+	   or exe_stat.st_gid != 0){
+	  free(exe_link);
+	  continue;
+	}
+	
+	sret = readlink(exe_link, exe_target, sizeof(exe_target));
+	free(exe_link);
+      }
+      if((sret == ((ssize_t)sizeof(exe_target)-1))
+	 and (memcmp(usplash_name, exe_target,
+		     sizeof(exe_target)-1) == 0)){
+	usplash_pid = pid;
+	/* Read and save the command line of usplash in "cmdline" */
+	{
+	  /* Open /proc/<pid>/cmdline  */
+	  int cl_fd;
+	  {
+	    char *cmdline_filename;
+	    ret = asprintf(&cmdline_filename, "/proc/%s/cmdline",
+			   proc_ent->d_name);
+	    if(ret == -1){
+	      perror("asprintf");
+	      free(prompt);
+	      closedir(proc_dir);
+	      return EXIT_FAILURE;
+	    }
+	    cl_fd = open(cmdline_filename, O_RDONLY);
+	    if(cl_fd == -1){
+	      perror("open");
+	      free(cmdline_filename);
+	      free(prompt);
+	      closedir(proc_dir);
+	      return EXIT_FAILURE;
+	    }
+	    free(cmdline_filename);
+	  }
+	  size_t cmdline_allocated = 0;
+	  char *tmp;
+	  const size_t blocksize = 1024;
+	  do{
+	    if(cmdline_len + blocksize > cmdline_allocated){
+	      tmp = realloc(cmdline, cmdline_allocated + blocksize);
+	      if(tmp == NULL){
+		perror("realloc");
+		free(cmdline);
+		free(prompt);
+		closedir(proc_dir);
+		return EXIT_FAILURE;
+	      }
+	      cmdline = tmp;
+	      cmdline_allocated += blocksize;
+	    }
+	    sret = read(cl_fd, cmdline + cmdline_len,
+			cmdline_allocated - cmdline_len);
+	    if(sret == -1){
+	      perror("read");
+	      free(cmdline);
+	      free(prompt);
+	      closedir(proc_dir);
+	      return EXIT_FAILURE;
+	    }
+	    cmdline_len += (size_t)sret;
+	  } while(sret != 0);
+	  close(cl_fd);
+	}
+	break;
+      }
+    }
+    closedir(proc_dir);
+  }
+  if(usplash_pid == 0){
+    free(prompt);
+    return EXIT_FAILURE;
+  }
+  
+  /* Set up the signal handler */
+  {
+    struct sigaction old_action,
+      new_action = { .sa_handler = termination_handler,
+		     .sa_flags = 0 };
+    sigemptyset(&new_action.sa_mask);
+    sigaddset(&new_action.sa_mask, SIGINT);
+    sigaddset(&new_action.sa_mask, SIGHUP);
+    sigaddset(&new_action.sa_mask, SIGTERM);
+    ret = sigaction(SIGINT, NULL, &old_action);
+    if(ret == -1){
+      perror("sigaction");
+      free(prompt);
+      return EXIT_FAILURE;
+    }
+    if(old_action.sa_handler != SIG_IGN){
+      ret = sigaction(SIGINT, &new_action, NULL);
+      if(ret == -1){
+	perror("sigaction");
+	free(prompt);
+	return EXIT_FAILURE;
+      }
+    }
+    ret = sigaction(SIGHUP, NULL, &old_action);
+    if(ret == -1){
+      perror("sigaction");
+      free(prompt);
+      return EXIT_FAILURE;
+    }
+    if(old_action.sa_handler != SIG_IGN){
+      ret = sigaction(SIGHUP, &new_action, NULL);
+      if(ret == -1){
+	perror("sigaction");
+	free(prompt);
+	return EXIT_FAILURE;
+      }
+    }
+    ret = sigaction(SIGTERM, NULL, &old_action);
+    if(ret == -1){
+      perror("sigaction");
+      free(prompt);
+      return EXIT_FAILURE;
+    }
+    if(old_action.sa_handler != SIG_IGN){
+      ret = sigaction(SIGTERM, &new_action, NULL);
+      if(ret == -1){
+	perror("sigaction");
+	free(prompt);
+	return EXIT_FAILURE;
+      }
+    }
+  }
+  
+  /* Write command to FIFO */
+  if(not interrupted_by_signal){
+    if(not usplash_write("TIMEOUT", "0")
+       and (errno != EINTR)){
+      perror("usplash_write");
+      an_error_occured = true;
+    }
+  }
+  if(not interrupted_by_signal and not an_error_occured){
+    if(not usplash_write("INPUTQUIET", prompt)
+       and (errno != EINTR)){
+      perror("usplash_write");
+      an_error_occured = true;
+    }
+  }
+  free(prompt);
+  
+  /* This is not really a loop; while() is used to be able to "break"
+     out of it; those breaks are marked "Big" */
+  while(not interrupted_by_signal and not an_error_occured){
+    char *buf = NULL;
+    size_t buf_len = 0;
+    
+    /* Open FIFO */
+    int fifo_fd;
+    do{
+      fifo_fd = open("/dev/.initramfs/usplash_outfifo", O_RDONLY);
+      if(fifo_fd == -1){
+	if(errno != EINTR){
+	  perror("open");
+	  an_error_occured = true;
+	  break;
+	}
+	if(interrupted_by_signal){
+	  break;
+	}
+      }
+    }while(fifo_fd == -1);
+    if(interrupted_by_signal or an_error_occured){
+      break;			/* Big */
+    }
+    
+    /* Read from FIFO */
+    size_t buf_allocated = 0;
+    const size_t blocksize = 1024;
+    do{
+      if(buf_len + blocksize > buf_allocated){
+	char *tmp = realloc(buf, buf_allocated + blocksize);
+	if(tmp == NULL){
+	  perror("realloc");
+	  an_error_occured = true;
+	  break;
+	}
+	buf = tmp;
+	buf_allocated += blocksize;
+      }
+      do{
+	sret = read(fifo_fd, buf + buf_len, buf_allocated - buf_len);
+	if(sret == -1){
+	  if(errno != EINTR){
+	    perror("read");
+	    an_error_occured = true;
+	    break;
+	  }
+	  if(interrupted_by_signal){
+	    break;
+	  }
+	}
+      }while(sret == -1);
+      if(interrupted_by_signal or an_error_occured){
+	break;
+      }
+      
+      buf_len += (size_t)sret;
+    }while(sret != 0);
+    close(fifo_fd);
+    if(interrupted_by_signal or an_error_occured){
+      break;			/* Big */
+    }
+    
+    if(not usplash_write("TIMEOUT", "15")
+       and (errno != EINTR)){
+	perror("usplash_write");
+	an_error_occured = true;
+    }
+    if(interrupted_by_signal or an_error_occured){
+      break;			/* Big */
+    }
+    
+    /* Print password to stdout */
+    size_t written = 0;
+    while(written < buf_len){
+      do{
+	sret = write(STDOUT_FILENO, buf + written, buf_len - written);
+	if(sret == -1){
+	  if(errno != EINTR){
+	    perror("write");
+	    an_error_occured = true;
+	    break;
+	  }
+	  if(interrupted_by_signal){
+	    break;
+	  }
+	}
+      }while(sret == -1);
+      if(interrupted_by_signal or an_error_occured){
+	break;
+      }
+      written += (size_t)sret;
+    }
+    free(buf);
+    if(not interrupted_by_signal and not an_error_occured){
+      free(cmdline);
+      return EXIT_SUCCESS;
+    }
+    break;			/* Big */
+  }				/* end of non-loop while() */
+  
+  /* If we got here, an error or interrupt must have happened */
+  
+  /* Create argc and argv for new usplash*/
+  int cmdline_argc = 0;
+  char **cmdline_argv = malloc(sizeof(char *));
+  {
+    size_t position = 0;
+    while(position < cmdline_len){
+      char **tmp = realloc(cmdline_argv,
+			   (sizeof(char *)
+			    * (size_t)(cmdline_argc + 2)));
+      if(tmp == NULL){
+	perror("realloc");
+	free(cmdline_argv);
+	return EXIT_FAILURE;
+      }
+      cmdline_argv = tmp;
+      cmdline_argv[cmdline_argc] = cmdline + position;
+      cmdline_argc++;
+      position += strlen(cmdline + position) + 1;
+    }
+    cmdline_argv[cmdline_argc] = NULL;
+  }
+  /* Kill old usplash */
+  kill(usplash_pid, SIGTERM);
+  sleep(2);
+  while(kill(usplash_pid, 0) == 0){
+    kill(usplash_pid, SIGKILL);
+    sleep(1);
+  }
+  pid_t new_usplash_pid = fork();
+  if(new_usplash_pid == 0){
+    /* Child; will become new usplash process */
+    
+    /* Make the effective user ID (root) the only user ID instead of
+       the real user ID (_mandos) */
+    ret = setuid(geteuid());
+    if(ret == -1){
+      perror("setuid");
+    }
+    
+    setsid();
+    ret = chdir("/");
+/*     if(fork() != 0){ */
+/*       _exit(EXIT_SUCCESS); */
+/*     } */
+    ret = dup2(STDERR_FILENO, STDOUT_FILENO); /* replace our stdout */
+    if(ret == -1){
+      perror("dup2");
+      _exit(EXIT_FAILURE);
+    }
+    
+    execv(usplash_name, cmdline_argv);
+    if(not interrupted_by_signal){
+      perror("execv");
+    }
+    free(cmdline);
+    free(cmdline_argv);
+    _exit(EXIT_FAILURE);
+  }
+  free(cmdline);
+  free(cmdline_argv);
+  sleep(2);
+  if(not usplash_write("PULSATE", NULL)
+     and (errno != EINTR)){
+    perror("usplash_write");
+  }
+  
+  return EXIT_FAILURE;
+}

=== added file 'plugins.d/usplash.xml'
--- plugins.d/usplash.xml	1970-01-01 00:00:00 +0000
+++ plugins.d/usplash.xml	2009-01-04 21:54:55 +0000
@@ -0,0 +1,297 @@
+<?xml version="1.0" encoding="UTF-8"?>
+<!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
+	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
+<!ENTITY COMMANDNAME "usplash">
+<!ENTITY TIMESTAMP "2009-01-04">
+<!ENTITY % common SYSTEM "../common.ent">
+%common;
+]>
+
+<refentry xmlns:xi="http://www.w3.org/2001/XInclude">
+  <refentryinfo>
+    <title>Mandos Manual</title>
+    <!-- NWalsh’s docbook scripts use this to generate the footer: -->
+    <productname>Mandos</productname>
+    <productnumber>&version;</productnumber>
+    <date>&TIMESTAMP;</date>
+    <authorgroup>
+      <author>
+	<firstname>Björn</firstname>
+	<surname>Påhlsson</surname>
+	<address>
+	  <email>belorn@fukt.bsnet.se</email>
+	</address>
+      </author>
+      <author>
+	<firstname>Teddy</firstname>
+	<surname>Hogeborn</surname>
+	<address>
+	  <email>teddy@fukt.bsnet.se</email>
+	</address>
+      </author>
+    </authorgroup>
+    <copyright>
+      <year>2008</year>
+      <year>2009</year>
+      <holder>Teddy Hogeborn</holder>
+      <holder>Björn Påhlsson</holder>
+    </copyright>
+    <xi:include href="../legalnotice.xml"/>
+  </refentryinfo>
+  
+  <refmeta>
+    <refentrytitle>&COMMANDNAME;</refentrytitle>
+    <manvolnum>8mandos</manvolnum>
+  </refmeta>
+  
+  <refnamediv>
+    <refname><command>&COMMANDNAME;</command></refname>
+    <refpurpose>Mandos plugin to use usplash to get a
+    password.</refpurpose>
+  </refnamediv>
+  
+  <refsynopsisdiv>
+    <cmdsynopsis>
+      <command>&COMMANDNAME;</command>
+    </cmdsynopsis>
+  </refsynopsisdiv>
+  
+  <refsect1 id="description">
+    <title>DESCRIPTION</title>
+    <para>
+      This program prompts for a password using <citerefentry>
+      <refentrytitle>usplash</refentrytitle><manvolnum>8</manvolnum>
+      </citerefentry> and outputs any given password to standard
+      output.  If no <citerefentry><refentrytitle
+      >usplash</refentrytitle><manvolnum>8</manvolnum></citerefentry>
+      process can be found, this program will immediately exit with an
+      exit code indicating failure.
+    </para>
+    <para>
+      This program is not very useful on its own.  This program is
+      really meant to run as a plugin in the <application
+      >Mandos</application> client-side system, where it is used as a
+      fallback and alternative to retrieving passwords from a
+      <application >Mandos</application> server.
+    </para>
+    <para>
+      If this program is killed (presumably by
+      <citerefentry><refentrytitle>plugin-runner</refentrytitle>
+      <manvolnum>8mandos</manvolnum></citerefentry> because some other
+      plugin provided the password), it cannot tell <citerefentry>
+      <refentrytitle>usplash</refentrytitle><manvolnum>8</manvolnum>
+      </citerefentry> to abort requesting a password, because
+      <citerefentry><refentrytitle>usplash</refentrytitle>
+      <manvolnum>8</manvolnum></citerefentry> does not support this.
+      Therefore, this program will then <emphasis>kill</emphasis> the
+      running <citerefentry><refentrytitle>usplash</refentrytitle>
+      <manvolnum>8</manvolnum></citerefentry> process and start a
+      <emphasis>new</emphasis> one using the same command line
+      arguments as the old one was using.
+    </para>
+  </refsect1>
+  
+  <refsect1 id="options">
+    <title>OPTIONS</title>
+    <para>
+      This program takes no options.
+    </para>
+  </refsect1>
+  
+  <refsect1 id="exit_status">
+    <title>EXIT STATUS</title>
+    <para>
+      If exit status is 0, the output from the program is the password
+      as it was read.  Otherwise, if exit status is other than 0, the
+      program was interrupted or encountered an error, and any output
+      so far could be corrupt and/or truncated, and should therefore
+      be ignored.
+    </para>
+  </refsect1>
+  
+  <refsect1 id="environment">
+    <title>ENVIRONMENT</title>
+    <variablelist>
+      <varlistentry>
+	<term><envar>cryptsource</envar></term>
+	<term><envar>crypttarget</envar></term>
+	<listitem>
+	  <para>
+	    If set, these environment variables will be assumed to
+	    contain the source device name and the target device
+	    mapper name, respectively, and will be shown as part of
+	    the prompt.
+	</para>
+	<para>
+	  These variables will normally be inherited from
+	  <citerefentry><refentrytitle>plugin-runner</refentrytitle>
+	  <manvolnum>8mandos</manvolnum></citerefentry>, which will
+	  normally have inherited them from
+	  <filename>/scripts/local-top/cryptroot</filename> in the
+	  initial <acronym>RAM</acronym> disk environment, which will
+	  have set them from parsing kernel arguments and
+	  <filename>/conf/conf.d/cryptroot</filename> (also in the
+	  initial RAM disk environment), which in turn will have been
+	  created when the initial RAM disk image was created by
+	  <filename
+	  >/usr/share/initramfs-tools/hooks/cryptroot</filename>, by
+	  extracting the information of the root file system from
+	  <filename >/etc/crypttab</filename>.
+	</para>
+	<para>
+	  This behavior is meant to exactly mirror the behavior of
+	  <command>askpass</command>, the default password prompter.
+	</para>
+	</listitem>
+      </varlistentry>
+    </variablelist>
+  </refsect1>
+  
+  <refsect1 id="files">
+    <title>FILES</title>
+    <variablelist>
+      <varlistentry>
+	<term><filename>/dev/.initramfs/usplash_fifo</filename></term>
+	<listitem>
+	  <para>
+	    This is the <acronym>FIFO</acronym> to where this program
+	    will write the commands for <citerefentry><refentrytitle
+	    >usplash</refentrytitle><manvolnum>8</manvolnum>
+	    </citerefentry>.  See <citerefentry><refentrytitle
+	    >fifo</refentrytitle><manvolnum>7</manvolnum>
+	    </citerefentry>.
+	  </para>
+	</listitem>
+      </varlistentry>
+      <varlistentry>
+	<term><filename>/dev/.initramfs/usplash_outfifo</filename></term>
+	<listitem>
+	  <para>
+	    This is the <acronym>FIFO</acronym> where this program
+	    will read the password from <citerefentry><refentrytitle
+	    >usplash</refentrytitle><manvolnum>8</manvolnum>
+	    </citerefentry>.  See <citerefentry><refentrytitle
+	    >fifo</refentrytitle><manvolnum>7</manvolnum>
+	    </citerefentry>.
+	  </para>
+	</listitem>
+      </varlistentry>
+      <varlistentry>
+	<term><filename>/proc</filename></term>
+	<listitem>
+	  <para>
+	    To find the running <citerefentry><refentrytitle
+	    >usplash</refentrytitle><manvolnum>8</manvolnum>
+	    </citerefentry>, this directory will be searched for
+	    numeric entries which will be assumed to be directories.
+	    In all those directories, the <filename>exe</filename> and
+	    <filename>cmdline</filename> entries will be used to
+	    determine the name of the running binary, effective user
+	    and group <abbrev>ID</abbrev>, and the command line
+	    arguments.  See <citerefentry><refentrytitle
+	    >proc</refentrytitle><manvolnum>5</manvolnum>
+	    </citerefentry>.
+	  </para>
+	</listitem>
+      </varlistentry>
+      <varlistentry>
+	<term><filename>/sbin/usplash</filename></term>
+	<listitem>
+	  <para>
+	    This is the name of the binary which will be searched for
+	    in the process list.  See <citerefentry><refentrytitle
+	    >usplash</refentrytitle><manvolnum>8</manvolnum>
+	    </citerefentry>.
+	  </para>
+	</listitem>
+      </varlistentry>
+    </variablelist>
+  </refsect1>
+  
+  <refsect1 id="bugs">
+    <title>BUGS</title>
+    <para>
+      Killing <citerefentry><refentrytitle>usplash</refentrytitle>
+      <manvolnum>8</manvolnum></citerefentry> and starting a new one
+      is ugly, but necessary as long as it does not support aborting a
+      password request.
+    </para>
+  </refsect1>
+  
+  <refsect1 id="example">
+    <title>EXAMPLE</title>
+    <para>
+      Note that normally, this program will not be invoked directly,
+      but instead started by the Mandos <citerefentry><refentrytitle
+      >plugin-runner</refentrytitle><manvolnum>8mandos</manvolnum>
+      </citerefentry>.
+    </para>
+    <informalexample>
+      <para>
+	This program takes no options.
+      </para>
+      <para>
+	<userinput>&COMMANDNAME;</userinput>
+      </para>
+    </informalexample>
+  </refsect1>
+  
+  <refsect1 id="security">
+    <title>SECURITY</title>
+    <para>
+      If this program is killed by a signal, it will kill the process
+      <abbrev>ID</abbrev> which at the start of this program was
+      determined to run <citerefentry><refentrytitle
+      >usplash</refentrytitle><manvolnum>8</manvolnum></citerefentry>
+      as root (see also <xref linkend="files"/>).  There is a very
+      slight risk that, in the time between those events, that process
+      <abbrev>ID</abbrev> was freed and then taken up by another
+      process; the wrong process would then be killed.  Now, this
+      program can only be killed by the user who started it; see
+      <citerefentry><refentrytitle>plugin-runner</refentrytitle>
+      <manvolnum>8mandos</manvolnum></citerefentry>.  This program
+      should therefore be started by a completely separate
+      non-privileged user, and no other programs should be allowed to
+      run as that special user.  This means that it is not recommended
+      to use the user "nobody" to start this program, as other
+      possibly less trusted programs could be running as "nobody", and
+      they would then be able to kill this program, triggering the
+      killing of the process <abbrev>ID</abbrev> which may or may not
+      be <citerefentry><refentrytitle>usplash</refentrytitle>
+      <manvolnum>8</manvolnum></citerefentry>.
+    </para>
+    <para>
+      The only other thing that could be considered worthy of note is
+      this:  This program is meant to be run by <citerefentry>
+      <refentrytitle>plugin-runner</refentrytitle><manvolnum
+      >8mandos</manvolnum></citerefentry>, and will, when run
+      standalone, outside, in a normal environment, immediately output
+      on its standard output any presumably secret password it just
+      received.  Therefore, when running this program standalone
+      (which should never normally be done), take care not to type in
+      any real secret password by force of habit, since it would then
+      immediately be shown as output.
+    </para>
+  </refsect1>
+  
+  <refsect1 id="see_also">
+    <title>SEE ALSO</title>
+    <para>
+      <citerefentry><refentrytitle>crypttab</refentrytitle>
+      <manvolnum>5</manvolnum></citerefentry>,
+      <citerefentry><refentrytitle>fifo</refentrytitle>
+      <manvolnum>7</manvolnum></citerefentry>,
+      <citerefentry><refentrytitle>plugin-runner</refentrytitle>
+      <manvolnum>8mandos</manvolnum></citerefentry>,
+      <citerefentry><refentrytitle>proc</refentrytitle>
+      <manvolnum>5</manvolnum></citerefentry>,
+      <citerefentry><refentrytitle>usplash</refentrytitle>
+      <manvolnum>8</manvolnum></citerefentry>
+    </para>
+  </refsect1>
+</refentry>
+<!-- Local Variables: -->
+<!-- time-stamp-start: "<!ENTITY TIMESTAMP [\"']" -->
+<!-- time-stamp-end: "[\"']>" -->
+<!-- time-stamp-format: "%:y-%02m-%02d" -->
+<!-- End: -->

